package Metrics;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import communication.KillPacket;
import communication.SwarmCommunication;

import simulation.SimulatedRobot;
import simulation.SimulationParameters;
import simulation.SimulationStarter;
import simulation.Simulation;
import workerCode.Coord;
import workerCode.ProbMap;

public class Metrics implements Observer {

	private HashMap<SimulatedRobot, MetricRobotInformation> mriList = new HashMap<SimulatedRobot, MetricRobotInformation>();
	private Vector<Collection<SimulatedRobot>> commandQ = new Vector<Collection<SimulatedRobot>>();
	private static Simulation sim;
	private static Metrics metrics = new Metrics();
	static boolean isRunning = true;
	
	public  void printReport(int x, int y, int numLevels) {
		printReportToScreen(x,y, numLevels);
		printReportToFile(x,y, numLevels);
	}

	public void setUpMetricData() {	
		this.mriList.clear();
		
		for (SimulatedRobot sr : sim.getSimRobots()){
			this.mriList.put(sr, new MetricRobotInformation(sr));
		}
	}

	synchronized public void collectData(){
		commandQ.clear();
		isRunning = true;
		while(isRunning){
			//while there are no commands to execute, sleep
			
			while(commandQ.isEmpty()) {
				try {
					this.wait();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			//pass command to command handler
			Collection<SimulatedRobot> tempCol = this.commandQ.remove(0);
			this.updateHandler(tempCol);
		}
	}
	
	private void updateHandler(Collection<SimulatedRobot> tempCol) {
//		System.out.println("****************************** UPDATING");
		for (SimulatedRobot sr: tempCol){
			this.mriList.get(sr).update(sr);
		}
		
	}

	public static void setupSimulation(int mapx, int mapy, int numLevels, Vector<Integer>robotLevelNumbers){
		//The World Map
		ProbMap realMap= new ProbMap(mapx, mapy, 10, ProbMap.NO_OFFSET);
		
		//Worker Start Locations
		ArrayList<Coord> workerStartingLocations= new ArrayList<Coord>();
		workerStartingLocations.clear();
//		workerStartingLocations.add(new Coord(60, 0));
//		workerStartingLocations.add(new Coord(190, 0));
		
		//Scout Start Location
		ArrayList<Coord> scoutStartingLocations= new ArrayList<Coord>();
		scoutStartingLocations.clear();
//		scoutStartingLocations.add(new Coord(30, 0));
//		scoutStartingLocations.add(new Coord(90, 0));
//		scoutStartingLocations.add(new Coord(160, 0));
//		scoutStartingLocations.add(new Coord(220, 0));
		
		setRobotStartLocations(workerStartingLocations, scoutStartingLocations, robotLevelNumbers.get(1), robotLevelNumbers.get(2));
		
		//Simulation Starter, Robots, and Swarmbots
		SimulationStarter swarmMaker= new SimulationStarter();
		Collection<SimulatedRobot> swarmBots;
		swarmBots = swarmMaker.setupSwarm(robotLevelNumbers.get(1), workerStartingLocations, robotLevelNumbers.get(2), scoutStartingLocations, realMap);
		
		//The Simulation
		sim = new Simulation(realMap, swarmBots);
		
		//Add the Metric as an observer to the simulation
		sim.addObserver(metrics);
		
//		ProbMap map= new ProbMap(32, 32, 10, ProbMap.NO_OFFSET);
//		ArrayList<Coord> workerStartingLocations= new ArrayList<Coord>();
//		workerStartingLocations.add(new Coord(60, 0));
//		workerStartingLocations.add(new Coord(200, 0));
//		
//		ArrayList<Coord> scoutStartingLocations= new ArrayList<Coord>();
//		scoutStartingLocations.add(new Coord(30, 0));
//		scoutStartingLocations.add(new Coord(90, 0));
//		scoutStartingLocations.add(new Coord(160, 0));
//		scoutStartingLocations.add(new Coord(220, 0));
//		SimulationStarter swarmMaker= new SimulationStarter();
//		Collection<SimulatedRobot> swarmBots;
//		swarmBots= swarmMaker.setupSwarm(2, workerStartingLocations, 2, scoutStartingLocations, map);
//		Simulation_v2 sim= new Simulation_v2(map, swarmBots);
//		sim.run();
	}
	
	private static void setRobotStartLocations(ArrayList<Coord> workerStart, ArrayList<Coord> scoutStart, int numWorkers, int numScouts) {
		int x = SimulationParameters.initialOffsetFromZero; //The initial x offset for the first scout
		int numScoutsLeft; //The number of scouts on the left of a worker
		int numScoutsRight; //The number of scouts on the right of a worker
		
		//This if / else determines the number of scouts on the left and right of the worker
		if (numScouts %2 == 0){
			numScoutsLeft = numScouts / 2;
			numScoutsRight = numScouts / 2;
		}
		else {
			numScoutsLeft = numScouts / 2 + 1;
			numScoutsRight = numScouts /2;
		}
		
		//For each worker
		for (int i = 0; i < numWorkers; i++){
			
			//For each scout on the left, place a scout on the left and increment its position by 30cm
			for (int j = 0; j < numScoutsLeft; j++){
				scoutStart.add(new Coord(x, 0));
				x += SimulationParameters.placementIncrement;
			}
			
			//Place the worker in the middle
			workerStart.add(new Coord(x, 0));
			x+= SimulationParameters.placementIncrement;
			
			//For each scout on the right, place a scout and increment by 30
			for (int j = 0; j < numScoutsRight; j++){
				scoutStart.add(new Coord(x, 0));
				x += SimulationParameters.placementIncrement;
			}

			x+= (2 * numScouts + 10) - SimulationParameters.placementIncrement;
		}
		
		
//		for (Coord c : workerStart){
//			System.out.println(c);
//		}
//		for (Coord c : scoutStart){
//			System.out.println(c);
//		}
		
//		this.minXSize = ((x-SimulationParameters.placementIncrement) / simulationMap.getCellConversion());
		
	}
	
	/**
	 * Print a report to the screen
	 * @param y 
	 * @param x 
	 */
	public void printReportToScreen(int x, int y, int numLevels){
		int level0 = 0;
		int level1 = 0;
		int level2 = 0;
		
	     for (int i = 0; i < numLevels; i++){
			 for (MetricRobotInformation mri : this.mriList.values()){
				if (mri.getLevel() == i){
					if (i == 0){
						level0++;
					}
					else if (i == 1){
						level1++;
					}
					else if (i == 2){
						level2++;
					}
				}
	 	     }
//			 System.out.println();
	     }
		
		
		System.out.println("Map X: " + x + ", Map Y: " + y + ", Robots: " + level0 + " " + level1 + " " + level2);
		
		System.out.println(new String(//"ID, " + 
    	    					 "Level, " + 
    	    					 //"Runtime, " + 
    	    					 //"Scan Time, " + 
    	    					 //"Drive Time, " +
    	    					 //"Packets Sent, " +
    	    					 //"Packets Received, " + 
    	    					 "# Paths, " + 
    	    					 "Distance Driven, " + 
    	    					 "# Scans, " + 
    	    					 "Area Scanned, "));
		
		
	     for (int i = 0; i < numLevels; i++){
			 for (MetricRobotInformation mri : this.mriList.values()){
				if (mri.getLevel() == i){
					System.out.println(mri.toString());
				}
	 	     }
//			 System.out.println();
	     }
		
	}
	
	/**
	 * Print a report to the file system in the "Simulation Output" folder
	 * @param y 
	 * @param x 
	 */
	public void printReportToFile(int x, int y, int numLevels){
		int level0 = 0;
		int level1 = 0;
		int level2 = 0;
		
	     for (int i = 0; i < numLevels; i++){
			 for (MetricRobotInformation mri : this.mriList.values()){
				if (mri.getLevel() == i){
					if (i == 0){
						level0++;
					}
					else if (i == 1){
						level1++;
					}
					else if (i == 2){
						level2++;
					}
				}
	 	     }
			 System.out.println();
	     }
		
      try{
    	    // Create file 
    	  	String fileName = new String(x + "x, " + y + "y, " + level0 + " " + level1 + " " + level2 + ", time " + System.currentTimeMillis() + ".txt");
    	    FileWriter fstream = new FileWriter(new String("Simulation Output/" + fileName));
    	    BufferedWriter out = new BufferedWriter(fstream);
    	        
    	    out.write(new String("Map X: " + x + ", Map Y: " + y + ", Robots: " + this.mriList.size()));
    	    
    	    out.write(new String(//"ID, " + 
    	    					 "Level, " + 
    	    					 //"Runtime (ms), " + 
    	    					 //"Scan Time (ms), " + 
    	    					 //"Drive Time (ms), " +
    	    					 //"Packets Sent, " +
    	    					// "Packets Received, " + 
    	    					 "# Paths, " + 
    	    					 "Distance Driven (cm), " + 
    	    					 "# Scans, " + 
    	    					 "Area Scanned (cm), "));
    	    out.newLine();
    	    out.flush();
   	     for (int i = 0; i < numLevels; i++){   
   	    	 for (MetricRobotInformation mri : this.mriList.values()){
   	    		 if (mri.getLevel() == i){
   	    			 out.write(mri.toString());
   	    			 out.newLine();
   	    			 out.flush();
   	    		 }
    	     }
   	     }
    	    
    	  //Close the output stream
    	    out.close();
    	    }catch (Exception e){//Catch exception if any
    	      System.err.println("Error: " + e.getMessage());
    	    }
	}
	
	@Override
	public void update(Observable o, Object arg) {

		if (arg instanceof SwarmCommunication){
			SwarmCommunication tempComm = (SwarmCommunication) arg;
			if (tempComm.getPacket() instanceof KillPacket){
				isRunning = false;
				//Do stuff since its not a kill packet
				this.commandQ.add(sim.getSimRobots());
				//notify the command thread
				synchronized(this) {
					this.notifyAll();
				}
			}
		}
		
	}

	public void run(int x, int y, int numLevels, Vector<Integer> numRobots) {
		//Setup the Simulation
		Metrics.setupSimulation(x, y, numLevels, numRobots);
		
		//Set up the metrics now that the simulation is setup
		metrics.setUpMetricData();
		
		//Run the Simulation
		sim.run();	
		
		//Start Data Collection
		metrics.collectData();
		
		metrics.printReport(x,y, numLevels);
		
	}

}
