package communication;


import java.util.ArrayList;

import communication.RemoteCommDriver;

public class VirtualRemoteCommDriver extends RemoteCommDriver {
	
	private String name; //Name of this driver
	private VirtualRemoteCommDriver[] communicationTable;
	private VirtualRemoteCommDriver sender; //reference to the remote driver which
	                                        //sent us something
	private ArrayList<Byte> dataBuf;  //holds the data from the last received packet
	private boolean hasReceivedData; //true if there is data in the id/data
	                                 //buffer, false otherwise
	
	public static final int MAX_COMMUNICATION_TABLE= 100;
	public static final int MAX_PAYLOAD_SIZE= 40000;
	
	private int numChildren;
	private boolean isRunning;
	
	
	public VirtualRemoteCommDriver() {
		this.dataBuf= new ArrayList<Byte>();
		this.communicationTable= new VirtualRemoteCommDriver[MAX_COMMUNICATION_TABLE];
		this.hasReceivedData= false;
		this.numChildren= 0;
		this.name= "Driver";
		this.isRunning= true;
	}
	
	public VirtualRemoteCommDriver(String name) {
		this.dataBuf= new ArrayList<Byte>();
		this.communicationTable= new VirtualRemoteCommDriver[MAX_COMMUNICATION_TABLE];
		this.hasReceivedData= false;
		this.numChildren= 0;
		this.name= name;
		this.isRunning= true;
	}
	

	@Override
	public void run() {
		while(this.isRunning) {
			synchronized(this) {
				while((this.hasReceivedData == false) && (this.isRunning == true)) {
					try {
						this.wait();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
				if(this.isRunning == false) break;
				//wake up and notify the observers
				this.setChanged();
				//empty buffer
				byte[] bufferCopy;
				byte id;
				bufferCopy= new byte[this.dataBuf.size()];
				id= this.lookupSenderID(this.sender);
				for(int i= 0; i < this.dataBuf.size(); i++) {
					bufferCopy[i]= this.dataBuf.get(i);
				}
				this.dataBuf.clear();
				this.hasReceivedData = false;
				this.notifyAll();
				//notify the observers with the copy of the buffer
				this.notifyObservers(new RemoteCommDriver.ReceiveDataArray(id, bufferCopy));
			}
		}
	}

	@Override
	public boolean send(byte id, byte[] serialData) {
		VirtualRemoteCommDriver recipient= communicationTable[id];
		if(recipient == null) {
			System.out.println("Recipient Null, ID:" + id + this.name);
			return false;
		}
		
		//try filling the buffer until successful
		recipient.fillBuffer(this, serialData);
		return true;
	}
	
	@Override
	public void kill() {
		this.isRunning= false;
		synchronized(this) {
			this.notifyAll();
		}
	}
	
	
	/**
	 * Fill the receive buffer of this driver with the given data.  This
	 * function will automatically perform a deep copy of the data before it
	 * is placed into the buffer.
	 * @param sender The object which is filling the buffer.
	 * @param data The data to be copied into the buffer.
	 * @return true if the data is now in the buffer, false if the data could
	 * not be stored because the buffer is full.
	 */
	public void fillBuffer(VirtualRemoteCommDriver sender, byte[] data) {
		synchronized(this) {
			while(this.hasReceivedData == true) {
			//	System.out.println("Going to Sleep");
				try {
					this.wait();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			
			this.sender= sender;
			for(int i= 0; i < data.length; i++) {
				this.dataBuf.add(data[i]);
			}

			this.hasReceivedData= true;
			this.notifyAll();
		}
	}
	
	public void addToTable(byte id, VirtualRemoteCommDriver entry) {
		this.communicationTable[id]= entry;
	}
	
	public byte lookupSenderID(VirtualRemoteCommDriver sender) {
		for(byte i= 0; i < this.communicationTable.length; i++) {
			if(this.communicationTable[i] == sender) {
				return i;
			}
		}
		return -1;
	}
	
	public void addParent(VirtualRemoteCommDriver parent) {
		this.communicationTable[0]= parent;
	}
	
	public void addChild(VirtualRemoteCommDriver child) {
		numChildren++;
		this.communicationTable[numChildren]= child;
		
	}
	
	public String getName() {
		return this.name;
	}

}
