package gui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.JComponent;

import communication.Comm;
import communication.VirtualRemoteCommDriver;

import simulation.VirtualLocomotionDriver;

import workerCode.*;

/**
 * @author Ricky
 *
 */

@SuppressWarnings("serial")
public class DrawingArea extends JComponent implements MouseListener {
		
		private static final int MINIMUM_USEFUL_SIZE = 4;
		
		private int xSize = 200;
		private int ySize = 200;
		
		int squareSize = 15;
		int selectedX = 1000;
		int selectedY = 1000;
		private JComponent parent;
		private int scale = 10;
		
		//Expects blockColors between 0 and 1
		double[][] blockColors = new double[200][200];
		boolean[][] scanned = new boolean[200][200];
		
		private ArrayList<GUIRobot> guiRobotList = new ArrayList<GUIRobot>();
		
		private HashMap<Robot, GUIRobot> robotMap = new HashMap<Robot,GUIRobot>();
		
		public DrawingArea(JComponent container) {
			addMouseListener(this);
//			this.testScanned();
//			this.testBlockColors();
			this.parent = container;
			this.zoomToFit(container);
//			this.testGUIRobots();
//			this.testHashedRobots();
		}
		
		public void setxSize(int xSize) {
			this.xSize = xSize;
			this.updateSize();
			repaint();
		}

		public void setySize(int ySize) {
			this.ySize = ySize;
			this.updateSize();
			repaint();
		}

		public int getySize() {
			return ySize;
		}

		public int getxSize() {
			return xSize;
			
		}

		/**
		 * Sets the pixel size of the cells in the grid
		 * @param newSize
		 */
		public void setSquareSize(int newSize) {
			this.squareSize = newSize;
			this.updateSize();
		}


		/**
		 * Resizes the grid to best fit in the available window
		 */
		public void zoomToFit(JComponent container) {
			int minDimension = Math.min(container.getWidth(), container.getHeight());
			int newSize = minDimension / (Math.max(xSize, ySize));
			if(newSize > MINIMUM_USEFUL_SIZE) {
				this.setSquareSize(newSize);
			}
			else {
				this.setSquareSize(MINIMUM_USEFUL_SIZE);
			}
			repaint();
		}
		
		/**
		 * Resizes the grid to best fit in the available window.
		 * This overloaded version assumes the current parent.
		 */
		public void zoomToFit() {
			int minDimension = Math.min(parent.getWidth(), parent.getHeight());
			int newSize = minDimension / (Math.max(xSize, ySize));
			if(newSize > MINIMUM_USEFUL_SIZE) {
				this.setSquareSize(newSize);
			}
			else {
				this.setSquareSize(MINIMUM_USEFUL_SIZE);
			}
			repaint();
		}

		/**
		 * Draws the map and everything on it
		 */
		public void paintComponent(Graphics g) {
			
			Graphics2D g2d = (Graphics2D) g;
			int cellColor;
			float[] tempVals = new float[2];
			g2d.setStroke(new BasicStroke(1));
			
			// For each cell in the grid
			for (int x = 0; x < xSize; x++) {
				for (int y = 0; y < ySize; y++) {
					int drawY = ySize - y -1;
					// Pick the color
//					if(selectedX == x && selectedY == y) {
//						g2d.setColor(Color.RED);
//					}
					if (!scanned[x][y]) {
						g2d.setColor(Color.RED);
					}
					else {
						cellColor = 255 - ((int)(((double) 255) * blockColors[x][y]));
						tempVals = Color.RGBtoHSB(cellColor, cellColor, cellColor, null);
						g2d.setColor(Color.getHSBColor(tempVals[0], tempVals[1], tempVals[2]));
					}
					
					// Fill the box
					g2d.fillRect(x*squareSize, (drawY*squareSize), squareSize, squareSize);
					
					// Draw the grid
					g2d.setColor(Color.BLACK);
					g2d.drawRect(x*squareSize, (drawY*squareSize), squareSize, squareSize);
					
				}
			}
			// Draw the guirobots and the radius if appropriate (and the path)
			
			if(!robotMap.isEmpty()) {
				
				for(GUIRobot tempRobot : robotMap.values()) {
					if(tempRobot.getRobotType() != 'Q') {

						Coord tempCoord = new Coord(tempRobot.getLocation().getX()/scale, tempRobot.getLocation().getY()/scale);
						int tempRadius = tempRobot.getRadius()/scale;
						
						// Draw the robot's path
						g2d.setColor(Color.BLUE);
						
						if(tempRobot.getPath().size() > 0) {
							g2d.setStroke(new BasicStroke(squareSize/MINIMUM_USEFUL_SIZE));
							g2d.drawLine((((tempCoord.getX())) * squareSize + squareSize / 2), 
									 ((ySize - (tempCoord.getY())) * squareSize) - (squareSize / 2),
									 ((tempRobot.getPath().get(0).getX()/scale) * squareSize + squareSize / 2), 
									 ((ySize - (tempRobot.getPath().get(0).getY() / scale)) * squareSize) - (squareSize / 2));
							
								for(int j = 0; j < tempRobot.getPath().size() - 1; j++) {
									g2d.drawLine(((tempRobot.getPath().get(j).getX()/scale) * squareSize + squareSize / 2), 
												 ((ySize - (tempRobot.getPath().get(j).getY() / scale)) * squareSize) - (squareSize / 2),
												 ((tempRobot.getPath().get(j+1).getX()/scale) * squareSize + squareSize / 2), 
												 ((ySize - (tempRobot.getPath().get(j+1).getY() / scale)) * squareSize) - (squareSize / 2));
								}
						}
						
						// Drawing robot
						g2d.setStroke(new BasicStroke(1));
						g2d.setColor(Color.DARK_GRAY); // Change later on colour wheel
				
						g2d.fillOval((tempCoord.getX()*squareSize - (squareSize / 2)),
									((ySize - tempCoord.getY() - 2) * squareSize + (squareSize / 2)),
									squareSize*2,
									squareSize*2);
						
						//Labeling the robot
						g2d.setFont(new Font("Times New Roman", Font.PLAIN, squareSize));
						g2d.setColor(Color.ORANGE);
						char[] tempArr = new char[1];
						tempArr[0] = tempRobot.getRobotType();
						g2d.drawChars(tempArr, 0, 1, tempCoord.getX()*squareSize, ((ySize - tempCoord.getY()) * squareSize));
						
						// Draw the scan radius
						g2d.setColor(Color.DARK_GRAY);
						g2d.setStroke(new BasicStroke(squareSize/(MINIMUM_USEFUL_SIZE)));
						if(tempRobot.shouldDrawRadius()) {
							g2d.drawOval((tempCoord.getX()*squareSize - squareSize*tempRadius + (squareSize / 2)), 
									(ySize - tempCoord.getY() - 1)*squareSize - (squareSize*tempRadius) + (squareSize) / 2, 
										(squareSize*tempRadius*2), 
										(squareSize*tempRadius*2));
						}
					}
				}
			}
		}

		/**
		 * Updates the size of the drawing area and its container object
		 */
		private void updateSize() {
			this.setSize(new Dimension(xSize*squareSize, ySize*squareSize));
			parent.setSize(new Dimension(xSize*squareSize, ySize*squareSize));
		}
		


		public ArrayList<GUIRobot> getGuiRobotList() {
			return guiRobotList;
		}
		
		@Override
		public void mouseClicked(MouseEvent e) {
			// Intentionally left blank
		}

		@Override
		public void mouseEntered(MouseEvent e) {
			// Intentionally left blank
		}

		@Override
		public void mouseExited(MouseEvent e) {
			// Intentionally left blank

		}

		@Override
		public void mousePressed(MouseEvent e) {
			//selectedX = e.getX() / squareSize;
			//selectedY = ySize - ((e.getY()) / squareSize) - 1;
			//repaint();
		}

		@Override
		public void mouseReleased(MouseEvent e) {
			// Intentionally left blank
		}
		
		/**
		 * Replaces the data for the gui with that of the given map
		 * @param map
		 */
		public void replaceMap(ProbMap map) {
			this.xSize = map.getxSize(); 
			this.ySize = map.getySize();
			for(int x = 0; x < map.getxSize(); x++) {
				for(int y = 0; y < map.getySize(); y++) {
					this.blockColors[x][y] = map.cellGetBlockValue(new Coord(x+map.getXOffset(),y+map.getYOffset()));
				}
			}
		}

		/**
		 * Used to set the Robot to GUIrobot HashMap to the one provided
		 * @param robotMap
		 */
		public void setRobotMap(HashMap<Robot, GUIRobot> robotMap) {
			this.robotMap = robotMap;
		}

		/**
		 * Returns the HashMap of Robots to GUIrobots
		 * @return
		 */
		public HashMap<Robot, GUIRobot> getRobotMap() {
			return robotMap;
		}
		
		/**
		 * Updates the GUIrobot in the hashmap with the given location
		 * @param robot
		 * @param newLocation
		 */
		public void updateRobotPosition(Robot robot, Coord newLocation) {
			this.getRobotMap().get(robot).setLocation(newLocation);
			repaint();
		}
		
		/**
		 * Updates the GUIrobot's scan radius and tells the GUI that it
		 * should be drawn.
		 * 
		 * @param robot
		 * @param newRadius
		 */
		public void updateScanRadius(Robot robot, int newRadius) {
			this.getRobotMap().get(robot).setRadius(newRadius);
			this.getRobotMap().get(robot).setDrawRadius(true);
			repaint();
		}
		
		/**
		 * Simply sets the GUIrobot to not draw a scan radius
		 * @param robot
		 */
		public void updateScanComplete(Robot robot) {
			this.getRobotMap().get(robot).setDrawRadius(false);
			repaint();
		}
		
		/**
		 * Replaces the path of the robot given with the provided path
		 * @param robot
		 * @param path
		 */
		public void updatePath(Robot robot, ArrayList<Coord> path) {
			this.clearPath(robot);
			for(Coord c : path) {
				this.getRobotMap().get(robot).getPath().add(c.deepCopy());
			}
			repaint();
		}
		
		/**
		 * Clears the path of the robot given (does not make it NULL)
		 * @param robot
		 */
		public void clearPath(Robot robot) {
			this.getRobotMap().get(robot).getPath().clear();
			repaint();
		}
		
		/**
		 * Updates the local scanned and blocked variables using the given map chunk
		 * @param mapChunk
		 */
		public void updateMapChunk(ProbMap mapChunk) {
			Coord tempCoord;
			for(int x = mapChunk.getXBoundMin(); x < mapChunk.getXBoundMax(); x++) {
				for(int y = mapChunk.getYBoundMin(); y < mapChunk.getYBoundMax(); y++) {
					tempCoord = new Coord(x,y);
					this.scanned[x][y] = mapChunk.cellIsScanned(tempCoord);
					this.blockColors[x][y] = mapChunk.cellGetBlockValue(tempCoord);
				}
			}
			repaint();
		}
		
		
		
		/* Sample and testing methods  */
		/* * * * * * * * * * * * * * * */
		
		/**
		 * Sets all values of scanned[][] to true
		 */
		private void testScanned() {
			for (int x = 0; x < xSize; x++) {
				for (int y = 0; y < ySize; y++) {
					scanned[x][y] = true;
				}
			}
		}
		
		/**
		 * Sets blockColor[][] values from min to max
		 */
		private void testBlockColors() {
			for (int x = 0; x < xSize; x++) {
				for (int y = 0; y < ySize; y++) {
					blockColors[x][y] = (double) (1 - ((double) y / (double) Math.min(xSize, ySize)));
				}
			}
		}
		
		/**
		 * Sets up a couple sample robots with paths
		 */
		private void testGUIRobots() {
			int i;
			for(int j = 0; j < 3; j++) {
				i = j*5;
				this.guiRobotList.add(new GUIRobot());
				this.guiRobotList.get(j).setDrawRadius(true);
				this.guiRobotList.get(j).setLocation(new Coord(i*scale, i*scale));
				this.guiRobotList.get(j).setRadius(i*scale);
				ArrayList<Coord> tempList = new ArrayList<Coord>();
				tempList.add(new Coord(i*scale,i*scale));
				tempList.add(new Coord(i*4*scale,i*2*scale));
				//tempList.add(new Coord(i*8,i*4));
				this.guiRobotList.get(j).setPath(tempList);
			}
			this.guiRobotList.get(0).getPath().add(new Coord(10*scale,10*scale));
			this.guiRobotList.get(0).getPath().add(new Coord(10*scale,20*scale));
			this.guiRobotList.get(0).getPath().add(new Coord(20*scale,20*scale));
		}
		
		private void testHashedRobots() {
			Worker tempBot = new Worker(100, 10,
					new Comm(new VirtualRemoteCommDriver()), 1,
					new VirtualLocomotionDriver(new Coord(10,10)));
			robotMap.put(tempBot, new GUIRobot(tempBot));
			robotMap.get(tempBot).setDrawRadius(true);
			robotMap.get(tempBot).setRadius(100);
			ArrayList<Coord> tempPath = new ArrayList<Coord>();
			tempPath.add(new Coord(20,20));
			tempPath.add(new Coord(20,50));
			tempPath.add(new Coord(100,70));
			tempPath.add(new Coord(30,200));
			tempPath.add(new Coord(200,20));
			robotMap.get(tempBot).setPath(tempPath);
			robotMap.get(tempBot).setLocation(new Coord(10,10));
			repaint();
		}

	}
