package gui;

import java.awt.Color;
import java.awt.Graphics2D;

public class GUICell {
	private double blockVal;
	private boolean isScanned;
	
	public GUICell() {
		this.blockVal= 0;
		this.isScanned= false;
	}
	/**
	 * @param blockVal the blockVal to set
	 */
	public void setBlockVal(double blockVal) {
		if(blockVal > 1) blockVal= 1;
		if(blockVal < 0) blockVal= 0;
		this.blockVal = blockVal;
	}
	/**
	 * @return the blockVal
	 */
	public double getBlockVal() {
		return blockVal;
	}
	/**
	 * @param isScanned the isScanned to set
	 */
	public void setScanned(boolean isScanned) {
		this.isScanned = isScanned;
	}
	/**
	 * @return the isScanned
	 */
	public boolean isScanned() {
		return isScanned;
	}
	
	/**
	 * Draw this GUICell at the given x,y location on the given graphics
	 * object. 0,0 corresponds to the top left corner.
	 * @param g2d The graphics object to draw on.
	 * @param x The x location to draw (top left corner).
	 * @param y The y location to draw (top left corner).
	 * @param cellSize The side length to draw.
	 */
	public void drawCell(Graphics2D g2d, int x, int y, int cellSize) {
		//if cell is unscanned draw as red
		if(!this.isScanned()) {
			g2d.setColor(Color.RED);
		}
		//otherwise draw in grayscale based on block value
		else {
			int level= (int)(255*(1- this.getBlockVal()));
			Color gray= new Color(level, level, level);
			g2d.setColor(gray);
		}
		//fill in the cell internal color
		g2d.fillRect(x, y, cellSize, cellSize);
		
		//draw the border
		g2d.setColor(Color.BLACK);
		g2d.drawRect(x, y, cellSize, cellSize);
	}
}
