package gui;

import java.awt.Graphics2D;

public class GUIMap {
	private GUICell[][] map;
	private int cellSize= 10; //cell size in cm
	
	/**
	 * Create a guimap
	 * @param xSize the number of cells in the width
	 * @param ySize the number of cells in the height
	 */
	public GUIMap(int xSize, int ySize, int cellSize) {
		this.map= new GUICell[ySize][xSize];
		for(int x= 0; x < xSize; x++) {
			for(int y= 0; y < ySize; y++) {
				this.map[y][x]= new GUICell();
			}
		}
		this.cellSize= cellSize;
	}
	
	/**
	 * Get the GUICell at the given x,y. 0,0 is defined as the bottom left
	 * corner of the map.
	 * @param x The x position.
	 * @param y The y position.
	 * @return The GUICell at x,y.
	 */
	public GUICell getGUICell(int x, int y) {
		return this.map[y][x];
	}
	
	/**
	 * Set the GUICell located at x,y to be some new value.  0,0 is defined as
	 * the bottom left corner of the map.
	 * @param x The x coordinate of the new GUICell
	 * @param y The y coordinate of the new GUICell
	 * @param newCell The new cell to store at x,y.
	 */
	public void setGUICell(int x, int y, GUICell newCell) {
		this.map[y][x]= newCell;
	}
	
	/**
	 * 
	 * @return x size in pixels
	 */
	public int getXSize() {
		return this.map[0].length*this.cellSize;
	}
	
	/**
	 * 
	 * @return y size in pixels
	 */
	public int getYSize() {
		return this.map.length*this.cellSize;
	}
	
	/**
	 * Draw this map with the given graphics object
	 * @param g2d
	 */
	public void drawMap(Graphics2D g2d) {
		for(int x= 0; x < this.map[0].length; x++) {
			for(int y= 0; y < this.map.length; y++) {
				// draw each cell accounting for the fact that Graphics2D draws
				// with 0,0 at top left
				this.map[y][x].drawCell(g2d, x*getCellSize(), (this.map.length - 1 - y)*getCellSize(), this.getCellSize());
			}
		}
	}

	/**
	 * @param cellSize the cellSize to set
	 */
	public void setCellSize(int cellSize) {
		this.cellSize = cellSize;
	}

	/**
	 * @return the cellSize in px
	 */
	public int getCellSize() {
		return cellSize;
	}
}
