package gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTree;
import javax.swing.SpinnerNumberModel;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.ZoneView;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import communication.ChildInformationPacket;
import communication.Comm;
import communication.KillPacket;
import communication.PathPacket;
import communication.ReachedDestinationPacket;
import communication.ScanPacket;
import communication.SimulationDonePacket;
import communication.SimulationScanAreaUpdatedPacket;
import communication.SwarmCommunication;
import communication.SwarmPacket;
import communication.VirtualRemoteCommDriver;

import net.miginfocom.swing.MigLayout;

import simulation.Maps;
import simulation.Scout;
import simulation.SimulatedRobot;
import simulation.Simulation;
import simulation.SimulationPacket;
import simulation.SimulationStarter;
import simulation.VirtualLocomotionDriver;
import simulation.VirtualScanDriver;
import workerCode.Coord;
import workerCode.ProbMap;
import workerCode.Queen;
import workerCode.Robot;
import workerCode.Worker;

import javax.swing.ScrollPaneConstants;
import javax.swing.JTextField;

public class MainGUI implements Runnable, Observer {

	private JFrame frame;
	private Simulation theSim;
	private Thread simThread;
	private DrawingArea_v2 da;
	private Map<GUIRobot, SimulatedRobot> guiBotToSimBot= new HashMap<GUIRobot, SimulatedRobot>();
	private double pixelsPerCm= .1;
	private JButton runButton;
	private JSlider runSpeedSlider;
	private JSpinner xSizeSpinner;
	private JSpinner ySizeSpinner;
	private JScrollPane scrollPane;
	private JTextField swarmConfigTextField;
	public static int TIMESCALE= 1;
	
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
		} catch (Throwable e) {
			e.printStackTrace();
		}
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					MainGUI window = new MainGUI();
					window.frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public MainGUI() {
		initialize();
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		frame = new JFrame("Hierarchical Swarm Simulation");
		frame.setBounds(100, 100, 1024, 850);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().setLayout(new BorderLayout(5, 5));
		frame.setMinimumSize(new Dimension(640, 480));
		
		Component glue = Box.createGlue();
		frame.getContentPane().add(glue, BorderLayout.NORTH);
		
		JPanel westPanel = new JPanel();
		frame.getContentPane().add(westPanel, BorderLayout.WEST);
		westPanel.setLayout(new BorderLayout(0, 0));
		
		Component glue_1 = Box.createGlue();
		westPanel.add(glue_1, BorderLayout.CENTER);
		
		
		JPanel configPanel = new JPanel();
		westPanel.add(configPanel, BorderLayout.NORTH);
		configPanel.setLayout(new MigLayout("", "[][][]", "[][][][][]"));
		configPanel.setMinimumSize(new Dimension(250, 480));
		configPanel.setMaximumSize(new Dimension(250, 480));
		
		JLabel swarmCfgLabel = new JLabel("Swarm Configuration");
		swarmCfgLabel.setEnabled(false);
		swarmCfgLabel.setFont(new Font("Tahoma", Font.BOLD, 12));
		configPanel.add(swarmCfgLabel, "cell 0 0 3 1,alignx center");
		
		JPanel westTopPanel = new JPanel();
		configPanel.add(westTopPanel, "flowx,cell 0 1 3 1");
		
		
		Component glue_4 = Box.createGlue();
		configPanel.add(glue_4, "cell 1 2");
		
		JLabel xSizeLabel = new JLabel("Map X Size (cells):");
		configPanel.add(xSizeLabel, "cell 0 3");
		
		Component glue_3 = Box.createGlue();
		configPanel.add(glue_3, "cell 1 3");
		
		xSizeSpinner = new JSpinner();
		xSizeSpinner.setModel(new SpinnerNumberModel(105, 5, 200, 5));
		configPanel.add(xSizeSpinner, "cell 2 3,growx");
		
		xSizeSpinner.addChangeListener( new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				da.setGuiMap(new GUIMap((Integer)(xSizeSpinner.getValue()), (Integer)(ySizeSpinner.getValue()),
						(int)(pixelsPerCm*10)));
				da.validate();
				da.repaint();
			}
			
		});
		
		JLabel ySizeLabel = new JLabel("Map Y Size (cells): ");
		configPanel.add(ySizeLabel, "cell 0 4");
		
		ySizeSpinner = new JSpinner();
		ySizeSpinner.setModel(new SpinnerNumberModel(105, 5, 200, 5));
		configPanel.add(ySizeSpinner, "cell 2 4,growx");
		
		swarmConfigTextField = new JTextField();
		swarmConfigTextField.setText("1 2 2");
		configPanel.add(swarmConfigTextField, "cell 0 1");
		swarmConfigTextField.setColumns(10);
		
		ySizeSpinner.addChangeListener( new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				da.setGuiMap(new GUIMap((Integer)(xSizeSpinner.getValue()), (Integer)(ySizeSpinner.getValue()),
						(int)(pixelsPerCm*10)));
				scrollPane.revalidate();
				scrollPane.repaint();
			}
			
		});
		
		JPanel runPanel = new JPanel();
		westPanel.add(runPanel, BorderLayout.SOUTH);
		runPanel.setLayout(new MigLayout("", "[]", "[]"));
		
		JLabel runLabel = new JLabel("Simulation Operation");
		runLabel.setEnabled(false);
		runLabel.setFont(new Font("Tahoma", Font.BOLD, 12));
		runPanel.add(runLabel, "cell 0 0,alignx center");
		
		JLabel lblZoom = new JLabel("Zoom: ");
		runPanel.add(lblZoom, "cell 0 1, alignx center");
		
		final JSlider zoomSlider = new JSlider();
		zoomSlider.setSnapToTicks(true);
		zoomSlider.setMajorTickSpacing(4);
		zoomSlider.setPaintTicks(true);
		zoomSlider.setMinorTickSpacing(1);
		zoomSlider.setMinimum(1);
		zoomSlider.setMaximum(100);
		zoomSlider.setValue(5);
		
		// Slider actions
		zoomSlider.addChangeListener( new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				pixelsPerCm= ((double)zoomSlider.getValue())/10;
				da.getGuiMap().setCellSize((int) (pixelsPerCm*10));
				//update the robots
				for(GUIRobot gr : da.getGuiRobots()) {
					SimulatedRobot sr= guiBotToSimBot.get(gr);
					gr.setRadius((int) (sr.getCurrentScanRadius()*pixelsPerCm));
					if(gr.getRadius() < 1) {
						gr.setDrawRadius(false);
					}
					else {
						gr.setDrawRadius(true);
					}
					Coord loc= sr.getCurrentLocation().deepCopy();
					loc.setX((int) (loc.getX()*pixelsPerCm));
					loc.setY((int) (da.getGuiMap().getYSize() - loc.getY()*pixelsPerCm));
					gr.setLocation(loc);
					ArrayList<Coord> path= sr.getCurrentPath();
					for(Coord c : path) {
						c.setX((int) (c.getX()*pixelsPerCm));
						c.setY((int) (da.getGuiMap().getYSize() - c.getY()*pixelsPerCm));
					}
					gr.setPath(path);
					gr.setRobotSize((int) (pixelsPerCm*25));
				}
				scrollPane.revalidate();
				scrollPane.repaint();
//				panel.setPreferredSize(da.getSize());
//				scrollPane.setPreferredSize(centerPanel.getSize());
//				centerPanel.validate();
			}
			
		});
		
		runPanel.add(zoomSlider, "cell 0 2,growx");		
		
		JLabel runSpeedLabel = new JLabel("Simulation Run Speed:");
		runPanel.add(runSpeedLabel, "cell 0 3,alignx center");
		
		runSpeedSlider = new JSlider();
		runSpeedSlider.setValue(1);
		runSpeedSlider.setSnapToTicks(true);
		runSpeedSlider.setPaintTicks(true);
		runSpeedSlider.setMajorTickSpacing(3);
		runSpeedSlider.setMinorTickSpacing(1);
		runSpeedSlider.setMaximum(9);
		runPanel.add(runSpeedSlider, "cell 0 4,growx");
		
		runSpeedSlider.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				MainGUI.TIMESCALE = (int) Math.pow(2, runSpeedSlider.getValue());
			}
		});
		
		runButton = new JButton("Run Simulation");
		runPanel.add(runButton, "cell 0 5,growx");
		
		JPanel bottomPanel = new JPanel();
		frame.getContentPane().add(bottomPanel, BorderLayout.SOUTH);
		bottomPanel.setLayout(new MigLayout("", "[624px,grow]", "[28px]"));
		
		JButton ZoomToFitButton = new JButton("Zoom to fit");
		
		// Button Actions
		ZoomToFitButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
//				panel.setPreferredSize(da.getSize());
//				scrollPane.setPreferredSize(centerPanel.getSize());	
//				da.zoomToFit(scrollPane);
				
				//Do twice for desired effect
//				panel.setPreferredSize(da.getSize());
//				scrollPane.setPreferredSize(centerPanel.getSize());	
//				centerPanel.validate();
//				da.zoomToFit(scrollPane);	
			}
		});
		
		bottomPanel.add(ZoomToFitButton, "cell 0 0,alignx leading,aligny top");
		
		final JPanel centerPanel = new JPanel();
		frame.getContentPane().add(centerPanel, BorderLayout.CENTER);
		centerPanel.setLayout(new BorderLayout(0, 0));
		
		scrollPane = new JScrollPane();
		centerPanel.add(scrollPane, BorderLayout.CENTER);
		
		final JPanel panel = new JPanel(new BorderLayout(0, 0));
	
		da = new DrawingArea_v2(panel);
		scrollPane.setViewportView(panel);
		panel.add(da, BorderLayout.CENTER);
		
		runButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				simulationSetup();
			}
		});
		centerPanel.setMaximumSize(new Dimension(4000,4000));
		centerPanel.setSize(da.getSize());

		centerPanel.add(scrollPane);
		panel.setPreferredSize(da.getSize());
		scrollPane.setPreferredSize(centerPanel.getSize());
		
		Component horizontalStrut = Box.createHorizontalStrut(1);
		centerPanel.add(horizontalStrut, BorderLayout.EAST);
		
		frame.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				frame.validate();
			}
		});
		
		centerPanel.validate();
	}
	

	@Override
	public void run() {
		try {
			MainGUI window = new MainGUI();
			window.frame.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public void update(Observable notifier, Object simPacket) {
		if(simPacket instanceof SwarmCommunication) {
			if(((SwarmCommunication)simPacket).getPacket() instanceof KillPacket) {
				this.setRunComponentEnable(true);
			}
		}
		if(notifier == this.theSim) {
			this.simUpdate();
		}
	}
	
	private void simUpdate() {
		//update the map
		ProbMap observedMap= this.theSim.getObservedMap();
		for(int x= 0; x < observedMap.getxSize(); x++) {
			for(int y= 0; y < observedMap.getySize(); y++) {
				this.da.getGuiMap().getGUICell(x, y).setBlockVal(
						observedMap.getCell(new Coord(x, y)).getBlockVal());
				this.da.getGuiMap().getGUICell(x, y).setScanned(
						observedMap.getCell(new Coord(x, y)).isScanned());
			}
		}
		
		//update the robots
		for(GUIRobot gr : this.da.getGuiRobots()) {
			SimulatedRobot sr= this.guiBotToSimBot.get(gr);
			gr.setRadius((int) (sr.getCurrentScanRadius()*this.pixelsPerCm));
			if(gr.getRadius() < 1) {
				gr.setDrawRadius(false);
			}
			else {
				gr.setDrawRadius(true);
			}
			Coord loc= sr.getCurrentLocation().deepCopy();
			loc.setX((int) (loc.getX()*this.pixelsPerCm));
			loc.setY((int) (da.getGuiMap().getYSize() - loc.getY()*this.pixelsPerCm));
			gr.setLocation(loc);
			ArrayList<Coord> path= sr.getCurrentPath();
			for(Coord c : path) {
				c.setX((int) (c.getX()*this.pixelsPerCm));
				c.setY((int) (da.getGuiMap().getYSize() - c.getY()*this.pixelsPerCm));
			}
			gr.setPath(path);
			gr.setRobotSize((int) (pixelsPerCm*25));
		}
	}

	private void simulationSetup() {
		if(this.swarmConfigTextField.getText().length() == 0) return;
		this.da.setGuiMap(new GUIMap((Integer)(this.xSizeSpinner.getValue()),
				(Integer)(this.ySizeSpinner.getValue()), (int)(this.pixelsPerCm*10)));
//		//TODO create pseudo random blocked map
		ProbMap newMap=
			new ProbMap((Integer)(this.xSizeSpinner.getValue()), 
					(Integer)(this.ySizeSpinner.getValue()), 10, ProbMap.NO_OFFSET);
		
//		for (int x = (newMap.getxSize() / 2) - 20; x < (newMap.getxSize() / 2); x++){
//			for (int y = (newMap.getySize() / 2) - 20; y < (newMap.getySize() / 2); y++){
//				newMap.cellSetBlockValue(new Coord(x,y), 1.0);
//			}
//		}
		
		
		//create sim robots
		this.makeSimulatedRobot(null, new Coord(0,0), 0, 0, this.parseString(this.swarmConfigTextField.getText()), newMap);
		this.da.setGuiRobots(this.guiBotToSimBot.keySet());
		this.theSim= new Simulation(newMap, this.guiBotToSimBot.values());
		theSim.addObserver(this);
//		createRobotMapping(da);
		simThread = new Thread(theSim);
		simThread.start();
		setRunComponentEnable(false);
	}
	
	private SimulatedRobot makeSimulatedRobot(SimulatedRobot parent, Coord parentLoc, int level, int childNum, Vector<Integer> config, ProbMap worldMap) {
		VirtualRemoteCommDriver newVirtualRemoteComm= new VirtualRemoteCommDriver();
		if(parent != null) {
			newVirtualRemoteComm.addParent(parent.getRobotVirtualRemoteCommDriver());
		}
		Comm c= new Comm(newVirtualRemoteComm);
		Robot r;
		int numChildren;
		int minBubbleSize= this.calcMinBubbleSize(config, level);
		Coord startingLocation= this.calculateStartingLocation(config, level, childNum, parentLoc);
		VirtualLocomotionDriver locDriver= new VirtualLocomotionDriver(startingLocation);
		if(level + 1 >= config.size()) {
			numChildren= 0;
			System.out.println("making scout at "+startingLocation);
			r= new Scout(80, 20, c, 0,
					locDriver,
					worldMap);
		}
		else if(level == 0) {
			numChildren= config.get(level+1);
			startingLocation= new Coord(0, 0);
			System.out.println("making queen with "+numChildren);
			r= new Queen(0, 0, c, numChildren,
					new VirtualLocomotionDriver(new Coord()),
					new ProbMap((Integer)(this.xSizeSpinner.getValue()),
							(Integer)(this.ySizeSpinner.getValue()), 10, ProbMap.NO_OFFSET));
		}
		else {
			numChildren= config.get(level+1);
			System.out.println("making worker with "+numChildren+" at "+startingLocation);
			r= new Worker(Comm.MAX_RANGE, minBubbleSize, c, config.get(level+1),
					locDriver);
		}
		SimulatedRobot simRobot= new SimulatedRobot(r, startingLocation);
		simRobot.setRobotComm(c);
		simRobot.setRobotVirtualRemoteCommDriver(newVirtualRemoteComm);
		simRobot.addParent(parent);
		for(int i= 0; i < numChildren; i++) {
			SimulatedRobot childBot= this.makeSimulatedRobot(simRobot, startingLocation, level+1, i, config, worldMap);
			simRobot.addChild(childBot);
			newVirtualRemoteComm.addChild(childBot.getRobotVirtualRemoteCommDriver());
		}
		this.guiBotToSimBot.put(new GUIRobot(r), simRobot);
		return simRobot;
	}
	
	private Coord calculateStartingLocation(Vector<Integer> config, int level, int childNum, Coord parentLoc) {
		int minBubbleSize= this.calcMinBubbleSize(config, level);
		int xLoc= minBubbleSize*childNum*2 + minBubbleSize+parentLoc.getX();
		return new Coord(xLoc, 0);
	}
	
	private int calcMinBubbleSize(Vector<Integer> config, int level) {
		int retVal= 20;
		for(int i= level; i < config.size() - 1; i++) {
			retVal*= 2*config.get(i+1);
		}
		return retVal;
	}
	
	private Vector<Integer> parseString(String s) {
		String[] splitString= s.split("[, ]");
		Vector<Integer> retVal= new Vector<Integer>();
		for(String intString : splitString) {
			retVal.add(new Integer(intString));
		}
		return retVal;
	}
	
	private void setRunComponentEnable(boolean enabled) {
		runButton.setEnabled(enabled);
		xSizeSpinner.setEnabled(enabled);
		ySizeSpinner.setEnabled(enabled);
	}

}
