package robots;


import simulation.VirtualLocomotionDriver;
import communication.Comm;
import communication.XbeeRemoteCommDriver;

import workerCode.ProbMap;
import workerCode.Queen;

import gnu.io.*;

public class VirtualQueen {
	
	public static final void main(String [] args){
		System.out.println("STARTING THE QUEEN");
		ProbMap queenMap;
		
		CommPortIdentifier xbeePortID= VirtualConfig.requestComPort(VirtualConfig.defaultQueenComm);
		
		XbeeRemoteCommDriver queenCommDriver = new XbeeRemoteCommDriver(xbeePortID);
		
		if(VirtualConfig.defaultQueenNumWorkers == 1){
			queenCommDriver.addAddressTableEntry(VirtualConfig.worker1XbeeAddress, 1);
		}
		else if (VirtualConfig.defaultQueenNumWorkers == 2){
			queenCommDriver.addAddressTableEntry(VirtualConfig.worker1XbeeAddress, 1);
			queenCommDriver.addAddressTableEntry(VirtualConfig.worker2XbeeAddress, 2);
		}
		else {
			System.out.println("Invalid Number of Workers");
		}
		
		queenMap = new ProbMap(25, 15, 10, ProbMap.NO_OFFSET);
		Comm queenComm = new Comm(queenCommDriver);
		Queen queen = new Queen(0, 0, queenComm, VirtualConfig.defaultQueenNumWorkers, new VirtualLocomotionDriver(VirtualConfig.defaultQueenPosition), queenMap);
		Thread queenThread = new Thread(queen);
		
		queenThread.start();
		
		System.out.println("THE QUEEN IS NOW RUNNING");

	}
	
	/**
	 * This is the generic version, it is a work in progress
	 */
//	//add a wait to the end of main to stop it from exiting?
//	Scanner scanner = new Scanner(System.in);
//	ArrayList<Long> ParentComms = new ArrayList<Long>();
//	ArrayList<Long> ChildComms = new ArrayList<Long>();
//	ProbMap queenMap;
//	
//	System.out.print("Welcome to the virtual robot generator. \n \n" +
//					 "You will be asked a variety of questions pertaining to starting a virtual robot. \n \n" +
//					 "The 'Parent' is the robot that directly controls this robot.  The 'child(ren)' are \n" +
//					 "robots directly controlled by this robot.  When entering information, please enter \n" +
//					 "S for Scout, W for Worker, and Q for Queen.  Example input for for the children of a \n" +
//					 "queen would be W1, W2.  Please seperate multiple robots by commas as shown, there is no \n" +
//					 "termination character needed. Enter 0 at any point if there are no children, or no parents. \n");
//	
//	//Get and parse input for the parents
//	System.out.print("Input the parents of this robot: ");
//	String parents = scanner.nextLine();
//	ParentComms = GeneralSwarmInformation.getRobotXbeeAddress(parents);
//	
//	//Get and parse input for the children
//	System.out.print("Input the children of this robot: ");
//	String children = scanner.nextLine();
//	ChildComms = GeneralSwarmInformation.getRobotXbeeAddress(children);	
//	
//	//Get the serial port ID
//	System.out.println("Input the serial Port ID that the XBee will us to communicate: ");
//	String serialID = scanner.nextLine();
//	
//	XbeeRemoteCommDriver queenCommDriver = new XbeeRemoteCommDriver();
//	//Add the parents (just in case)
//	queenCommDriver.addAddressTableEntry(ParentComms.get(0).longValue(), 0);
//	
//	//Add the children
//	for (int i = 0; i < ChildComms.size(); i++){
//		queenCommDriver.addAddressTableEntry(ChildComms.get(i), i + 1);
//	}
//	
//	System.out.println("Please enter the X Size of the ProbMap: ");
//	int xSize = scanner.nextInt();
//	
//	System.out.println("Please enter the Y Size of the ProbMap: ");
//	int ySize = scanner.nextInt();
//	
//	System.out.println("Please enter the cell size of the ProbMap: ");
//	int cellSize = scanner.nextInt();
//	
//	queenMap = new ProbMap(xSize, ySize, cellSize, ProbMap.NO_OFFSET);
//	
//	//Setup the comm
//	Comm queenComm = new Comm(queenCommDriver);
//	
//	//Setup the queen
//	Queen queen = new Queen(0, 0, queenComm, ChildComms.size(), new VirtualLocomotionDriver(new Coord(0,0)), queenMap);
//
//	//Setup the thread
//	Thread queenThread = new Thread(queen);
//	
//	//Start the thread
//	queenThread.start();
//	
//	System.out.println("The Queen is now running\n\n");
//	
//	System.out.println("Start Swarm? (Y/N): ");
//	String answer = scanner.nextLine();
//	if (answer == "Y"){
//		
//	}
}
