package robots;

import gnu.io.CommPortIdentifier;
import simulation.SimulationParameters;
import simulation.VirtualLocomotionDriver;
import workerCode.ProbMap;
import workerCode.Queen;
import workerCode.Worker;

import communication.Comm;
import communication.XbeeRemoteCommDriver;

public class VirtualSwarm {
	public static final void main(String [] args){
		
		/**
		 * START THE QUEEN
		 */
		System.out.println("STARTING THE QUEEN");
		ProbMap queenMap;
		
		CommPortIdentifier QueenXbeePortID= VirtualConfig.requestComPort(VirtualConfig.defaultQueenComm);
		
		XbeeRemoteCommDriver queenCommDriver = new XbeeRemoteCommDriver(QueenXbeePortID);
		
		if(VirtualConfig.defaultQueenNumWorkers == 1){
			queenCommDriver.addAddressTableEntry(VirtualConfig.worker1XbeeAddress, 1);
		}
		else if (VirtualConfig.defaultQueenNumWorkers == 2){
			queenCommDriver.addAddressTableEntry(VirtualConfig.worker1XbeeAddress, 1);
			queenCommDriver.addAddressTableEntry(VirtualConfig.worker2XbeeAddress, 2);
		}
		else {
			System.out.println("Invalid Number of Workers");
		}
		
		queenMap = new ProbMap(25, 15, 10, ProbMap.NO_OFFSET);
		Comm queenComm = new Comm(queenCommDriver);
		Queen queen = new Queen(0, 0, queenComm, VirtualConfig.defaultQueenNumWorkers, new VirtualLocomotionDriver(VirtualConfig.defaultQueenPosition), queenMap);
		Thread queenThread = new Thread(queen);
		
		queenThread.start();
		
		System.out.println("THE QUEEN IS NOW RUNNING");

		
		/**
		 * START WORKER 1
		 */
		System.out.println("STARTING WORKER 1");
		
		CommPortIdentifier worker1XbeePortID= VirtualConfig.requestComPort(VirtualConfig.defaultWorker1Comm);
		
		XbeeRemoteCommDriver worker1CommDriver = new XbeeRemoteCommDriver(worker1XbeePortID);
		
		//Parent
		worker1CommDriver.addAddressTableEntry(VirtualConfig.queen1XbeeAddress, 0);
		
		//Children
		if(VirtualConfig.defaultWorker1Scouts == 1){
			worker1CommDriver.addAddressTableEntry(VirtualConfig.scout1XbeeAddress, 1);
		}
		else if (VirtualConfig.defaultWorker1Scouts == 2){
			worker1CommDriver.addAddressTableEntry(VirtualConfig.scout1XbeeAddress, 1);
			worker1CommDriver.addAddressTableEntry(VirtualConfig.scout2XbeeAddress, 2);
		}
		else {
			System.out.println("Invalid Number of Scouts");
		}
		
		Comm worker1Comm = new Comm(worker1CommDriver);
		Worker worker1 = new Worker(Comm.MAX_RANGE, SimulationParameters.workerMinRadius, worker1Comm, VirtualConfig.defaultWorker1Scouts, new VirtualLocomotionDriver(VirtualConfig.defaultWorker1Position));
		Thread worker1Thread = new Thread(worker1);
		
		worker1Thread.start();
		
		System.out.println("WORKER 1 IS NOW RUNNING");
		
		/**
		 * START WORKER 2
		 */
		
		System.out.println("STARTING WORKER 2");
		
		CommPortIdentifier xbeePortID= VirtualConfig.requestComPort(VirtualConfig.defaultWorker2Comm);
		
		XbeeRemoteCommDriver worker2CommDriver = new XbeeRemoteCommDriver(xbeePortID);
		
		//Parent
		worker2CommDriver.addAddressTableEntry(VirtualConfig.queen1XbeeAddress, 0);
		
		//Children
		if(VirtualConfig.defaultWorker2Scouts == 1){
			worker2CommDriver.addAddressTableEntry(VirtualConfig.scout3XbeeAddress, 1);
		}
		else if (VirtualConfig.defaultWorker2Scouts == 2){
			worker2CommDriver.addAddressTableEntry(VirtualConfig.scout3XbeeAddress, 1);
			worker2CommDriver.addAddressTableEntry(VirtualConfig.scout4XbeeAddress, 2);
		}
		else {
			System.out.println("Invalid Number of Scouts");
		}
		
		Comm worker2Comm = new Comm(worker2CommDriver);
		Worker worker2 = new Worker(Comm.MAX_RANGE, SimulationParameters.workerMinRadius, worker2Comm, VirtualConfig.defaultWorker2Scouts, new VirtualLocomotionDriver(VirtualConfig.defaultWorker2Position));
		Thread worker2Thread = new Thread(worker2);
		
		worker2Thread.start();
		
		System.out.println("WORKER 2 IS NOW RUNNING");
	}
}
