package simulation;

import java.util.concurrent.CopyOnWriteArraySet;

import communication.AddPointToMapPacket;
import communication.ChildInformationPacket;
import communication.Comm;
import communication.ScanCompletedUnknownShadowingPacket;

import workerCode.Coord;
import workerCode.Locomotion;
import workerCode.ProbMap;
import workerCode.Robot;

public class Scout extends Robot {
	
	ProbMap virtualMap;

	public Scout(int scanMax, int scanMin, Comm robotComm,
			int numChildren, Locomotion robotLocomotion, ProbMap totalMap) {
		super(scanMax, scanMin, robotComm, numChildren, robotLocomotion);
		this.virtualMap= totalMap;
	}

	@Override
	protected void scan(int radius) {
		CopyOnWriteArraySet<Coord> blockedPointsSeen = new CopyOnWriteArraySet<Coord>();
		blockedPointsSeen = VirtualScanDriver.performScan(this.virtualMap, this.robotLocomotion.getCurrentLocation(), radius);
		
		for (Coord c : blockedPointsSeen){
//			System.out.println(this + " saw a blocked point at: " + c);
			this.robotComm.send(Comm.PARENT_ID, new AddPointToMapPacket(c, 1, false, false));
		}
		
		this.robotComm.send(Comm.PARENT_ID, new ScanCompletedUnknownShadowingPacket());
	}

	@Override
	protected void allChildrenReady() {
		this.robotComm.send(Comm.PARENT_ID, new ChildInformationPacket(this.robotLocomotion.getCurrentLocation(),
                this.scanRadiusMax, this.scanRadiusMin, false));
	}

}
