package simulation;

import java.util.ArrayList;
import java.util.Collection;

import workerCode.Coord;
import workerCode.ProbMap;
import workerCode.Queen;
import workerCode.Worker;

import communication.Comm;
import communication.VirtualRemoteCommDriver;

public class SimulationStarter {
	private Collection<SimulatedRobot> robots= new ArrayList<SimulatedRobot>();
	private SimulatedRobot simQueen;
	private ArrayList<SimulatedRobot> simWorkers= new ArrayList<SimulatedRobot>();
	private ArrayList<SimulatedRobot> simScouts= new ArrayList<SimulatedRobot>();
	
	private VirtualRemoteCommDriver queenCommDriver;
	private ArrayList<VirtualRemoteCommDriver> workerCommDrivers= new ArrayList<VirtualRemoteCommDriver>();
	private ArrayList<VirtualRemoteCommDriver> scoutCommDrivers= new ArrayList<VirtualRemoteCommDriver>();
	private ProbMap worldMap;
	
	public void addQueen(int numChildren) {
		this.queenCommDriver= new VirtualRemoteCommDriver();
		Comm c= new Comm(this.queenCommDriver);
		Queen q= new Queen(0, 0, c, numChildren,
				new VirtualLocomotionDriver(new Coord(-1,-1)),
				new ProbMap(worldMap.getxSize(), worldMap.getySize(), worldMap.getCellConversion(), ProbMap.NO_OFFSET));
		this.simQueen= new SimulatedRobot(q, new Coord(-1, -1));
		this.simQueen.setRobotComm(c);
		this.simQueen.setSwarmLevel(0);
		this.robots.add(this.simQueen);
	}
	
	public void addWorker(int numChildren, Coord startingLocation) {
		VirtualRemoteCommDriver driver= new VirtualRemoteCommDriver();
		driver.addToTable((byte)0, this.queenCommDriver);
		this.queenCommDriver.addChild(driver);
		this.workerCommDrivers.add(driver);
		Comm c= new Comm(driver);
		Worker w= new Worker(SimulationParameters.workerMaxRadius, SimulationParameters.scoutMinRadius * numChildren + (10 * numChildren + 10),
				c, numChildren, new VirtualLocomotionDriver(startingLocation));
		
		SimulatedRobot simWorker= new SimulatedRobot(w, startingLocation.deepCopy());
		simWorker.setRobotComm(c);
		simWorker.setSwarmLevel(1);
		this.simWorkers.add(simWorker);
		simWorker.addParent(this.simQueen);
		this.simQueen.addChild(simWorker);
		this.robots.add(simWorker);
	}
	public void addScout(int parentNumber, Coord startingLocation) {
		VirtualRemoteCommDriver driver= new VirtualRemoteCommDriver();
		driver.addParent(this.workerCommDrivers.get(parentNumber));
		this.workerCommDrivers.get(parentNumber).addChild(driver);
		this.scoutCommDrivers.add(driver);
		Comm c= new Comm(driver);
		Scout s= new Scout(SimulationParameters.scoutMaxRadius, SimulationParameters.scoutMinRadius,
				c, 0, new VirtualLocomotionDriver(startingLocation), worldMap);
		
		SimulatedRobot simScout= new SimulatedRobot(s, startingLocation.deepCopy());
		this.simScouts.add(simScout);
		simScout.addParent(this.simWorkers.get(parentNumber));
		simScout.setRobotComm(c);
		simScout.setSwarmLevel(2);
		this.simWorkers.get(parentNumber).addChild(simScout);
		this.robots.add(simScout);
		
	}
	public Collection<SimulatedRobot> setupSwarm(int numWorkers, ArrayList<Coord> workerStartingLocations,
			                                     int numScoutsPerWorker, ArrayList<Coord> scoutStartingLocations,
			                                     ProbMap map) {
		this.worldMap= map;
//		this.worldMap.testMakeBlockedAreas(5);
		
		//create queen
		this.addQueen(numWorkers);
		for(int i= 0; i < numWorkers; i++) {
			this.addWorker(numScoutsPerWorker, workerStartingLocations.get(i));
			for(int j= 0; j < numScoutsPerWorker; j++) {
				this.addScout(i, scoutStartingLocations.get(i*numScoutsPerWorker+j));
			}
		}		
		return this.robots;
	}
}
