package simulation;

import gui.MainGUI;
import workerCode.Coord;
import workerCode.Locomotion;

public class VirtualLocomotionDriver extends Locomotion {

	private Coord currentLocation;
	
	public static final double driveSpeed = 24.3; //The drive speed in cm / sec
	
	public VirtualLocomotionDriver(Coord startingLocation) {
		this.currentLocation= startingLocation;
	}

	@Override
	public boolean driveToLocation(Coord c) {
		
		try {
			Thread.sleep(this.calculateDriveTimeMS(c)/MainGUI.TIMESCALE);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		this.currentLocation = c.deepCopy();
		return true;
	}

	private int calculateDriveTimeMS(Coord c) {
		double distanceCM = Math.sqrt(Math.pow(c.getX() - this.getCurrentLocation().getX(), 2) +
								Math.pow(c.getY() - this.getCurrentLocation().getY(), 2));
		
		return (int)((distanceCM / VirtualLocomotionDriver.driveSpeed) * 1000);
	}

	@Override
	public Coord getCurrentLocation() {
		return this.currentLocation.deepCopy();
	}

}
