package simulation;

import gui.MainGUI;

import java.util.ArrayList;
import java.util.concurrent.CopyOnWriteArraySet;

import workerCode.Coord;
import workerCode.ProbMap;

public class VirtualScanDriver {

	public static final int servoSettleTime = 50;
	public static final int rangeFinderSettleTime = 50;
	public static final int numServos = 2;
	
	static public CopyOnWriteArraySet<Coord> performScan(ProbMap mapToScan, Coord currentLocation, int radius){
		int totalDelay = 3000 / MainGUI.TIMESCALE;
		try {
			Thread.sleep(totalDelay);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		CopyOnWriteArraySet<Coord> blockedList = new CopyOnWriteArraySet<Coord>();

		// Get the angular increment
		double angleStep = Math.atan2(mapToScan.getCellConversion(), radius);
		double angle = 0;
		Coord tempCoord = new Coord();
		
		/* Phase I: Detect shadows */
		ArrayList<Coord> intersectedPoints = new ArrayList<Coord>();
		while (angle < Math.PI * 2) { 
			tempCoord = new Coord((int)(currentLocation.getX() + radius*Math.cos(angle)), 
								  (int)(currentLocation.getY() + radius*Math.sin(angle)));
			intersectedPoints = mapToScan.raytrace(currentLocation, tempCoord);
			for(Coord c : intersectedPoints) {

				if(mapToScan.cellIsBlocked(c.scaleCoordDown(mapToScan.getCellConversion()))) {
//					System.out.println("Scout is at: " + currentLocation + " Saw a point at: " + c);
					blockedList.add(c); // C is in external
					break;
				}

			}

			angle += angleStep;
		}
		
		return blockedList;
		
		

	}
}
