	package virtualRemoteCommDriverTest;

	import java.util.Observable;
	import java.util.Observer;

	import org.junit.Test;
	import static org.junit.Assert.*;

	import workerCode.Coord;
	import workerCode.ProbMap;

	import communication.Comm;
	import communication.MapPacket;
	import communication.SwarmCommunication;
	import communication.VirtualRemoteCommDriver;

public class MapPacketTest implements Observer{
		
		ProbMap newMap;
		ProbMap oldMap = new ProbMap(2,2,10,ProbMap.NO_OFFSET);
		MapPacket packet = new MapPacket();
		
		@Test
		public void basicCommTest() {
			this.oldMap.cellScan(new Coord(0,1));
			this.oldMap.cellSetBlockValue(new Coord(0,1), .25);
			this.oldMap.cellScan(new Coord(1,1));
			this.oldMap.cellSetBlockValue(new Coord(1,1), .50);
			this.oldMap.cellScan(new Coord(1,0));
			this.oldMap.cellSetBlockValue(new Coord(1,0), .75);
			
			VirtualRemoteCommDriver parent= new VirtualRemoteCommDriver();
			VirtualRemoteCommDriver child= new VirtualRemoteCommDriver();
			parent.addChild(child);
			child.addParent(parent);
			Comm parentComm= new Comm(parent);
			Comm childComm= new Comm(child);
			childComm.addObserver(this);
			parentComm.send(1, new MapPacket(this.oldMap));
			
			synchronized(this){
				try {
					this.wait();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				assertTrue(this.newMap.cellIsScanned(new Coord(0,1)));
				assertTrue(this.newMap.cellIsScanned(new Coord(1,1)));
				assertTrue(this.newMap.cellIsScanned(new Coord(1,0)));
				assertFalse(this.newMap.cellIsScanned(new Coord(0,0)));
				
				assertEquals(.25, this.newMap.cellGetBlockValue(new Coord(0,1)), 0);
				assertEquals(.50, this.newMap.cellGetBlockValue(new Coord(1,1)), 0);
				assertEquals(.75, this.newMap.cellGetBlockValue(new Coord(1,0)), 0);
				assertEquals(0, this.newMap.cellGetBlockValue(new Coord(0,0)), 0);
			}
			
		}

		@Override
		public void update(Observable arg0, Object arg1) {
			
			if(arg1 instanceof SwarmCommunication) {
				
				synchronized(this){
					packet = (MapPacket)((SwarmCommunication)arg1).getPacket();
					this.newMap = packet.getMap();
					this.notify();
				}
			}
			
		}
	}

