package communication;

import workerCode.Coord;

public class AddPointToMapPacket extends SwarmPacket {

	private Coord loc;
	private int stdDev;
	private boolean beacon;
	private boolean candle;

	public AddPointToMapPacket(){
		//Blank By Design
	}
	
	public AddPointToMapPacket(Coord loc, int stdDev, boolean beacon, boolean candle){
		super(Comm.ADD_POINT_TO_MAP_OPCODE);
		this.loc = loc.deepCopy();
		this.stdDev = stdDev;
		this.beacon = beacon;
		this.candle = candle;
	}
	
	@Override
	public byte[] serializeData() {
		//the return value array
		byte[]retVal = new byte[7];
		
		//temp arrays for the various fields
		byte[] tempArrayX = new byte[2];
		byte[] tempArrayY = new byte[2];
		byte[] tempArrayStdDev = new byte[2];
	
		//Convert the numbers to their corresponding byte values
		tempArrayX = TypeConverter.shortToBytes((short)this.loc.getX());
		tempArrayY = TypeConverter.shortToBytes((short)this.loc.getY());
		tempArrayStdDev = TypeConverter.shortToBytes((short)this.stdDev);
		
		//Add the X
		retVal[0] = tempArrayX[0];
		retVal[1] = tempArrayX[1];
		
		//Add the Y
		retVal[2] = tempArrayY[0];
		retVal[3] = tempArrayY[1];
		
		//Add the stdDev
		retVal[4] = tempArrayStdDev[0];
		retVal[5] = tempArrayStdDev[1];
		
		retVal[6]= 0;
		
		if(this.beacon){
			retVal[6] |= (byte)1;
		}
		
		if(this.candle){
			retVal[6] |= (byte)2;
		}
		
		return retVal;
	}

	@Override
	public void deSerializeData(byte[] data) {
		int tempX;
		int tempY;
		
		//temp arrays for the various fields
		byte[] tempArrayX = new byte[2];
		byte[] tempArrayY = new byte[2];
		byte[] tempArrayStdDev = new byte[2];
		
		tempArrayX[0] = data[0];
		tempArrayX[1] = data[1];
		tempX = TypeConverter.bytesToShort(tempArrayX);
		
		tempArrayY[0] = data[2];
		tempArrayY[1] = data[3];
		tempY = TypeConverter.bytesToShort(tempArrayY);
		
		this.loc = new Coord(tempX, tempY);
		
		tempArrayStdDev[0] = data[4];
		tempArrayStdDev[1] = data[5];
		this.stdDev = TypeConverter.bytesToShort(tempArrayStdDev);
		
		if((data[6] & 0x01) != 0){
			this.beacon = true;
		}
		else this.beacon = false;
		
		if((data[6] & 0x02) != 0){
			this.candle = true;
		}
		else this.candle = false;
		
	}
	
	public Coord getLocation(){
		return this.loc;
	}
	
	public int getStdDev(){
		return this.stdDev;
	}
	
	
	public boolean isBeacon(){
		return this.beacon;
	}
	
	public boolean isCandle(){
		return this.candle;
	}

}
