package communication;

import workerCode.Coord;

public class ChildInformationPacket extends SwarmPacket {
	
	private Coord location = new Coord();
	private int scanRadiusMax;
	private int scanRadiusMin;
	private boolean requiresMap;
	
	public ChildInformationPacket(){
		super(Comm.CHILD_INFORMATION_REPLY_OPCODE);
	}
	
	public ChildInformationPacket(Coord location, int scanMax, int scanMin, boolean requiresMap){
		super(Comm.CHILD_INFORMATION_REPLY_OPCODE);
		this.location = location;
		this.scanRadiusMax = scanMax;
		this.scanRadiusMin = scanMin;
		this.requiresMap= requiresMap;
	}
	
	@Override
	protected void deSerializeData(byte[] data) {
		//make temporary array for conversions
		byte[] tempArray = new byte[2];
		
		//grab two bytes of x position		
		tempArray[0] = data[0];
		tempArray[1] = data[1];
		this.location.setX(TypeConverter.bytesToShort(tempArray));
		
		//grab two bytes of y position
		tempArray[0] = data[2];
		tempArray[1] = data[3];
		this.location.setY(TypeConverter.bytesToShort(tempArray));
		
		//convert two bytes to 16 bit integer
		tempArray[0] = data[4];
		tempArray[1] = data[5];
		this.scanRadiusMax = (TypeConverter.bytesToShort(tempArray));
		
		//convert two bytes to 16 bit integer
		tempArray[0] = data[6];
		tempArray[1] = data[7];
		this.scanRadiusMin = (TypeConverter.bytesToShort(tempArray));
		
		if(data[8] != 0) {
			this.requiresMap= true;
		}
		else {
			this.requiresMap= false;
		}
		
	}

	@Override
	protected byte[] serializeData() {
		//A Coord is stored as two 16-bit integers
		//Scan Max is a 16-bit integer
		//Scan Min is a 16-bit integer
		byte[] retVal = new byte[9];
		byte[] tempArray = new byte[2];
		
		//Grab the coord X value and store
		tempArray = TypeConverter.shortToBytes((short) this.location.getX());
		retVal[0]= tempArray[0];
		retVal[1]= tempArray[1];
		
		
		//Grab the coord Y value and store
		tempArray = TypeConverter.shortToBytes((short) this.location.getY());
		retVal[2]= tempArray[0];
		retVal[3]= tempArray[1];
		
		
		//Grab the scan Max and store
		tempArray = TypeConverter.shortToBytes((short) this.scanRadiusMax);
		retVal[4]= tempArray[0];
		retVal[5]= tempArray[1];
		
		
		//Grab the scan Min and store
		tempArray = TypeConverter.shortToBytes((short) this.scanRadiusMin);
		retVal[6]= tempArray[0];
		retVal[7]= tempArray[1];
		
		if(this.requiresMap== true) {
			retVal[8]= 1;
		}
		else {
			retVal[8]= 0;
		}
		
		
		return retVal;
	}

	public Coord getLocation() {
		return this.location;
	}

	public int getMaxScanRadius() {
		return this.scanRadiusMax;
	}

	public int getMinScanRadius() {
		return this.scanRadiusMin;
	}
	
	public boolean checkRequiresMap() {
		return this.requiresMap;
	}

}
