package communication;

import java.util.Observable;
import java.util.Observer;


public class Comm extends Observable implements Observer {
	
	
	//Ranges
	public static final int MAX_RANGE = 1000;
	
	//IDs
	public static final byte PARENT_ID= 0;
	
	//OpCodes
	public static final byte SCAN_PACKET_OPCODE= 0;
	public static final byte CHILD_INFORMATION_REPLY_OPCODE = 1;
	public static final byte REQUEST_INFORMATION_OPCODE = 2;
	public static final byte READY_OPCODE = 3;
	public static final byte NEW_PATH_OPCODE = 4;
	public static final byte REACHED_DESTINATION_OPCODE = 5;
	public static final byte MAP_PACKET_OPCODE = 6;
	public static final byte MAP_COMPLETED_SUCCESSFULLY_OPCODE = 7;
	public static final byte KILL_OPCODE= 8;
	public static final byte ADD_POINT_TO_MAP_OPCODE = 9;
	public static final byte SCAN_DONE_UNKNOWN_SHADOWING_OPCODE = 10;
	public static final byte SCAN_DONE_KNOWN_SHADOWING_OPCODE = 11;
	
	public static final byte SIMULATION_DONE_OPCODE = 127;
	public static final byte SIMULATION_SCAN_AREA_UPDATE_OPCODE = 126;
	
	RemoteCommDriver driver;
	
	public Comm(RemoteCommDriver driver) {
		this.driver= driver;
		this.driver.addObserver(this);
		
		Thread driverThread= new Thread(this.driver);
		
		driverThread.start();
	}
	
	public boolean send(int id, SwarmPacket packet) {
		
		return driver.send((byte)id, packet.serialize());
	}
	
	public boolean send(byte id, SwarmPacket packet) {
		return driver.send(id, packet.serialize());
	}
	
	public void kill() {
		this.driver.kill();
	}
	
	
	public void alertReceived(SwarmCommunication communication) {
		this.setChanged();
		this.notifyObservers(communication);
	}
	
	/*
	 * 	public static final byte SCAN_PACKET_OPCODE= 0;
	public static final byte CHILD_INFORMATION_REPLY_OPCODE = 1;
	public static final byte REQUEST_INFORMATION_OPCODE = 2;
	public static final byte READY_OPCODE = 3;
	public static final byte NEW_PATH_OPCODE = 4;
	public static final byte REACHED_DESTINATION_OPCODE = 5;
	public static final byte MAP_PACKET_OPCODE = 6;
	public static final byte MAP_COMPLETED_SUCCESSFULLY_OPCODE = 7;
	public static final byte KILL_OPCODE= 8;
	public static final byte ADD_POINT_TO_MAP = 9;
	public static final byte SCAN_DONE_UNKNOWN_SHADOWING = 10;
	public static final byte SCAN_DONE_KNOWN_SHADOWING = 11;(non-Javadoc)
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */

	@Override
	public void update(Observable o, Object arg) {
		if(arg instanceof RemoteCommDriver.ReceiveDataArray) {
			RemoteCommDriver.ReceiveDataArray rawPacket= (RemoteCommDriver.ReceiveDataArray)arg;
			SwarmPacket packet;
			switch(SwarmPacket.getOpcode(rawPacket.data)) {
				case SCAN_PACKET_OPCODE:
					packet= new ScanPacket();
					packet.deSerialize(rawPacket.data);
					break;
				case CHILD_INFORMATION_REPLY_OPCODE:
					packet = new ChildInformationPacket();
					break;
				case REQUEST_INFORMATION_OPCODE:
					packet = new RequestInformationPacket();
					break;
				case READY_OPCODE:
					packet = new ReadyPacket();
					break;
				case NEW_PATH_OPCODE:
					packet = new PathPacket();
					break;
				case REACHED_DESTINATION_OPCODE:
					packet = new ReachedDestinationPacket();
					break;
				case MAP_PACKET_OPCODE:
					packet = new MapPacket();
					break;
				case MAP_COMPLETED_SUCCESSFULLY_OPCODE:
					packet = new ScanCompletedKnownShadowingPacket();
					break;
				case SIMULATION_DONE_OPCODE:
					packet = new SimulationDonePacket();
					break;
				case SIMULATION_SCAN_AREA_UPDATE_OPCODE:
					packet = new SimulationScanAreaUpdatedPacket();
					break;
				case KILL_OPCODE:
					packet= new KillPacket();
					break;
				case ADD_POINT_TO_MAP_OPCODE:
					packet = new AddPointToMapPacket();
					break;
				case SCAN_DONE_UNKNOWN_SHADOWING_OPCODE:
					packet = new ScanCompletedUnknownShadowingPacket();
					break;
				case SCAN_DONE_KNOWN_SHADOWING_OPCODE:
					packet = new ScanCompletedKnownShadowingPacket();
					break;
				default:
					packet= null;
					break;
			}
			if(packet!= null) {
				packet.deSerialize(rawPacket.data);
				SwarmCommunication communication= new SwarmCommunication(rawPacket.id, packet);
				this.alertReceived(communication);
			}
		}
	}
	
	
}
