package communication;

import workerCode.Coord;
import workerCode.ProbMap;

public class MapPacket extends SwarmPacket {
	
	private ProbMap map;
	
	public MapPacket(){
		//Blank by Design
	}
	
	public MapPacket(ProbMap mapToSend) {
		super(Comm.MAP_PACKET_OPCODE);
		this.map = mapToSend;
	}

	@Override
	protected byte[] serializeData() {
		
		byte[] tempArray = new byte[2];
		
		byte[] retVal = new byte[10 + this.map.getxSize() * this.map.getySize()];
		
		//First add in the x size of the map
		tempArray = TypeConverter.shortToBytes((short) this.map.getxSize());
		retVal[0] = tempArray[0];
		retVal[1] = tempArray[1];
		
		//Add in the y size of the map
		tempArray = TypeConverter.shortToBytes((short) this.map.getySize());
		retVal[2] = tempArray[0];
		retVal[3] = tempArray[1];
		
		//Add in the x offset of the map
		tempArray = TypeConverter.shortToBytes((short) this.map.getXOffset());
		retVal[4] = tempArray[0];
		retVal[5] = tempArray[1];
		
		//Add in the y offset of the map
		tempArray = TypeConverter.shortToBytes((short) this.map.getYOffset());
		retVal[6] = tempArray[0];
		retVal[7] = tempArray[1];
		
		//Get the cell conversion
		tempArray = TypeConverter.shortToBytes((short) this.map.getCellConversion());
		retVal[8] = tempArray[0];
		retVal[9] = tempArray[1];
		
		//Add in the blockValues of the cells
		for (int x = 0; x < this.map.getxSize(); x++){
			for(int y = 0; y < this.map.getySize(); y++){
				Coord tempCoord = new Coord(x + this.map.getXOffset(), y + this.map.getYOffset());
				if (this.map.cellIsScanned(tempCoord)){
					retVal[10 + x * this.map.getySize() + y] = 
						(byte)(this.map.cellGetBlockValue(tempCoord) * 100);
				}
				
				else {
					retVal[10 + x * this.map.getySize() + y] = (byte)127;
				}
			}
		}
		
		return retVal;
	}

	@Override
	protected void deSerializeData(byte[] data) {

		byte[] tempArray = new byte[2];
		
		int tempX;
		int tempY;
		int tempXOffset;
		int tempYOffset;
		int tempCellConversion;
		
		//Get the xSize of the map
		tempArray[0] = data[0];
		tempArray[1] = data[1];
		tempX = TypeConverter.bytesToShort(tempArray);
		
		//Get the ySize of the map
		tempArray[0] = data[2];
		tempArray[1] = data[3];
		tempY = TypeConverter.bytesToShort(tempArray);
		
		//Get the xOffset of the map
		tempArray[0] = data[4];
		tempArray[1] = data[5];
		tempXOffset = TypeConverter.bytesToShort(tempArray);
		
		//Get the yOffset of the map
		tempArray[0] = data[6];
		tempArray[1] = data[7];
		tempYOffset = TypeConverter.bytesToShort(tempArray);
		
		//Get the cellConversion of the map
		tempArray[0] = data[8];
		tempArray[1] = data[9];
		tempCellConversion = TypeConverter.bytesToShort(tempArray);
		
		this.map = new ProbMap(tempX, tempY, tempCellConversion, new Coord(tempXOffset, tempYOffset));

		//Add in the blockValues of the cells
		for (int x = 0; x < this.map.getxSize(); x++){
			for(int y = 0; y < this.map.getySize(); y++){
				Coord tempCoord = new Coord(x + this.map.getXOffset(), y + this.map.getYOffset());
				byte tempBlock = data[10 + x * this.map.getySize() + y];
				
				if (tempBlock != (byte)127){
					this.map.cellScan(tempCoord);
					this.map.cellSetBlockValue(tempCoord, ((double)tempBlock) / 100);
				}

			}
		}
	}
	
	public ProbMap getMap(){
		return this.map;
	}

}
