package communication;

import java.util.ArrayList;
import java.util.Collection;

import workerCode.Coord;

public class ScanCompletedKnownShadowingPacket extends SwarmPacket {
	private ArrayList<Coord> shadowedPoints = new ArrayList<Coord>();

	public ScanCompletedKnownShadowingPacket(){
		super(Comm.SCAN_DONE_KNOWN_SHADOWING_OPCODE);
	}
	
	public ScanCompletedKnownShadowingPacket(ArrayList<Coord> shadowList){
		super(Comm.SCAN_DONE_KNOWN_SHADOWING_OPCODE);
		this.shadowedPoints = shadowList;
	}
	
	@Override
	public void deSerializeData(byte[] data) {
		byte[] tempArrayX = new byte[2];
		byte[] tempArrayY = new byte[2];

		for (int i = 0; i < data.length/4; i++){
			this.shadowedPoints.add(new Coord());
		}
		
		for (int i = 0; i < data.length/4; i++){
			tempArrayX[0] = data[(i * 4 + 0)];
			tempArrayX[1] = data[(i * 4 + 1)];
			tempArrayY[0] = data[(i * 4 + 2)];
			tempArrayY[1] = data[(i * 4 + 3)];
			
			int tempX = TypeConverter.bytesToShort(tempArrayX);
			int tempY = TypeConverter.bytesToShort(tempArrayY);
			this.shadowedPoints.set(i, new Coord(tempX, tempY));
		}
			
	}

	@Override
	public byte[] serializeData() {
		byte[] tempArrayX = new byte[2];
		byte[] tempArrayY = new byte[2];
		
		byte[] retVal = new byte[this.shadowedPoints.size() * 4];
		
		
		for (int i = 0; i<this.shadowedPoints.size(); i++){
			
			// Retrieve the coordinates as data bytes
			tempArrayX = TypeConverter.shortToBytes((short)this.shadowedPoints.get(i).getX());
			tempArrayY = TypeConverter.shortToBytes((short)this.shadowedPoints.get(i).getY());
			
			// Add them to the array of bytes
			retVal[(i * 4 + 0)] = tempArrayX[0];
			retVal[(i * 4 + 1)] = tempArrayX[1];
			retVal[(i * 4 + 2)] = tempArrayY[0];
			retVal[(i * 4 + 3)] = tempArrayY[1];
		}
		
		return retVal;
	}
	
	public Collection<Coord> getShadowedPoints() {
		return this.shadowedPoints;
	}

}
