package communication;

public class TxReqAPI extends XbeeAPIFrame {
	private byte frameID;
	private long addr64bit;
	private short addr16bit;
	private byte broadcastRadius;
	private byte options;
	private byte[] payload;
	
	
	public TxReqAPI() {
		super(XbeeAPIFrame.TX_REQ);
	}
	
	public TxReqAPI(int frameID, long addr64bit, int addr16bit,
			        int broadcastRadius, int options, byte[] payload) {
		super(XbeeAPIFrame.TX_REQ);
		this.frameID= (byte)frameID;
		this.addr64bit= addr64bit;
		this.addr16bit= (short)addr16bit;
		this.broadcastRadius= (byte)broadcastRadius;
		this.options= (byte)options;
		this.payload= payload;
		
	}

	@Override
	protected byte[] serializeFrame() {
		byte[] tempBuf;
		byte[] retVal= new byte[13 + this.payload.length];
		
		//add frame ID
		retVal[0]= (byte) frameID;
		
		//serialize 64 bit address
		tempBuf= TypeConverter.longToBytes(this.addr64bit);
		retVal[1]= tempBuf[0];
		retVal[2]= tempBuf[1];
		retVal[3]= tempBuf[2];
		retVal[4]= tempBuf[3];
		retVal[5]= tempBuf[4];
		retVal[6]= tempBuf[5];
		retVal[7]= tempBuf[6];
		retVal[8]= tempBuf[7];
		
		//serialize 16 bit address
		tempBuf= TypeConverter.shortToBytes((short)this.addr16bit);
		retVal[9]= tempBuf[0];
		retVal[10]= tempBuf[1];
		
		//serialize broadcast radius
		retVal[11]= (byte)this.broadcastRadius;
		
		//serialize options
		retVal[12]= (byte)this.options;
		
		//add in the payload
		for(int i= 0; i < this.payload.length; i++) {
			retVal[13 + i]= this.payload[i];
		}
		return retVal;
	}

	@Override
	protected void deSerializeFrame(byte[] serialData) {
		byte[] tempBuf;
		
		//pull out the frame id
		this.frameID= serialData[0];
		
		//pull out 64 bit address
		tempBuf= new byte[8];
		tempBuf[0]= serialData[1];
		tempBuf[1]= serialData[2];
		tempBuf[2]= serialData[3];
		tempBuf[3]= serialData[4];
		tempBuf[4]= serialData[5];
		tempBuf[5]= serialData[6];
		tempBuf[6]= serialData[7];
		tempBuf[7]= serialData[8];
		this.addr64bit= TypeConverter.bytesToLong(tempBuf);
		
		//pull out 16 bit address
		tempBuf= new byte[2];
		tempBuf[0]= serialData[9];
		tempBuf[1]= serialData[10];
		this.addr16bit= TypeConverter.bytesToShort(tempBuf);
		
		//get broadcast radius
		this.broadcastRadius= serialData[11];
		
		//get options
		this.options= serialData[12];
		
		//get payload
		this.payload= new byte[serialData.length - 13];
		for(int i= 0; i < this.payload.length; i++) {
			this.payload[i]= serialData[i+13];
		}
	}

	/**
	 * @param frameID the frameID to set
	 */
	public void setFrameID(int frameID) {
		this.frameID = (byte)frameID;
	}

	/**
	 * @return the frameID
	 */
	public byte getFrameID() {
		return frameID;
	}

	/**
	 * @param addr64bit the addr64bit to set
	 */
	public void setAddr64bit(long addr64bit) {
		this.addr64bit = addr64bit;
	}

	/**
	 * @return the addr64bit
	 */
	public long getAddr64bit() {
		return addr64bit;
	}

	/**
	 * @param addr16bit the addr16bit to set
	 */
	public void setAddr16bit(int addr16bit) {
		this.addr16bit = (short)addr16bit;
	}

	/**
	 * @return the addr16bit
	 */
	public short getAddr16bit() {
		return addr16bit;
	}

	/**
	 * @param broadcastRadius the broadcastRadius to set
	 */
	public void setBroadcastRadius(int broadcastRadius) {
		this.broadcastRadius = (byte)broadcastRadius;
	}

	/**
	 * @return the broadcastRadius
	 */
	public byte getBroadcastRadius() {
		return broadcastRadius;
	}

	/**
	 * @param options the options to set
	 */
	public void setOptions(int options) {
		this.options = (byte)options;
	}

	/**
	 * @return the options
	 */
	public byte getOptions() {
		return options;
	}

	/**
	 * @param payload the payload to set
	 */
	public void setPayload(byte[] payload) {
		this.payload = payload;
	}

	/**
	 * @return the payload
	 */
	public byte[] getPayload() {
		return payload;
	}
	
	
}
