package communication;

public abstract class XbeeAPIFrame {
	private int frameType;
	
	//FRAME ID's
	public static int AT_COMMAND= 0x08;
	public static int TX_REQ= 0x10; 
	public static int AT_COMMAND_RESPONSE= 0x88;
	public static int MODEM_STATUS= 0x8A;
	public static int TX_STATUS= 0x8B;
	public static int RX_PACKET= 0x90;
	
	//Start delim
	public static int XBEE_START_DELIM= 0x7E;
	
	
	public XbeeAPIFrame(byte frameType) {
		this.setFrameType(frameType);
	}
	
	public XbeeAPIFrame(int frameType) {
		this.setFrameType(frameType);
	}
	
	protected abstract byte[] serializeFrame();
	
	protected abstract void deSerializeFrame(byte[] serialData);
	
	public byte[] serialize() {
		byte[] frameData= this.serializeFrame();
		byte[] totalFrame= new byte[frameData.length + 5];
		byte[] tempBuf;
		
		totalFrame[0]= (byte) XBEE_START_DELIM;
		
		//calculate frame length.  Note that this isn't the same as the total
		//number of bytes in the frame but is the amount of bytes between
		//the length bytes and the checksum
		int frameLength= totalFrame.length - 4;
		tempBuf= TypeConverter.shortToBytes((short)frameLength);
		totalFrame[1]= tempBuf[0];
		totalFrame[2]= tempBuf[1];
		
		//add frame type
		totalFrame[3]= (byte)this.getFrameType();
		
		//now append the frame specific data
		for(int i= 0; i < frameData.length; i++) {
			totalFrame[i+4]= frameData[i];
		}
		
		//calculate the checksum
		totalFrame[totalFrame.length - 1]= calculateChecksum(totalFrame);
		
		return totalFrame;
	}
	
	public void deSerialize(byte[] serialData) {
		//set frame type
		this.frameType= serialData[3];
		
		//first copy out the frame specific stuff
		byte[] frameData= new byte[serialData.length - 5];
		
		//copy bytes into frameData
		for(int i= 0; i < frameData.length; i++) {
			frameData[i]= serialData[i+4];
		}
		
		this.deSerializeFrame(frameData);
	}
	
	/**
	 * Calculate the checksum of a serial API frame
	 * @param serialFrame The frame to calculate the checksum of.
	 * @return The expected checksum value for the given frame.
	 */
	private byte calculateChecksum(byte[] serialFrame) {
		int retVal= 0;
		int frameLength;
		byte[] tempBuf= new byte[2];
		tempBuf[0]= serialFrame[1];
		tempBuf[1]= serialFrame[2];
		frameLength= TypeConverter.bytesToShort(tempBuf);
		for(int i= 3; i < frameLength + 3; i++) {
			retVal+= serialFrame[i];
		}
		//remove all but last byte
		retVal&= 0xFF;
		
		return (byte)(0xFF - retVal);
	}

	/**
	 * @param frameType the frameType to set
	 */
	public void setFrameType(int frameType) {
		this.frameType = frameType;
	}

	/**
	 * @return the frameType
	 */
	public int getFrameType() {
		return frameType;
	}
	
}
