package dijkstra;

import java.util.ArrayList;

import workerCode.Coord;

public class DijkstraAlgorithm {
	ArrayList<Coord> unvisitedNodes;
	
	public DijkstraAlgorithm() {
		this.unvisitedNodes= new ArrayList<Coord>();
	}
	
	/**
	 * Run Dijkstra's algorithm on a given map.  This WILL modify the nodes
	 * in the map so the user should perform a deep copy before handing it to
	 * the run method if the user wants to preserve the original state of the map.
	 * @param map The map to run Dijkstra's algorithm on.
	 * @param startingNode The starting location.  The starting location must be
	 *                     a reference to a node that resides in map.
	 */
	public void run(DijkstraCapableMap map, Coord startingNode) {
		//set up the map for running dijkstra's
		map.initDijkstra(startingNode);
		this.unvisitedNodes.clear();
		//add the starting node to the priority queue.
		this.unvisitedNodes.add(startingNode);
		//while there are still unvisited nodes
		while(!unvisitedNodes.isEmpty()) {
			
			//grab the head of the queue and remove it from the queue at the same time
			Coord currentNode= unvisitedNodes.remove(0);
					
			//grab all neighbors
			for(Coord neighbor : map.getNeighbors(currentNode)) {
				
				//if not visited yet
				if(map.getNode(neighbor).isVisited() == false) {
					//add it to the list of unvisited nodes if it isn't in there already
					if(!this.unvisitedNodes.contains(neighbor)) {
						unvisitedNodes.add(neighbor);
					}
					//if the current node provides a shorter path than the prevoiusly
					//recorded best
					if(map.getNode(neighbor).getDistance() > (map.getNode(currentNode).getDistance() + map.getEdgeCost(currentNode, neighbor))) {
						//update the best found distance in the neighbor and remember that
						//the best was found by visiting this node.
						map.getNode(neighbor).setDistance(map.getNode(currentNode).getDistance() + map.getEdgeCost(currentNode, neighbor));
						map.getNode(neighbor).setPrevoiusNode(currentNode);
					}
				}
			}
			//now that we checked all the neighbors set the current node as visited
			map.getNode(currentNode).setVisited(true);
		}
		//all done!
	}
	
}