package dijkstra;

import java.util.Collection;

import workerCode.Coord;

public interface DijkstraCapableMap {
	/**
	 * Get all the neighbors of the given node.
	 * @param node The node whose neighbors need to be found.
	 * @return An array of all nodes adjacent to the given node.
	 */
	public Collection<Coord> getNeighbors(Coord node);
	
	public DijkstraNode getNode(Coord node);
	
	/**
	 * Get the cost of traveling to an adjacent node.
	 * @return The cost of traveling to an adjacent node.
	 */
	public int getEdgeCost(Coord node0, Coord node1);
	
	/**
	 * Set all nodes except the starting node to have a distance of infinity.
	 * The starting node should have a distance of 0.
	 * @param startingNode The node to start with.
	 */
	public void initDijkstra(Coord startingNode);
}
