package dijkstra;

import workerCode.Coord;

public class DijkstraNode {
	private int distance;
	private boolean visited;
	private Coord previousNode;
	
	/**
	 * Default constructor.  Sets the initial distance to positive infinity and
	 * visited to false.
	 */
	public DijkstraNode() {
		this.distance= Integer.MAX_VALUE;
		this.visited= false;
		this.previousNode= new Coord();
	}
	
	/**
	 * Create a DijkstraNode with the given distance associated with it.
	 * @param distance The initial distance to this node.
	 */
	public DijkstraNode(int distance) {
		this.distance= distance;
		this.visited= false;
		this.previousNode= new Coord();
	}
	
	/**
	 * Get the distance associated with this Dijkstra node.
	 * @return The shortest known distance from the starting point to this node
	 */
	public int getDistance() {
		return this.distance;
	}
	
	/**
	 * Set the distance to this node from the starting location.
	 * @param distance The new value for the distance to this node.
	 */
	public void setDistance(int distance) {
		this.distance= distance;
	}
	
	/**
	 * Check if this node has been visited.
	 * @return true if visited, false if unvisited.
	 */
	public boolean isVisited() {
		return this.visited;
	}
	
	/**
	 * Set this node's visited status.
	 * @param b True to make visited, false to make unvisited.
	 */
	public void setVisited(boolean b) {
		this.visited= b;
	}
	
	/**
	 * Get the previous node in the path to get to this one.
	 * @return The node before this one in the optimal path from the starting
	 *         location.
	 */
	public Coord getPreviousNode() {
		return this.previousNode;
	}
	
	/**
	 * Set the previous node in the path to get to this one.  This should be called
	 * whenever the shortest distance to this node is updated.
	 * @param previousNode The node to travel through to get to this current node.
	 */
	public void setPrevoiusNode(Coord previousNode) {
		this.previousNode= previousNode;
	}
	
	/**
	 * Reset this node to have an infinite distance, no previous node, and to
	 * be unvisited.
	 */
	public void reset() {
		this.distance= Integer.MAX_VALUE;
		this.previousNode= new Coord();
		this.visited= false;
	}
}
