package workerCode;

import dijkstra.DijkstraNode;
/**
 * Main cell class used for storing map data
 */

/**
 * @author hsrmqp
 *
 */
public class Cell extends DijkstraNode{
	
	private boolean scanned;  //Whether or not the cell has been scanned
	private double blockVal; //The block value of the cell
	private int timesScanned; //The number of times the cell has been scanned
	private boolean inBounds; //Whether or not the cell should ever be touched
	private boolean occupied; //cell is temporarily "untouchable" can't drive or scan here.

	/**
	 * Default constructor
	 */
	public Cell() {
		this.scanned = false;
		this.blockVal = 0;
		this.setTimesScanned(0);
		this.inBounds = true;
		this.occupied= false;
	}
	
	/**
	 * This method scans the cell (sets 'scanned' to true)
	 */
	public void scan() {
		this.scanned = true;
	}
	
	/**
	 * This method un-scans the cell (sets 'scanned' to false)
	 */
	public void unscan() {
		this.scanned = false;
	}

	/**
	 * This method returns whether or not the cell has been scanned
	 *
	 * @return boolean whether or not the cell has been scanned
	 */
	public boolean isScanned() {
		return scanned;
	}

	/**
	 * This method sets the blocked value of the cell
	 *
	 * @param newBlockVal the new blocked value of the cell (0 - 100)
	 */
	public void setBlockVal(double newBlockVal) {
		this.blockVal = newBlockVal;
	}

	/**
	 * This method gets the blocked value of the cell
	 *
	 * @return double the blocked value of the cell (0 - 100)
	 */
	public double getBlockVal() {
		return blockVal;
	}

	/**
	 * This method sets the number of times scanned
	 * @param timesScanned
	 */
	public void setTimesScanned(int timesScanned) {
		this.timesScanned = timesScanned;
	}

	/**
	 * This method gets the number of times a cell has been scanned
	 * @return
	 */
	public int getTimesScanned() {
		return timesScanned;
	}
	
	/**
	 * Increments the number of times the cell has been scanned by 1
	 */
	public void incrementScanned(){
		this.timesScanned++;
	}

	public void setInBounds(boolean inBounds) {
		this.inBounds = inBounds;
	}

	public boolean isInBounds() {
		return inBounds;
	}
	
	public void setOccupied(boolean occupied) {
		this.occupied= occupied;
	}
	
	public boolean isOccupied() {
		return this.occupied;
	}
	
	/**
	 * Apply a given probability to this cell.
	 * If the current probability that a cell is blocked is A.  This function
	 * changes the probability that the cell is blocked to be A & prob or
	 * A= A+prob - A*prob.  The two exceptions to this is that if the probability
	 * given is that the cell is blocked is zero it will not use the formula
	 * and instead set the probability that the cell is blocked to be zero.
	 * Also, if the current probability that a cell is blocked is zero and
	 * the cell has been scanned it will remain at zero.
	 * @param prob The most recently observed probability that a given
	 * cell is blocked.
	 */
	public void applyProbability(double prob) {
		if(prob == 0) {
			this.blockVal= 0;
		}
		else if((this.blockVal == 0) && (this.scanned == true)) {
			return;
		}
		else {
			double newBlockVal= this.blockVal + prob - this.blockVal*prob;
			this.blockVal= newBlockVal;
		}
	}

}
