package workerCode;
/**
 * 
 */

/**
 * @author hsrmqp
 *
 */
public class Coord {
	
	private int x; //X position
	private int y; //Y Position

	/**
	 * Default Constructor for an invalid coord
	 */
	public Coord(){
		this.setX(-1);
		this.setY(-1);
	}
	
	/**
	 * Default constructor for setting the x and y values of a coord
	 * 
	 * @param int initX the x value used to initialize
	 * @param int initY the y value used to initialize
	 */
	public Coord(int initX, int initY) {
		this.setX(initX);
		this.setY(initY);
	}

	/**
	 * Set the x value
	 * @param newX
	 */
	public void setX(int newX) {
		this.x = newX;
	}

	/**
	 * Gets the x value
	 * @return int x
	 */
	public int getX() {
		return x;
	}

	/**
	 * Sets the y value
	 * @param newY
	 */
	public void setY(int newY) {
		this.y = newY;
	}

	/**
	 * Gets the y value
	 * @return int y
	 */
	public int getY() {
		return y;
	}
	
	/**
	 * Returns true if the coord is valid
	 * 
	 * This method will return true if the x and y values are both >= 0
	 * @return
	 */
	public boolean isValid(){
		if (this.x < 0 || this.y < 0){
			return false;
		}
		return true;
	}
	
	/**
	 * Returns a deep copy of the coord
	 * @return
	 */
	public Coord deepCopy(){
		return new Coord(this.x, this.y);
	}
	
	/**
	 * determines if two coords are equal based of their x and y values
	 * 
	 * @param otherCoord
	 * @return true if the coords are the same
	 */
	@Override
	public boolean equals(Object other){
		if(other instanceof Coord) {
			Coord otherCoord= (Coord)other;
			if (this.x == otherCoord.getX() && this.y == otherCoord.getY()){
				return true;
			}
		}
		
		return false;
	}
	
	/**Scaled a Coord to internal units
	 * 
	 */
	public Coord scaleCoordDown(int scale){
		return new Coord(this.x / scale, this.y / scale);
	}
	
	/**
	 * Scaled a Coord up to external units
	 */
	public Coord scaleCoordUp(int scale){
		return new Coord((this.x * scale), (this.y * scale));
	}
	
	/**
	 * Returns the distance of the Coord
	 */
	public double distance (Coord c){
		return Math.sqrt(Math.pow(this.getX()-c.getX(), 2) + Math.pow(this.getY() - c.getY(), 2));
	}
	
	/**
	 * Returns true if the distance is greater than a given number
	 */
	public boolean distanceGreaterThan(Coord c, double cmpDist){
		double dist = this.distance(c);
		if(dist > cmpDist){
			return true;
		}
		else{
			return false;
		}
	}
	
	/**
	 * Returns true if the distance is greater than or equal to a number
	 */
	public boolean distanceGreaterThanOrEqual(Coord c, double cmpDist){
		double dist = this.distance(c);
		if(dist >= cmpDist){
			return true;
		}
		else{
			return false;
		}
	}
	
	/**
	 * Returns true if the distance is less than or equal to a given number
	 */
	public boolean distanceLessThanOrEqual(Coord c, double cmpDist){
		double dist = this.distance(c);
		if(dist <= cmpDist){
			return true;
		}
		else{
			return false;
		}
	}
	
	/**
	 * toString method
	 */
	@Override
	public String toString() {
		return new String("("+this.x+","+this.y+")");
	}

}
