package workerCode;

import java.util.List;


public abstract class Locomotion {
	
	/**
	 * Tells the robot to drive a path
	 * 
	 * Does so by calling driveToLocation repeatedly on a path.
	 * 
	 * A path is a list of coords in external units
	 */
	public boolean drivePath(List<Coord> path) {
		for(Coord waypoint : path) {
			if(driveToLocation(waypoint) == false) return false;
		}
		
		return true;
	}
	
	/**
	 * Drive to the given absolute coordinates in cm.  This function will block
	 * until the robot either reaches the end coordinates or until some error
	 * forces the robot to stop early.  If the robot does reach the final
	 * coordinates, this function will return false.
	 * @param c The coordinate to drive to in cm.
	 * @return True if the robot arrived at the given location.
	 */
	abstract public boolean driveToLocation(Coord c);
	
	/**
	 * Obtain the current absolute position of the robot (in cm).
	 * @return The coordinates of the robot.
	 */
	abstract public Coord getCurrentLocation();
}
