package workerCode;

import java.util.ArrayList;
import java.util.concurrent.CopyOnWriteArraySet;

public class PathPlanner {
	
	ArrayList<Child> childrenToPath = new ArrayList<Child>();
	ArrayList<Coord> currentConflics = new ArrayList<Coord>();
	ProbMap masterMap;
	private int currentWave = 1;
	
	public void makeAndOptimizePath(Child c, ProbMap parentMap){
		this.masterMap = parentMap;
		
//		System.out.println("Child " + c.getID() + " Current Location: " + c.getCurrentLocation());
		this.generateInitialPath(c); //Set the path
		this.convertPathToExternalUnits(c); //Convert it to external units
		this.optimizeFinalPath(c); //Optimize the path
		c.setPathListMade(false);//Set that we have NOT made a list of all the points in the child's path
		c.setWave(0);	
		
	}
	
	public void makeAndOptimizePaths(ArrayList<Child> childList, ProbMap parentMap){
		this.masterMap = parentMap;
		this.childrenToPath = this.getChildListToUse(childList);
		
		//First, for each child generate a path for the child and optimize it
		for (Child c : childrenToPath){ //For each child
//			System.out.println("Child " + c.getID() + " Current Location: " + c.getCurrentLocation());
			this.generateInitialPath(c); //Set the path
			this.convertPathToExternalUnits(c); //Convert it to external units
			this.optimizeFinalPath(c); //Optimize the path
			c.setPathListMade(false);//Set that we have NOT made a list of all the points in the child's path
			c.setWave(0);	
		}
	}
	
	public void assignWaves(ArrayList<Child> childList, ProbMap parentMap){
		this.masterMap = parentMap;
		this.childrenToPath = this.getChildListToUse(childList);
		
		//This will set the current wave to the current max wave of a child
		this.currentWave = findStartWave(childrenToPath);
		
		//Icrement the wave so that the next child placed will automatically be in the next wave
		this.currentWave++;
		
		//Now assign a wave to each child
		while(!this.allChildrenHaveWave()){
			//For each child that needs to be pathed
			for (Child c : this.childrenToPath){
				
				//If the child does not already have a wave number
				if (c.getWave() == 0){
					//Get a list of all points the child's path crosses through
					if (!c.isPathListMade()){
						this.getListPathPoints(c, this.masterMap);
						c.setPathListMade(true);
					}
					
					//If there are no conflicts with other paths
					if(!this.pathHasConflicts(c.getAllPointsInPath())){
						c.setWave(currentWave);
						this.setConflics(c.getAllPointsInPath());
					}
					
				} //End of if wave == 0
				
			} //End of for each child to path
			
			if (!this.allChildrenHaveWave()){
				currentWave++;
				this.currentConflics.clear();
			}
		} // End of while not all children have a wave
	} //End of assignWaves

	
	/**
	 * This method takes in a child list and returns the maximum wave in the child's list
	 * @param childrenToPath
	 * @return
	 */
	private int findStartWave(ArrayList<Child> childrenToPath) {
		int tempMaxWave = 0;
		
		for (Child c : childrenToPath){
			if (c.getWave() > tempMaxWave){
				tempMaxWave = c.getWave();
			}
		}
		
		return tempMaxWave;
	}	


	private void setConflics(ArrayList<Coord> allPointsInPath) {
		for (Coord c1 : allPointsInPath){
			this.currentConflics.add(c1);
		}	
	}

	private boolean pathHasConflicts(ArrayList<Coord> allPointsInPath) {
		
		//For each point in the current conflics
		for (Coord c1 : this.currentConflics){
			
			//For each point in the path
			for (Coord c2 : allPointsInPath){
				
				//If c1 == c2
				if (c1.equals(c2)){
					return true; // return true
				}
			}
		}
		return false; //If we never returned true, then return false
	}

	public void getListPathPoints(Child c, ProbMap map) {
		
		c.getAllPointsInPath().clear(); //Clear the List 
		ArrayList<Coord> tempListOfAllPoints = new ArrayList<Coord>(); //List of all points in the path
		ArrayList<Coord> tempShortList = new ArrayList<Coord>(); //List of points between two consecutive points in the path
//		ArrayList<Coord> tempShortListCopy = new ArrayList<Coord>();
		Coord pos1;
		Coord pos2;
		//For each point in the path
		for (int i = 1; i < c.getFinalPath().size(); i++){

			//Get the first position
			pos1 = c.getFinalPath().get(i - 1); //The previous point in the path
			
			//Get the current point as the 2nd position
			pos2 = c.getFinalPath().get(i);
			
			//The short list of points from the raytrace
			//TODO remove
//			System.out.println("Checking Child " + c + "'s path for intersected points from "
//								+ pos1.toString() + " to " + pos2.toString());
//			System.out.println("Calling From getListChid ... " + pos1);
			tempShortList = map.raytraceOutsideMap(pos1, pos2);
//			tempShortListCopy.addAll(tempShortList);
			tempListOfAllPoints.addAll(tempShortList);
			
//			for (Coord e : tempShortListCopy){
//				tempShortList.addAll(this.masterMap.getRobotBodyCells(e));
//			}
			
			//For each point in the short list, add it to the master list
//			tempListOfAllPoints.addAll(tempShortList);
			
			//Clear the short list
			tempShortList.clear();
		
		}
		
		//Set the child so that its list of all points are the ones determined
		c.getAllPointsInPath().addAll(tempListOfAllPoints);
	}
	
	public void getListPathAndRobotPoints(Child c, ProbMap map) {
		
		c.getAllPointsInPath().clear(); //Clear the List 
		ArrayList<Coord> tempListOfAllPoints = new ArrayList<Coord>(); //List of all points in the path
		ArrayList<Coord> tempShortList = new ArrayList<Coord>(); //List of points between two consecutive points in the path
		ArrayList<Coord> tempShortListCopy = new ArrayList<Coord>();
		Coord pos1;
		Coord pos2;
		//For each point in the path
		for (int i = 1; i < c.getFinalPath().size(); i++){

			//Get the first position
			pos1 = c.getFinalPath().get(i - 1); //The previous point in the path
			
			//Get the current point as the 2nd position
			pos2 = c.getFinalPath().get(i);
			
			//The short list of points from the raytrace
			//TODO remove
//			System.out.println("Checking Child " + c + "'s path for intersected points from "
//								+ pos1.toString() + " to " + pos2.toString());
//			System.out.println("Calling From getListChid ... " + pos1);
			tempShortList = map.raytraceOutsideMap(pos1, pos2);
			
			tempShortListCopy.addAll(tempShortList);
			
//			tempListOfAllPoints.addAll(tempShortList);
			
			for (Coord e : tempShortListCopy){
				tempShortList.addAll(this.masterMap.getRobotBodyCells(e));
			}
			
			//For each point in the short list, add it to the master list
			tempListOfAllPoints.addAll(tempShortList);
			
			//Clear the short list
			tempShortList.clear();
		
		}
		
		//Set the child so that its list of all points are the ones determined
		c.getAllPointsInPath().addAll(tempListOfAllPoints);
	}

	private ArrayList<Child> getChildListToUse(ArrayList<Child> childList) {
		ArrayList<Child> tempChildList = new ArrayList<Child>();

		for (Child c : childList){
			if (c.isMove()){
				tempChildList.add(c);
			}
		}
		
		return tempChildList;
	}
	
	

	/**
	 * This method will return:
	 * 
	 * TRUE if every child has a wave that is not zero
	 * FALSE if any child has a wave that is zero
	 * @param childList
	 * @return
	 */
	private boolean allChildrenHaveWave(){
		//For each child
		for (Child c : this.childrenToPath){
			if (c.getWave() == 0){ //If the wave is zero,
				return false;      //Return false
			}
		}
		
		return true; //If this point is reached every child has a wave, return true
	}

	private void convertPathToExternalUnits(Child c) {
		//Convert the path to external units
		for (int i = 0; i < c.getFinalPath().size(); i++){
			Coord tempCoord3 = c.getFinalPath().get(i).deepCopy();
			tempCoord3.setX((tempCoord3.getX() * c.getDijkstraMap().getCellConversion()) + (c.getDijkstraMap().getCellConversion() / 2));
			tempCoord3.setY((tempCoord3.getY() * c.getDijkstraMap().getCellConversion()) + (c.getDijkstraMap().getCellConversion() / 2));
			c.getFinalPath().set(i, tempCoord3.deepCopy());
		}	
	}

	private void optimizeFinalPath(Child c) {
		if(c.getFinalPath().size() <= 2){
			return;
		}
		ArrayList<Coord> tempPath = c.getFinalPath();
		ArrayList<Coord> intersectedPoints = new ArrayList<Coord>();
		int i = 0;
		boolean canSkip;
		
		// Index is size - 1; Subtract 2 to jump to next-next point
		while(i <= tempPath.size() - 3){
			canSkip = true;
			
			intersectedPoints = c.getDijkstraMap().raytraceOutsideMap(tempPath.get(i),
															tempPath.get(i+2));
			for(Coord loc : intersectedPoints){
				if(!c.getDijkstraMap().cellIsDrivable(loc.scaleCoordDown(this.masterMap.getCellConversion()))){
					i++;
					canSkip = false;
					break;
				}
			}
			if(canSkip) {
				tempPath.remove(i+1);
			}
		}
	}

	// TODO change back to private
	public void generateInitialPath(Child c) {
		
		//Clear the path to make sure its empty
		c.getFinalPath().clear();
		
		//Start with the child's next location
		Coord tempCoord = c.getNextLocation().deepCopy();
		
//		System.out.println("X: " + c.getDijkstraMap().getXBoundMin() + " to " + c.getDijkstraMap().getXBoundMax() + " \n" +
//						   "Y: " + c.getDijkstraMap().getYBoundMin() + " to " + c.getDijkstraMap().getYBoundMax() + " \n" + 
//						   " Current Location: " + c.getCurrentLocation() + " TempCord = " + tempCoord);
		
		//If its next location is valid, reachable, and not its current location
		if(tempCoord.isValid() && 
		   c.getDijkstraMap().cellIsReachable(tempCoord) &&
		   !tempCoord.equals(c.getCurrentLocation())){
		
			//Add as many elements into the final path = to the distance to the next location
			for (int i = 0; i <= c.getDijkstraMap().cellGetDistance(tempCoord); i++){
				c.getFinalPath().add(new Coord());
			}
			
			//Starting at the last location, step backwards through the map until you hit the start location
			for (int element = c.getDijkstraMap().cellGetDistance(tempCoord); element >= 0; element--){
				c.getFinalPath().set(element, tempCoord.deepCopy());
				tempCoord = c.getDijkstraMap().cellGetPrevious(tempCoord).deepCopy();
			}
			
		}
		
	}
	
	public void addToPath(ArrayList<Coord> originalPath, ArrayList<Coord> pathToAdd){
		for (Coord c : pathToAdd){
			originalPath.add(c);
		}
	}

	public void findNewEndpointDistanceFromEnd(Child c, int distanceIncrement,
			ProbMap map) {
		if (!c.getFinalPath().isEmpty()) {

			Coord tempFinalPoint = c.getFinalPath().get(
					c.getFinalPath().size() - 1);
			Coord newFinalPoint;
			Coord tempCoordToRemove = new Coord();
			boolean removePoint = false;

			// First make a list of all points that the child's path passes
			// through
			this.getListPathPoints(c, map);

			// Now, starting at the end, find the point closest to the end that
			// is distanceIncrement away
			for (int i = c.getAllPointsInPath().size() - 1; i >= 0; i--) {

				removePoint = false;

				// For each point in the child's current final path
				for (Coord e : c.getFinalPath()) {

					// If that point matches the current point we are looking at
					// in the list of all points
					if (e.equals(c.getAllPointsInPath().get(i))) {

						// System.out.println(e + " to " +
						// c.getAllPointsInPath().get(i) + " i = " + i);
						// Save the point so it can be removed
						tempCoordToRemove = e;
						removePoint = true;
					}
				}

				// Remove the point
				if (removePoint) {
					c.getFinalPath().remove(tempCoordToRemove);
				}

				// See if this point is far enough from the end points
				if (tempFinalPoint.distanceGreaterThanOrEqual(c
						.getAllPointsInPath().get(i), distanceIncrement)) {
					newFinalPoint = c.getAllPointsInPath().get(i);
					c.getFinalPath().add(newFinalPoint);
					i = -1;
				}
			}
			
			//FIXME
//			if (c.getFinalPath().size() != 0){
				c.setNextLocation(c.getFinalPath().get(c.getFinalPath().size() - 1));
//			}
//			else {
//				c.setNextLocation(c.getCurrentLocation());
//			}

		}

	}
	
}
