package workerCode;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Random;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;
import java.util.concurrent.CopyOnWriteArraySet;

import org.apache.commons.math.MathException;
import org.apache.commons.math.special.Erf;

import dijkstra.DijkstraCapableMap;
import dijkstra.DijkstraNode;

/**
 * 
 */

/**
 * @author hsrmqp
 *
 */
public class ProbMap implements DijkstraCapableMap {
	
	private int xSize; //The x size of the map
	private int ySize; //The y size of the map
	private int cellConversion; //The conversion factor to external units
	private Vector<Vector<Cell>> theMap = new Vector<Vector<Cell>>(); //The map itself
	private final static double ROOT2OVER2 = 0.707106781;
	private int xOffset;
	private int yOffset;
	public static final double BLOCKED_THRESHOLD= .25;
	
	public static final Coord NO_OFFSET = new Coord(0,0);

	/**
	 * Default map constructor
	 */
	public ProbMap(int width, int height, int cellDimension, Coord offset) {
		this.xSize = width;
		this.ySize = height;
		this.cellConversion = cellDimension;
		this.xOffset = offset.getX();
		this.yOffset = offset.getY();
		
		for (int i = 0; i < width; i++){
			theMap.add(new Vector<Cell>());
		}
		
		//Add Cells
		for (int x = 0; x < width; x++){
			for(int y = 0; y < height; y++){
				theMap.get(x).add(new Cell());
			}
		}
		
	}
	
	/**
	 * Constructor used by Dijkstra's to simultaneously make a new map and reset the dijkstra's information
	 * @param mapToCopy
	 */
	public ProbMap(ProbMap mapToCopy){
		this.xSize = mapToCopy.getxSize();
		this.ySize = mapToCopy.getySize();
		this.cellConversion = mapToCopy.getCellConversion();
		this.xOffset = mapToCopy.getXOffset();
		this.yOffset = mapToCopy.getYOffset();
		
		for (int i = 0; i < this.xSize; i++){
			theMap.add(new Vector<Cell>());
		}
		
		for (int x = 0; x < this.xSize; x++){
			for(int y = 0; y < this.ySize; y++){
				theMap.get(x).add(new Cell());
				Coord tempCoord = new Coord(x + this.xOffset,y + this.yOffset);
				this.cellSetScan(tempCoord, mapToCopy.cellIsScanned(tempCoord));
				this.cellSetBlockValue(tempCoord, mapToCopy.cellGetBlockValue(tempCoord));
				this.cellSetTimesScanned(tempCoord, mapToCopy.cellGetTimesScanned(tempCoord));
				this.cellSetInBounds(tempCoord, mapToCopy.cellIsInBounds(tempCoord));
			}
		}
	}
	
	/**
	 * Constructor for making a new Map that is a sub map of another map
	 * 
	 * Given two coordinates it will make the new map
	 */
	public ProbMap(ProbMap mapToCopy, Coord bottomLeft, Coord topRight){
		this.xSize = topRight.getX() - bottomLeft.getX() + 1;
		this.ySize = topRight.getY() - bottomLeft.getY() + 1;
		this.cellConversion = mapToCopy.getCellConversion();
		this.xOffset = bottomLeft.getX();
		this.yOffset = bottomLeft.getY();
		
		for (int i = 0; i < this.xSize; i++){
			theMap.add(new Vector<Cell>());
		}
		
		for (int x = 0; x < this.xSize; x++){
			for(int y = 0; y < this.ySize; y++){
				theMap.get(x).add(new Cell());
				Coord tempCoord = new Coord(x + getXOffset(),y + getYOffset());      
					
				this.cellSetScan(tempCoord, mapToCopy.cellIsScanned(tempCoord));
				this.cellSetBlockValue(tempCoord, mapToCopy.cellGetBlockValue(tempCoord));
				this.cellSetTimesScanned(tempCoord, mapToCopy.cellGetTimesScanned(tempCoord));
				this.cellSetInBounds(tempCoord, mapToCopy.cellIsInBounds(tempCoord));
			}
		}
	}

	/**
	 * Gets the x dimension of the map
	 * @return
	 */
	public int getxSize() {
		return xSize;
	}

	/**
	 * Gets the y dimension of the map
	 * @return
	 */
	public int getySize() {
		return ySize;
	}
	
	/**
	 * Gets the x offset of the map
	 */
	public int getXOffset(){
		return this.xOffset;
	}
	
	/**
	 * Gets the y offset of the map
	 */
	public int getYOffset(){
		return this.yOffset;
	}

	/**
	 * Gets the cell conversion 'constant' (to convert from cells to cm)
	 * @return
	 */
	public int getCellConversion() {
		return cellConversion;
	}
	
	public int getXBoundMin(){
		return this.xOffset;
	}
	
	/**
	 * Get one more than the accessible x bound.
	 * @return
	 */
	public int getXBoundMax(){
		return this.xSize + this.xOffset;
	}
	
	public int getYBoundMin(){
		return this.yOffset;
	}
	
	/**
	 * Get one more than the maximum accessible y bound.
	 * @return
	 */
	public int getYBoundMax(){
		return this.ySize + this.yOffset;
	}
	
	/**
	 * Gets the DijkstraCell at the given coordinate. 
	 * 
	 * Unlike getDCell, this returns a Cell, masking the extra interfaces of
	 * the DijkstraCell
	 * @param coordinate
	 * @return Cell
	 */
	public Cell getCell(Coord coordinate) {
		return theMap.get(coordinate.getX() - this.xOffset).get(coordinate.getY() - this.yOffset);
	}
	
	/**
	 * Method to get the distance between two coordinates
	 * 
	 * Computes the direct linear distance between two coordinates in the
	 * map in centimeters.
	 * @param pos1
	 * @param pos2
	 * @return double distance
	 */
	public double distance(Coord pos1, Coord pos2) {
		return Math.sqrt(Math.pow(pos1.getX()-pos2.getX(), 2) + Math.pow(pos1.getY() - pos2.getY(), 2));
	}
	
	/**
	 * Returns true/false if distance is greater than given value.
	 * 
	 * Calculates the distance between the two given coordinates and returns
	 * whether or not it is greater than the given distance in centimeters.
	 * @param pos1
	 * @param pos2
	 * @param cmpDist
	 * @return boolean
	 */ 
	public boolean distanceGreaterThan(Coord pos1, Coord pos2, double cmpDist) {
		double dist = this.distance(pos1, pos2);
		if(dist > cmpDist){
			return true;
		}
		else{
			return false;
		}
	}
	
	/**
	 * Returns true/false if distance is less than given value.
	 * 
	 * Calculates the distance between the two given coordinates and returns
	 * whether or not it is less than the given distance in centimeters.
	 * @param pos1
	 * @param pos2
	 * @param cmpDist
	 * @return boolean
	 */
	public boolean distanceLessThan(Coord pos1, Coord pos2, double cmpDist) {
		
		double dist = this.distance(pos1, pos2);
		if(dist < cmpDist){
			return true;
		}
		else{
			return false;
		}
	}
	
	/**
	 * Returns true/false if distance is less than or equal to given value.
	 * 
	 * Calculates the distance between the two given coordinates and returns
	 * whether or not it is less than the given distance in centimeters.
	 * @param pos1
	 * @param pos2
	 * @param cmpDist
	 * @return boolean
	 */
	public boolean distanceLessThanOrEqual(Coord pos1, Coord pos2, double cmpDist) {
		
		double dist = this.distance(pos1, pos2);
		if(dist <= cmpDist){
			return true;
		}
		else{
			return false;
		}
	}
	
	/**
	 * Returns the coordinate above the given one
	 * 
	 * Returns a coordinate with a y-value one greater than the one given.
	 * Error checking returns a coordinate of (-1, -1) if CUp would be
	 * out of bounds on the map.
	 * @param coordinate
	 * @return Coord
	 */
	public Coord CUp(Coord coordinate) {
		if(coordinate.getY() >= this.getYBoundMax() - 1){
			return new Coord();
		}
		else{
			return new Coord(coordinate.getX(), coordinate.getY()+1);
		}
	}
	
	/**
	 * Returns true if CUp would be in bounds of the Map
	 * @param coordinate
	 * @return boolean
	 */
	public boolean CUpIsInTheMap(Coord coordinate) {
		if(coordinate.getY() >= this.getYBoundMax() - 1){
			return false;
		}
		else{
			return true;
		}
	}
	
	/**
	 * Returns the coordinate below the given one
	 * 
	 * Returns a coordinate with a y-value one less than the one given.
	 * Error checking returns a coordinate of (-1, -1) if CDown would be
	 * out of bounds on the map.
	 * @param coordinate
	 * @return Coord
	 */
	public Coord CDown(Coord coordinate) {
		if(coordinate.getY() <= this.getYBoundMin()){
			return new Coord();
		}
		else{
			return new Coord(coordinate.getX(), coordinate.getY()-1);
		}
	}
	
	/**
	 * Returns true if CDown would be in bounds of the Map
	 * @param coordinate
	 * @return boolean
	 */
	public boolean CDownIsInTheMap(Coord coordinate) {
		if(coordinate.getY() <= this.getYBoundMin()){
			return false;
		}
		else{
			return true;
		}
	}
	
	/**
	 * Returns the coordinate to the right of the given one
	 * 
	 * Returns a coordinate with a x-value one greater than the one given.
	 * Error checking returns a coordinate of (-1, -1) if CRight would be
	 * out of bounds on the map.
	 * @param coordinate
	 * @return Coord
	 */
	public Coord CRight(Coord coordinate) {
		if(coordinate.getX() >= this.getXBoundMax() - 1){
			return new Coord();
		}
		else{
			return new Coord(coordinate.getX()+1, coordinate.getY());
		}
	}
	
	/**
	 * Returns true if CRight would be in bounds of the Map
	 * @param coordinate
	 * @return boolean
	 */
	public boolean CRightIsInTheMap(Coord coordinate) {
		if(coordinate.getX() >= this.getXBoundMax() - 1){
			return false;
		}
		else{
			return true;
		}
	}
	
	/**
	 * Returns the coordinate to the left of the given one
	 * 
	 * Returns a coordinate with a x-value one less than the one given.
	 * Error checking returns a coordinate of (-1, -1) if CLeft would be
	 * out of bounds on the map.
	 * @param coordinate
	 * @return Coord
	 */
	public Coord CLeft(Coord coordinate) {
		if(coordinate.getX() <= this.getXBoundMin()){
			return new Coord();
		}
		else{
			return new Coord(coordinate.getX()-1, coordinate.getY());
		}
	}
	
	/**
	 * Returns true if CLeft would be in bounds of the Map
	 * @param coordinate
	 * @return boolean
	 */
	public boolean CLeftIsInTheMap(Coord coordinate) {
		if(coordinate.getX() <= this.getXBoundMin()){
			return false;
		}
		else{
			return true;
		}
	}
	
	public Coord CUpRight(Coord coordinate) {
		if(coordinate.getX() >= this.getXBoundMax()-1 || coordinate.getY() >= this.getYBoundMax()-1){
			return new Coord();
		}
		else{
			return new Coord(coordinate.getX()+1, coordinate.getY()+1);
		}
	}
	
	public boolean CUpRightIsInTheMap(Coord coordinate) {
		if(coordinate.getX() >= (this.getXBoundMax() - 1) || coordinate.getY() >= (this.getYBoundMax() - 1)){
			return false;
		}
		else {
			return true;
		}
	}
	
	public Coord CUpLeft(Coord coordinate) {
		if(coordinate.getX() <= this.getXBoundMin() || coordinate.getY() >= this.getYBoundMax()-1) {
			return new Coord();
		}
		else {
			return new Coord(coordinate.getX()-1, coordinate.getY()+1);
		}
	}
	
	public boolean CUpLeftIsInTheMap(Coord coordinate) {
		if(coordinate.getX() <= this.getXBoundMin() || coordinate.getY() >= this.getYBoundMax()-1) {
			return false;
		}
		else {
			return true;
		}
	}
	
	public Coord CDownRight(Coord coordinate) {
		if(coordinate.getX() >= this.getXBoundMax()-1 || coordinate.getY() <= this.getYBoundMin()) {
			return new Coord();
		}
		else {
			return new Coord(coordinate.getX() +1, coordinate.getY() - 1);
		}
	}
	
	public boolean CDownRightIsInTheMap(Coord coordinate) {
		if(coordinate.getX() >= this.getXBoundMax()-1 || coordinate.getY() <= this.getYBoundMin()) {
			return false;
		}
		else {
			return true;
		}
	}
	
	public Coord CDownLeft(Coord coordinate) {
		if(coordinate.getX() <= this.getXBoundMin() || coordinate.getY() <= this.getYBoundMin()) {
			return new Coord();
		}
		else {
			return new Coord(coordinate.getX() - 1, coordinate.getY() - 1);
		}
	}
	
	public boolean CDownLeftIsInTheMap(Coord coordinate) {
		if(coordinate.getX() <= this.getXBoundMin() || coordinate.getY() <= this.getYBoundMin()) {
			return false;
		}
		else {
			return true;
		}
	}
	
	/**
	 * Checks to see if the cell is a valid point to scan.
	 * 
	 * Method that checks to see if a cell at the given coordinate is "valid". 
	 * The criteria are:
	 * 		- the cell must be scanned
	 * 		- at least one of its neighbors must be unscanned
	 * @param coordinate
	 * @return boolean
	 */
	public boolean isCellValid(Coord coordinate) {
		if(this.cellIsDrivable(coordinate) && this.cellIsInBounds(coordinate)){
			if(this.CUpIsInTheMap(coordinate)){
				if(!this.cellIsScanned(CUp(coordinate)) && this.cellIsInBounds(CUp(coordinate))){
					return true;
				}
			}
			if(this.CDownIsInTheMap(coordinate)){
				if(!this.cellIsScanned(CDown(coordinate)) && this.cellIsInBounds(CDown(coordinate))){
					return true;
				}
			}
			if(this.CRightIsInTheMap(coordinate)){
				if(!this.cellIsScanned(CRight(coordinate)) && this.cellIsInBounds(CRight(coordinate))){
					return true;
				}
			}
			if(this.CLeftIsInTheMap(coordinate)){
				if(!this.cellIsScanned(CLeft(coordinate)) && this.cellIsInBounds(CLeft(coordinate))){
					return true;
				}
			}
			return false;
		}
		else{
			return false;
		}
	}
	
	/**
	 * Returns the cell value at the given coordinate
	 * 
	 * @param coordinate The coordinate to check at
	 * @param radius the radius to check within
	 * @return int the cells value
	 */
	public int computeCellsUncovered(Coord coordinate, int radius) {
		int cellsUncovered=0;
		double innerOffset = ProbMap.ROOT2OVER2 * radius;
		
		//Determine the interanl offsets in order to reduce floating point calculations
		double XMinInner = (coordinate.getX() - innerOffset);
		double XMaxInner = (coordinate.getX() + innerOffset);
		double YMinInner = (coordinate.getY() - innerOffset);
		double YMaxInner = (coordinate.getY() + innerOffset);
		
		//Set the x min and max values in order to reduce number of calculations
		int Xmin = ((coordinate.getX() - radius) < this.getXBoundMin() ? getXBoundMin() : (coordinate.getX() - radius));
		int Xmax = ((coordinate.getX() + radius) >= this.getXBoundMax() ? getXBoundMax() - 1 : (coordinate.getX() + radius));
		
		int Ymin = ((coordinate.getY() - radius) < getYBoundMin() ? getYBoundMin() : (coordinate.getY() - radius));	
		int Ymax = ((coordinate.getY() + radius) >= getYBoundMax() ? getYBoundMax() - 1 : (coordinate.getY() + radius));
		
		//Run through between the x and y min and max values of the map
		for (int x = Xmin; x <= Xmax; x++){
			for(int y = Ymin; y <= Ymax; y++){
				
				//If the cell is unscanned
				if (!this.cellIsScanned(new Coord(x,y)) && this.cellIsInBounds(new Coord(x,y))){
				
					//If it is inside the innfer offsets, immediately add it to the number of cells uncovered
					if (x > XMinInner && x < XMaxInner && y > YMinInner && y < YMaxInner){
						cellsUncovered++;}
					
					//Otherwise calculate if it is within the radius, and if it is, add it 
					else if (this.distanceLessThanOrEqual(coordinate, new Coord(x,y), radius)) {
						cellsUncovered++;}
					
				}
				
			}
		}
		
		return cellsUncovered; //Return the number of cells uncovered
	}
	
	/**
	 * Sets an area of cells to have new blocked values given a distribution
	 * @param coordinate
	 * @param newBlockValue
	 * @param distribution
	 */
	public void updateCellAreaBlockValue(Coord objectPosExternal, int distribution) {
		//build a list of all cells within 3 standard deviations of the object
		//position

		Coord objectPosInternal = objectPosExternal.scaleCoordDown(this.cellConversion);
		for(int i= this.xOffset; i < this.xSize+this.xOffset; i++) {
			for(int j= this.yOffset; j < this.ySize+this.yOffset; j++) {
				Coord c= new Coord(i,j);
				//scale c to external units
				Coord cExt= new Coord(c.getX()*this.cellConversion + this.cellConversion/2, 
						              c.getY()*this.cellConversion + this.cellConversion/2);
				//exclude points which are over 3 standard deviations away.
				//We would like to include ANY cell which has even a portion of it
				//that is within 3 standard deviations.  In order to account for this,
				//the distance comparison is done in internal units so that the
				//distance is comparing integral numbers corresponding to whole cells
				//and the ceiling function is used to round up the distribution to
				//the nearest integer.  Therefore, if cell 5,5 is compared with 5,6
				//and 3*distribution= .3 (in internal units) the if statement will 
				//evaluate as true because .3 is rounded to 1.  Another way of
				//looking at this is that there is at least one point in 5,5 that
				//is .3 internal units away from at least one point in 5,6
				if(this.distanceLessThanOrEqual(objectPosInternal, c, Math.ceil(3*(double)distribution/this.cellConversion))) {
					double blockedValue= this.calculateCellBlockProbability(objectPosExternal, cExt, distribution);
					//apply probability
					this.applyProbability(c, blockedValue);
				}
			}
		}
		//done
	}
	
	/**
	 * Similar to the shadowing function, simply traces each angular increment
	 * to find all points possibly visible to the robot at externalCoord.
	 * @param externalCoord
	 * @param tempRadius
	 * @return
	 */
	public Collection<Coord> findVisiblePointsInBubble(Coord externalCoord, int tempRadius) {
		// Get the angular increment
		double angleStep = Math.atan2(this.cellConversion, tempRadius);
		double angle = 0;
		CopyOnWriteArraySet<Coord> fullCoordList = new CopyOnWriteArraySet<Coord>();
		Coord tempCoord = new Coord();
		
		ArrayList<Coord> intersectedPoints = new ArrayList<Coord>();
		while (angle < Math.PI * 2) { 
			tempCoord = new Coord((int)(externalCoord.getX() + tempRadius*Math.cos(angle)),
								  (int)(externalCoord.getY() + tempRadius*Math.sin(angle)));
			
			intersectedPoints = this.raytrace(externalCoord, tempCoord);
			for(Coord c : intersectedPoints) {
				fullCoordList.add(c);
				if(this.cellIsBlocked(c.scaleCoordDown(this.cellConversion))) {
					break;
				}

			}
			angle += angleStep;
		}
		return fullCoordList;
	}
	
	/**
	 * A simple method that returns a collection of coordinates that correspond
	 * to the bubble described by the parameters. Will not return coordinates
	 * outside of the map.
	 * @param externalCoord
	 * @param tempRadius
	 * @return
	 */
	public Collection<Coord> returnPointsInBubble(Coord externalCoord, int tempRadius) {
		// Get the angular increment
		double angleStep = Math.atan2(this.cellConversion, tempRadius);
		double angle = 0;
		CopyOnWriteArraySet<Coord> fullCoordList = new CopyOnWriteArraySet<Coord>();
		Coord tempCoord = new Coord();
		
		ArrayList<Coord> intersectedPoints = new ArrayList<Coord>();
		while (angle < Math.PI * 2) { 
			tempCoord = new Coord((int)(externalCoord.getX() + tempRadius*Math.cos(angle)),
								  (int)(externalCoord.getY() + tempRadius*Math.sin(angle)));
			
			intersectedPoints = this.raytrace(externalCoord, tempCoord);
			for(Coord c : intersectedPoints) {
				fullCoordList.add(c);
			}
			angle += angleStep;
		}
		return fullCoordList;
	}	
	
	/**
	 * Expand a given collection of coords in external units by the size of
	 * a robot.  In other words, this function returns a set of coordinates
	 * which if a robot were placed at would overlap at least one cell in
	 * the given input collection.
	 * @param externalCoords
	 * @return
	 */
	public Set<Coord> expandArea(Collection<Coord> externalCoords) {
		Set<Coord> retVal= new CopyOnWriteArraySet<Coord>();
		
		for(Coord c : externalCoords) {
			retVal.addAll(this.getRobotBodyCells(c));
		}
		
		return retVal;
	}
	
	public void updateCompletedScanAreaUnknownShadowingSimulation(Coord externalCoord, int tempRadius,
			Coord parent, int parentBubble) {
		
		// Get the angular increment
		double angleStep = Math.atan2(this.cellConversion, tempRadius);
		double angle = 0;
		CopyOnWriteArraySet<Coord> unmodifiedCoords = new CopyOnWriteArraySet<Coord>();
		Coord tempCoord = new Coord();
		boolean shadowed = false;
		
		/* Phase I: Detect shadows */
		ArrayList<Coord> intersectedPoints = new ArrayList<Coord>();
		while (angle < Math.PI * 2) { 
			tempCoord = new Coord((int)(externalCoord.getX() + tempRadius*Math.cos(angle)),
								  (int)(externalCoord.getY() + tempRadius*Math.sin(angle)));
			
			intersectedPoints = this.raytraceShadowing(externalCoord, tempCoord);
			for(Coord c : intersectedPoints) {
				if(shadowed) {
					if(!unmodifiedCoords.contains(c.scaleCoordDown(this.cellConversion))) {
						unmodifiedCoords.add(c.scaleCoordDown(this.cellConversion));
					}
				}

				if(this.cellIsBlocked(c.scaleCoordDown(this.cellConversion))) {
					shadowed = true;
					if(unmodifiedCoords.contains(c.scaleCoordDown(this.cellConversion))) {
						unmodifiedCoords.remove(c.scaleCoordDown(this.cellConversion));
					}
				}

			}
			shadowed = false;
			angle += angleStep;
		}
		
		/*Phase II: update scanned area*/
		ArrayList<Coord> unscannedList = new ArrayList<Coord>();
		for(Coord c : unmodifiedCoords) {
			unscannedList.add(c);
		}
		
		//trick simulation into thinking that all points outside the bubble are "in shadow"
		for(int x= 0; x < this.xSize; x++) {
			for(int y= 0; y < this.ySize; y++) {
				Coord fakeShadow= new Coord(x, y);
				if(fakeShadow.distanceGreaterThan(parent.scaleCoordDown(this.cellConversion),
						((double)parentBubble)/this.cellConversion)) {
					if(!unscannedList.contains(fakeShadow)) {
						unscannedList.add(fakeShadow);
					}
				}
			}
		}
		this.updateCompletedScanAreaUnseenSpaces(externalCoord, tempRadius, (ArrayList<Coord>) unscannedList);
	}
	
	/**
	 * Method for the worker (or more specifically one level up from scouts)
	 * to update the map without being given any shadowing or unknown space
	 * information. Generates shadowed list of cells using raytracing and
	 * leaves them unscanned.
	 *  
	 * @param externalCoord
	 * @param tempRadius
	 */
	public void updateCompletedScanAreaUnknownShadowing(Coord externalCoord, int tempRadius) {
		
		// Get the angular increment
		double angleStep = Math.atan2(this.cellConversion, tempRadius);
		double angle = 0;
		CopyOnWriteArraySet<Coord> unmodifiedCoords = new CopyOnWriteArraySet<Coord>();
		Coord tempCoord = new Coord();
		boolean shadowed = false;
		
		/* Phase I: Detect shadows */
		ArrayList<Coord> intersectedPoints = new ArrayList<Coord>();
		while (angle < Math.PI * 2) { 
			tempCoord = new Coord((int)(externalCoord.getX() + tempRadius*Math.cos(angle)),
								  (int)(externalCoord.getY() + tempRadius*Math.sin(angle)));
			
			intersectedPoints = this.raytraceShadowing(externalCoord, tempCoord);
			for(Coord c : intersectedPoints) {
				if(shadowed) {
					if(!unmodifiedCoords.contains(c.scaleCoordDown(this.cellConversion))) {
						unmodifiedCoords.add(c.scaleCoordDown(this.cellConversion));
					}
				}

				if(this.cellIsBlocked(c.scaleCoordDown(this.cellConversion))) {
					shadowed = true;
					if(unmodifiedCoords.contains(c.scaleCoordDown(this.cellConversion))) {
						unmodifiedCoords.remove(c.scaleCoordDown(this.cellConversion));
					}
				}

			}
			shadowed = false;
			angle += angleStep;
		}
		
		/*Phase II: update scanned area*/
		ArrayList<Coord> unscannedList = new ArrayList<Coord>();
		for(Coord c : unmodifiedCoords) {
			unscannedList.add(c);
		}
		this.updateCompletedScanAreaUnseenSpaces(externalCoord, tempRadius, (ArrayList<Coord>) unscannedList);
	}
	
	/**
	 * Method for the workers to get the list of unreachable (unscanned) points.
	 * @return List of unscanned Coords
	 */
	public ArrayList<Coord> getUnscannedList() {
		ArrayList<Coord> unscannedList = new ArrayList<Coord>();
		Coord tempCoord;
		for(int x = this.getXBoundMin(); x < this.getXBoundMax(); x++) {
			for(int y = this.getYBoundMin(); y < this.getYBoundMax(); y++) {
				tempCoord = new Coord(x,y);
				if(this.cellIsInBounds(tempCoord) && !this.cellIsScanned(tempCoord)) {
					unscannedList.add(tempCoord);
				}
			}
		}
		return unscannedList;
	}
	
	/**
	 * Updates all the cells in the given radius as scanned except for those in
	 * the given unscanned list, which are not modified.
	 * @param externalCoord
	 * @param tempRadius
	 * @param unscannedList
	 */
	public void updateCompletedScanAreaUnseenSpaces(Coord externalCoord, int tempRadius,
													Collection<Coord> unscannedList) {
		Coord tempCoord = new Coord();
		Coord coordinate = externalCoord.scaleCoordDown(this.cellConversion);
		int radius = tempRadius / this.cellConversion;
		
		double innerOffset = ProbMap.ROOT2OVER2 * radius;
		
		double XMinInner = (coordinate.getX() - innerOffset);
		double XMaxInner = (coordinate.getX() + innerOffset);
		double YMinInner = (coordinate.getY() - innerOffset);
		double YMaxInner = (coordinate.getY() + innerOffset);
		
		int Xmin = ((coordinate.getX() - radius) < this.getXBoundMin() ? this.getXBoundMin() : (coordinate.getX() - radius));
		int Xmax = ((coordinate.getX() + radius) >= this.getXBoundMax() ? this.getXBoundMax() - 1: (coordinate.getX() + radius));
		
		int Ymin = ((coordinate.getY() - radius) < this.getYBoundMin() ? this.getYBoundMin() : (coordinate.getY() - radius));	
		int Ymax = ((coordinate.getY() + radius) >= this.getYBoundMax() ? this.getYBoundMax() - 1: (coordinate.getY() + radius));
		
		for (int x = Xmin; x <= Xmax; x++){
			for(int y = Ymin; y <= Ymax; y++){
				tempCoord = new Coord(x,y);
				
				if (this.cellIsInBounds(tempCoord)){
				
					if (!this.cellIsScanned(tempCoord)){
					
						if(!unscannedList.contains(tempCoord)) {
							if (x > XMinInner && x < XMaxInner && y > YMinInner && y < YMaxInner){
								cellScan(tempCoord);}
						
							else if (this.distanceLessThanOrEqual(coordinate, tempCoord, radius)) {
								cellScan(tempCoord);}
						}
						
					}
				}
				
			}
		}
	}
	
	public void excludeOutOfRangePoints(Coord externalCurrentLocation, int tempRadius) {
		
		Coord coordinate = externalCurrentLocation.scaleCoordDown(this.cellConversion);
		int radius = tempRadius / this.cellConversion;
		
		double innerOffset = ProbMap.ROOT2OVER2 * radius;
		
		double XMinInner = (coordinate.getX() - innerOffset);
		double XMaxInner = (coordinate.getX() + innerOffset);
		double YMinInner = (coordinate.getY() - innerOffset);
		double YMaxInner = (coordinate.getY() + innerOffset);
		
		int Xmin = ((coordinate.getX() - radius) < this.getXBoundMin() ? this.getXBoundMin() : (coordinate.getX() - radius));
		int Xmax = ((coordinate.getX() + radius) >= this.getXBoundMax() ? this.getXBoundMax() - 1: (coordinate.getX() + radius));
		
		int Ymin = ((coordinate.getY() - radius) < this.getYBoundMin() ? this.getYBoundMin() : (coordinate.getY() - radius));	
		int Ymax = ((coordinate.getY() + radius) >= this.getYBoundMax() ? this.getYBoundMax() - 1: (coordinate.getY() + radius));
		
		for (int x = Xmin; x <= Xmax; x++){
			for(int y = Ymin; y <= Ymax; y++){
				Coord tempCoord = new Coord(x,y);
				
					if (x > XMinInner && x < XMaxInner && y > YMinInner && y < YMaxInner){}
					
					else if (this.distanceGreaterThan(coordinate, tempCoord, radius)) {
						cellSetInBounds(tempCoord, false);}
			}
		}
		
		
	}
	
	public ProbMap getMapToSend(Coord currentLocation, int externalRadius) {
		
		int radius = externalRadius / this.cellConversion;
		Coord middle = currentLocation.scaleCoordDown(this.cellConversion);
		
		int xMin = (middle.getX() - radius);
		xMin = xMin < this.getXBoundMin() ? this.getXBoundMin() : xMin;
		
		int xMax = middle.getX() + radius;
		xMax = xMax >= this.getXBoundMax() ? this.getXBoundMax() - 1 : xMax;
		
		int yMin = (middle.getY() - radius);
		yMin = yMin < this.getYBoundMin() ? this.getYBoundMin() : yMin;
		
		int yMax = middle.getY() + radius;
		yMax = yMax >= this.getYBoundMax() ? this.getYBoundMax() - 1 : yMax;
		
		Coord bottomLeft = new Coord(xMin, yMin);
		Coord topRight = new Coord(xMax, yMax);
		
		return new ProbMap(this, bottomLeft, topRight);
	}
	

	/**
	 * Checks if the cell at the given coordinate is scanned
	 * 
	 * @param coordinate
	 * @return boolean
	 */
	public boolean cellIsScanned(Coord coordinate) {
		return this.getCell(coordinate).isScanned();
	}
	
	/**
	 * Scans the cell at the given coordinate
	 * @param coordinate
	 */
	public void cellScan(Coord coordinate) {
		this.getCell(coordinate).scan();
	}
	
	/**
	 * Unscans the cell at the given coordinate
	 * @param coordinate
	 */
	public void cellUnscan(Coord coordinate) {
		this.getCell(coordinate).unscan();
	}
	
	/**
	 * Sets the scan value of the cenn
	 */
	public void cellSetScan(Coord coordinate, boolean scanVal){
		if(scanVal){
			this.getCell(coordinate).scan();
		}
		else this.getCell(coordinate).unscan();
	}
	
	/**
	 * Returns the blocked value of the cell at the given coordinate
	 *
	 * @param coordinate
	 * @return double
	 */
	public double cellGetBlockValue(Coord coordinate) {
		return this.getCell(coordinate).getBlockVal();
	}
	
	/**
	 * Returns true if the blockVal of the cell is greater than the threshold
	 * defined statically in ProbMap
	 * 
	 * @param coordinate
	 * @return boolean
	 */
	public boolean cellIsBlocked(Coord coordinate) {
		if(this.cellGetBlockValue(coordinate) > ProbMap.BLOCKED_THRESHOLD) {
			return true;
		}
		else{ return false; }
	}
	
	/**
	 * Sets the block value of a given cell
	 * 
	 * @param coordinate
	 * @param newBlockValue
	 */
	public void cellSetBlockValue(Coord coordinate, double newBlockValue){
		this.getCell(coordinate).setBlockVal(newBlockValue);
	}
	
	/**
	 * Sets the number of times a cell has been scanned 
	 * 
	 * @param coordinate
	 * @param timesScanned
	 */
	public void cellSetTimesScanned(Coord coordinate, int timesScanned){
		this.getCell(coordinate).setTimesScanned(timesScanned);
	}
	
	/**
	 * Gets the number of times a cell has been scanned 
	 * 
	 * @param coordinate
	 * @return
	 */
	public int cellGetTimesScanned(Coord coordinate){
		return this.getCell(coordinate).getTimesScanned();
	}
	
	/**
	 * Increments the number of times a cell has been scanned
	 * 
	 * @param coordinate
	 */
	public void cellIncrementTimesScanned(Coord coordinate){
		this.getCell(coordinate).incrementScanned();
	}
	
	
	/**
	 * Gets whether or not the cell is in bounds
	 */
	public boolean cellIsInBounds(Coord coordinate){
		return this.getCell(coordinate).isInBounds();
	}
	
	/**
	 * Sets whether or not the cell is in bounds
	 * 
	 * TRUE means in bounds
	 * FALSE means out of bounds
	 */
	public void cellSetInBounds(Coord coordinate, boolean inBounds){
		this.getCell(coordinate).setInBounds(inBounds);
	}
	
	/**
	 * Returns the list of coordinates between the two given.
	 * 
	 * In the case of a tie (crossing both a horizontal and vertical gridline)
	 * this method will include all cells touching that intersection point in
	 * its list. The is accessed using external units, however since it expects
	 * to be incremented cellConversion units at a time, differences are only
	 * handled in steps of that size.  
	 * For example: (2,4) to (3,16) will yield in some order:
	 * (2,4), (2,14), (12,4), (12,14)
	 * @param p1
	 * @param p2
	 * @return
	 */
	public ArrayList<Coord> raytrace(Coord p1, Coord p2) {
		
		ArrayList<Coord> tempList = new ArrayList<Coord>();
	    int dx = Math.abs(p2.getX() - p1.getX())/this.getCellConversion();
	    int dy = Math.abs(p2.getY() - p1.getY())/this.getCellConversion();
	    int x = p1.getX();
	    int y = p1.getY();
	    int xInc = (p2.getX() > p1.getX()) ? this.getCellConversion() : -this.getCellConversion();
	    int yInc = (p2.getY() > p1.getY()) ? this.getCellConversion() : -this.getCellConversion();
	    int error = dx - dy;
	    dx *= 2;
	    dy *= 2;

	    for (int i = 1+ dx/2 + dy/2; i > 0; --i)
	    {
	        if(this.cellInMap(new Coord(x,y))) {
	        	tempList.add(new Coord(x,y));
	        }


	        if (error > 0)
	        {
	            x += xInc;
	            error -= dy;
	        }
	        else if (error < 0)
	        {
	            y += yInc;
	            error += dx;
	        }
	        else {
	        	if(i > 1) {
	        		if(this.cellInMap(new Coord(x,y+yInc))) {
	        			tempList.add(new Coord(x, y + yInc));
	        		}
	        	}
	        	x+= xInc;
	        	error -= dy;
	        	
	        }
	    }
	    
	    if(!tempList.get(tempList.size() - 1).scaleCoordDown(this.getCellConversion()).equals(p2.scaleCoordDown(this.getCellConversion()))){
	    	if(this.cellInMap(p2)){
	    		tempList.add(p2);
	    	}
	    }
	    
	    return tempList;
	}
	
	/**
	 * Returns the list of coordinates between the two given. This version of
	 * raytrace is simplified to only include one of the tangential coordinates
	 * based on the line 'drawn.' This exists to resolve an issue with shadowing.
	 * 
	 * @param p1
	 * @param p2
	 * @return
	 */
	public ArrayList<Coord> raytraceShadowing(Coord p1, Coord p2) {
		
		ArrayList<Coord> tempList = new ArrayList<Coord>();
	    int dx = Math.abs(p2.getX() - p1.getX())/this.getCellConversion();
	    int dy = Math.abs(p2.getY() - p1.getY())/this.getCellConversion();
	    int x = p1.getX();
	    int y = p1.getY();
	    int xInc = (p2.getX() > p1.getX()) ? this.getCellConversion() : -this.getCellConversion();
	    int yInc = (p2.getY() > p1.getY()) ? this.getCellConversion() : -this.getCellConversion();
	    int error = dx - dy;
	    dx *= 2;
	    dy *= 2;

	    for (int i = 1+ dx/2 + dy/2; i > 0; --i)
	    {
	        if(this.cellInMap(new Coord(x,y))) {
	        	tempList.add(new Coord(x,y));
	        }


	        if (error >= 0)
	        {
	            x += xInc;
	            error -= dy;
	        }
	        else if (error < 0)
	        {
	            y += yInc;
	            error += dx;
	        }
	    }
	    
	    if(!tempList.get(tempList.size() - 1).scaleCoordDown(this.getCellConversion()).equals(p2.scaleCoordDown(this.getCellConversion()))){
	    	if(this.cellInMap(p2)){
	    		tempList.add(p2);
	    	}
	    }
	    
	    return tempList;
	}
	
	
	/**
	 * Returns the list of coordinates between the two given.
	 * 
	 * In the case of a tie (crossing both a horizontal and vertical gridline)
	 * this method will include all cells touching that intersection point in
	 * its list. The is accessed using external units, however since it expects
	 * to be incremented cellConversion units at a time, differences are only
	 * handled in steps of that size.  
	 * For example: (2,4) to (3,16) will yield in some order:
	 * (2,4), (2,14), (12,4), (12,14)
	 * @param p1
	 * @param p2
	 * @return
	 */
	public ArrayList<Coord> raytraceOutsideMap(Coord p1, Coord p2){
		
		ArrayList<Coord> tempList = new ArrayList<Coord>();
	    int dx = Math.abs(p2.getX() - p1.getX())/this.getCellConversion();
	    int dy = Math.abs(p2.getY() - p1.getY())/this.getCellConversion();
	    int x = p1.getX();
	    int y = p1.getY();
	    int xInc = (p2.getX() > p1.getX()) ? this.getCellConversion() : -this.getCellConversion();
	    int yInc = (p2.getY() > p1.getY()) ? this.getCellConversion() : -this.getCellConversion();
	    int error = dx - dy;
	    dx *= 2;
	    dy *= 2;

	    for (int i = 1+ dx/2 + dy/2; i > 0; --i)
	    {
	        
	        tempList.add(new Coord(x,y));
	        
	        if (error > 0)
	        {
	            x += xInc;
	            error -= dy;
	        }
	        else if (error < 0)
	        {
	            y += yInc;
	            error += dx;
	        }
	        else {
	        	if(i > 1) {
	        		tempList.add(new Coord(x, y + yInc));
	        	}
	        	x+= xInc;
	        	error -= dy;
	        	
	        }
	    }
	    
	    if(!tempList.get(tempList.size() - 1).scaleCoordDown(this.getCellConversion()).equals(p2.scaleCoordDown(this.getCellConversion()))){
	    	tempList.add(p2);
	    }
	    
	    return tempList;
	}
	
	/**
	 * Returns whether or not a cell is within the map bounds (works in
	 * internal units)
	 * @param coordinate
	 * @return true if within the map, false otherwise
	 */
	public boolean cellInMap(Coord coordinate) {
		int x = coordinate.getX();
		int y = coordinate.getY();
		if(x >= this.getXBoundMin() * this.getCellConversion() && 
		   x < this.getXBoundMax() * this.getCellConversion() &&
		   y >= this.getYBoundMin() * this.getCellConversion() &&
		   y < this.getYBoundMax() * this.getCellConversion()) {
			return true;
		}
		return false;
	}
	
	/* * * * * * * * * * * * * Methods for Dijkstra's * * * * * * * * * * * */
	/* * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * * * * * */
	/* * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * * * * * */
	
	
	@Override
	public Collection<Coord> getNeighbors(Coord node) {
		ArrayList<Coord> neighbors= new ArrayList<Coord>();
		
		//north neighbor
		if(CUpIsInTheMap(node)) {
			Coord north= CUp(node);
			if(cellAreaIsDrivable(north)) {
				neighbors.add(north);
			}
		}
		
		//south neighbor
		if(CDownIsInTheMap(node)) {
			Coord south= CDown(node);
			if(cellAreaIsDrivable(south)) {
				neighbors.add(south);
			}
		}
		
		//east neighbor
		if(CRightIsInTheMap(node)) {
			Coord east= CRight(node);
			if(cellAreaIsDrivable(east)) {
				neighbors.add(east);
			}
		}
		
		//west neighbor
		if(CLeftIsInTheMap(node)) {
			Coord west= CLeft(node);
			if(cellAreaIsDrivable(west)) {
				neighbors.add(west);
			}
		}
		
		
		return neighbors;
	}

	@Override
	public DijkstraNode getNode(Coord node) {
		return this.getCell(node);
	}

	@Override
	public int getEdgeCost(Coord node0, Coord node1) {
		return 1;
	}

	@Override
	public void initDijkstra(Coord startingNode) {
		for(Vector<Cell> col : this.theMap) {
			for(DijkstraNode dn : col) {
				dn.reset();
			}
		}
		this.getCell(startingNode).setDistance(0);
	}
	
	public int cellGetDistance(Coord coord) {
		return this.getCell(coord).getDistance();
	}
	
	public Coord cellGetPrevious(Coord coord) {
		return this.getCell(coord).getPreviousNode();
	}
	
	
	/**
	 * Returns true if the cell is drivable
	 * 
	 * @param coordinate
	 * @return
	 */
	public boolean cellIsReachable(Coord coordinate){
		if (this.getCell(coordinate).isVisited()){
			return true;
		}
		return false;
	}
	
	/**
	 * Determines if a cell is drivable
	 * 
	 * If the cell is both scanned and under a given blocked value and the cell is not occupied
	 * this method will return true, otherwise false.
	 * @param coordinate
	 * @return
	 */
	public boolean cellIsDrivable(Coord coordinate){
		if (this.cellIsScanned(coordinate) && this.cellGetBlockValue(coordinate) < ProbMap.BLOCKED_THRESHOLD && this.cellIsInBounds(coordinate)
				&& !(this.getCell(coordinate).isOccupied())){
			return true;
		}
		return false;
	}
	
	/**
	 * Determines if the area around a cell is drivable
	 * 
	 * This function will determine if a cell is driveable based off the idea taht no two oposing sides can both be unscanned or blocked.
	 * Therefore, if the north side is unscanned (or blocked) then the south side cannot be.  Same with the east - west, NE -SW, and NW - SE sides
	 * around the given cell.
	 * 
	 * Driveable also means that the cell is scanned, below a given block value, and in bounds
	 */
	public boolean cellAreaIsDrivable(Coord home){
		boolean homeIsDrivable = false;
		boolean N_S_Drivable = true;
		boolean E_W_Drivable = true;
		boolean NE_SW_Drivable= true;
		boolean NW_SE_Drivable= true;
		
		boolean north = false;
		boolean northEast= false;
		boolean northWest= false;
		boolean south = false;
		boolean southEast= false;
		boolean southWest= false;
		boolean east = false;
		boolean west = false;
		
		if(this.cellIsDrivable(home)){
			homeIsDrivable = true;
		}
		
		//North
		if (this.CUpIsInTheMap(home)){
			if (this.cellIsDrivable(this.CUp(home))){
				north = true;
			}
			if (this.cellGetBlockValue(this.CUp(home)) >= ProbMap.BLOCKED_THRESHOLD){
				N_S_Drivable = false;
//				System.out.println("Should not be run");
			}
		}
		else {north = true;}
		//North-East
		if(this.CUpRightIsInTheMap(home)) {
			if(this.cellIsDrivable(this.CUpRight(home))) {
				northEast= true;
			}
			if (this.cellGetBlockValue(this.CUpRight(home)) >= ProbMap.BLOCKED_THRESHOLD) {
				NE_SW_Drivable= false;
			}
		}
		else {northEast= true;}
		//North-West
		if(this.CUpLeftIsInTheMap(home)) {
			if(this.cellIsDrivable(this.CUpLeft(home))) {
				northWest= true;
			}
			if (this.cellGetBlockValue(this.CUpLeft(home)) >= ProbMap.BLOCKED_THRESHOLD) {
				NW_SE_Drivable= false;
			}
		}
		else {northWest= true;}
		
		
		//South
		if (this.CDownIsInTheMap(home)){
			if (this.cellIsDrivable(this.CDown(home))){
				south = true;
			}
			if (this.cellGetBlockValue(this.CDown(home)) >= ProbMap.BLOCKED_THRESHOLD){
				N_S_Drivable = false;
//				System.out.println("Should not be run");
			}
		}
		else {south = true;}
		
		//South-East
		if(this.CDownRightIsInTheMap(home)) {
			if(this.cellIsDrivable(this.CDownRight(home))) {
				southEast= true;
			}
			if (this.cellGetBlockValue(this.CDownRight(home)) >= ProbMap.BLOCKED_THRESHOLD) {
				NW_SE_Drivable= false;
			}
		}
		else {southEast= true;}
		//South-West
		if(this.CDownLeftIsInTheMap(home)) {
			if(this.cellIsDrivable(this.CDownLeft(home))) {
				southWest= true;
			}
			if (this.cellGetBlockValue(this.CDownLeft(home)) >= ProbMap.BLOCKED_THRESHOLD) {
				NE_SW_Drivable= false;
			}
		}
		else {southWest= true;}
		
		//East
		if (this.CRightIsInTheMap(home)){
			if (this.cellIsDrivable(this.CRight(home))){
				east = true;
			}
			if (this.cellGetBlockValue(this.CRight(home)) >= ProbMap.BLOCKED_THRESHOLD){
				E_W_Drivable = false;
//				System.out.println("Should not be run");
			}
		}
		else {east = true;}
		
		//West
		if (this.CLeftIsInTheMap(home)){
			if (this.cellIsDrivable(this.CLeft(home))){
				west = true;
			}
			if (this.cellGetBlockValue(this.CLeft(home)) >= ProbMap.BLOCKED_THRESHOLD){
				E_W_Drivable = false;
//				System.out.println("Should not be run");
			}
		}
		else {west = true;}
		
		if ((north || south) && N_S_Drivable){
			N_S_Drivable = true;
		}
		
		if ((east || west) && E_W_Drivable){
			E_W_Drivable = true;
		}
		if ((northEast || southWest) && NE_SW_Drivable) {
			NE_SW_Drivable= true;
		}
		if ((northWest || southEast) && NW_SE_Drivable) {
			NW_SE_Drivable= true;
		}
											//&&
		return homeIsDrivable && N_S_Drivable && E_W_Drivable && NE_SW_Drivable && NW_SE_Drivable;
	}
	
	/**
	 * Takes in the robots location in external coordinates and returns
	 * the cells around the robot
	 * 
	 * @param robotLocation
	 * @return
	 */
	public ArrayList<Coord> getRobotBodyCells(Coord robotExternalPosition){
		
		ArrayList<Coord> robotBodyPoints = new ArrayList<Coord>();
		
		Coord home = robotExternalPosition.scaleCoordDown(this.cellConversion);
		
		robotBodyPoints.add(home);
		
		//North
		if (this.CUpIsInTheMap(home)){
			robotBodyPoints.add(CUp(home));
//			System.out.println("Added: " + CUp(home));
		}
		
		//South
		if (this.CDownIsInTheMap(home)){
			robotBodyPoints.add(CDown(home));
//			System.out.println("Added: " + CDown(home));
		}
		
		//East
		if (this.CRightIsInTheMap(home)){
			robotBodyPoints.add(CRight(home));
	//		System.out.println("Added: " + CRight(home));
			
			Coord east = CRight(home);
			
			//Check the top right point
			if(this.CUpIsInTheMap(east)){
				robotBodyPoints.add(CUp(east));
	//			System.out.println("Added: " + CUp(east));
			}
			
			//Check the bottom right point
			if(this.CDownIsInTheMap(east)){
				robotBodyPoints.add(CDown(east));
	//			System.out.println("Added: " + CDown(east));
			}
		}
		
		//West
		if (this.CLeftIsInTheMap(home)){
			robotBodyPoints.add(CLeft(home));
//			System.out.println("Added: " + CLeft(home));
			
			Coord west = CLeft(home);
			
			//Check the top left point
			if(this.CUpIsInTheMap(west)){
				robotBodyPoints.add(CUp(west));
//				System.out.println("Added: " + CUp(west));
			}
			
			//Check the bottom left point
			if(this.CDownIsInTheMap(west)){
				robotBodyPoints.add(CDown(west));
//				System.out.println("Added: " + CDown(west));
			}
		}
		
		for (Coord c : robotBodyPoints){
			c.setX((c.getX() * this.cellConversion) + this.cellConversion / 2);
			c.setY((c.getY() * this.cellConversion) + this.cellConversion / 2);
		}
		
		
		//return the points that are in the robots body
		return robotBodyPoints;
	}
//	/**
//	 * Returns whether or not the cell and it's surrounding cells (diagonal 
//	 * included)are drivable. 
//	 * 
//	 * This method only considers cells that are in range and not drivable
//	 * to return false, cells that are out of bounds will be skipped.
//	 * @param coordinate
//	 * @return
//	 */
//	public boolean cellAreaIsDrivable(Coord coordinate) {
//		Coord tempCoord;
//		
//		// If the target cell is not drivable, return false immediately
//		if(!cellIsDrivable(coordinate)) {
//			return false;
//		}
//		else {
//			
//			//Area above is in range?
//			if(CUpIsInBounds(coordinate)){
//				tempCoord = CUp(coordinate);
//				
//				//Cell above drivable?
//				if(!cellIsDrivable(tempCoord)) {
//					return false;
//				}
//				//Cell up and left in range?
//				else if(CLeftIsInBounds(tempCoord)) {
//					
//					//Cell up and left drivable?
//					if(!cellIsDrivable(CLeft(tempCoord))) {
//						return false;
//					}
//				}
//			}
//			//Area to the right in range?
//			if(CRightIsInBounds(coordinate)){
//				tempCoord = CRight(coordinate);
//				
//				//Cell to the right drivable?
//				if(!cellIsDrivable(tempCoord)) {
//					return false;
//				}
//				//Cell up and right in range?
//				if(CUpIsInBounds(tempCoord)) {
//					
//					//Cell up and right drivable?
//					if(!cellIsDrivable(CUp(tempCoord))) {
//						return false;
//					}
//				}
//			}
//			
//			//Area below in range?
//			if(CDownIsInBounds(coordinate)){
//				tempCoord = CDown(coordinate);
//				
//				//Cell below drivable?
//				if(!cellIsDrivable(tempCoord)) {
//					return false;
//				}
//				//Cell down and right in range?
//				if(CRightIsInBounds(tempCoord)) {
//					
//					//Cell down and right drivable?
//					if(!cellIsDrivable(CRight(tempCoord))) {
//						return false;
//					}
//				}
//			}
//			
//			//Area left in range?
//			if(CLeftIsInBounds(coordinate)){
//				tempCoord = CLeft(coordinate);
//				
//				//Cell to the left drivable?
//				if(!cellIsDrivable(tempCoord)) {
//					return false;
//				}
//				//Cell down and left in range?
//				if(CDownIsInBounds(tempCoord)) {
//					
//					//Cell down and left drivable?
//					if(!cellIsDrivable(CDown(tempCoord))) {
//						return false;
//					}
//				}
//			}
//			
//			return true;
//		}
//	}
	
	/* * * * * * * * * * * * * Methods for Testing *  * * * * * * * * * * * */
	/* * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * * * * * */
	/* * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * * * * * */
	
	/**
	 * Method used for testing
	 * 
	 * Unscans a map
	 */
	public void testMapUnscan(){
		for (int x = 0; x < this.getxSize(); x++){
			for(int y = 0; y < this.getySize(); y++){
				cellUnscan(new Coord(x,y));
			}
		}
	}
	
	/**
	 * Method used for testings
	 * 
	 * scans a map
	 */
	public void testMapScan(){
		for (int x = this.xOffset; x < this.getxSize() + this.xOffset; x++){
			for(int y = this.yOffset; y < this.getySize() + this.yOffset; y++){
				cellScan(new Coord(x,y));
			}
		}
	}
	
	/**
	 * Method used for testings
	 * 
	 * Scans a given area of the map
	 * @param pos1
	 * @param pos2
	 */
	public void testMapScanArea(Coord pos1, Coord pos2){
		for (int x = pos1.getX(); x <= pos2.getX(); x++){
			for(int y = pos1.getY(); y <= pos2.getY(); y++){
				cellScan(new Coord(x,y));
			}
		}
	}
	
	/**
	 * Method used for testings
	 * 
	 * prints the map to the console
	 */
	public void testMapPrint(){
		Coord tempCoord = new Coord();
		System.out.println("Start Of Map");
		
		for (int y = 0; y < this.getySize(); y++){
			for(int x = 0; x < this.getxSize(); x++){
				tempCoord = new Coord(x + this.xOffset, y + this.yOffset);
				if(this.cellIsBlocked(tempCoord)){
					System.out.print("@");
				}
				else if(this.cellIsScanned(tempCoord)){
					System.out.print("*");
				}
				else System.out.print("?");
			}
			
			System.out.print("\n");
		}
		System.out.println("End Of Map\n");
	}
	
	/**
	 * Apply a given probability to the cell at the given coordinate.
	 * If the current probability that a cell is blocked is A.  This function
	 * changes the probability that the cell is blocked to be A & prob or
	 * A= A+prob - A*prob.  The one exception to this is that if the probability
	 * given is that the cell is blocked is zero it will not use the formula
	 * and instead set the probability that the cell is blocked to be zero.
	 * This is done to account for the fact that once a cell is observed to
	 * be free, it is known with 100% certainty that it is in fact free.
	 * @param c The coordinate of the cell to apply the given probability to.
	 * This must be in INTERNAL UNITS.
	 * @param prob The most recently observed probability that a given
	 * cell is blocked.
	 */
	public void applyProbability(Coord c, double prob) {
		//get the cell at the given coordinate
		Cell cell= this.getCell(c);
		cell.applyProbability(prob);
	}
	
	/**
	 * Calculate the probability that an object at a given point is blocked.
	 * This function will calculate the blocked probability given the origin
	 * of the normal distribution and the coordinate whose blocked value must
	 * be computed. All inputs MUST be in external units
	 * @param distributionCenterPoint The center of the distribution (where
	 * the object was seen).
	 * @param cellToCheck The cell to calculate the probability for.
	 * @param stdDeviation The std deviation of the distribution in cm.
	 */
	public double calculateCellBlockProbability(Coord distributionCenterPoint, Coord cellToCheck, int stdDeviation) {
		//calculate x and y offsets from the distribution center to the cell to check
		int dx= cellToCheck.getX() - distributionCenterPoint.getX();
		int dy= cellToCheck.getY() - distributionCenterPoint.getY();
		
		//a cell is  cellConversion*cellConversion cm so to obtain
		//the corner points of a cell from the center point we need to
		//add/subtract this.cellConversoin/2
		int x1= dx-this.cellConversion/2;
		int x2= dx+this.cellConversion/2;
		
		int y1= dy-this.cellConversion/2;
		int y2= dy+this.cellConversion/2;
		
		//this equation is derived from the volume under a normally distributed
		//pdf. By integrating the pdf over the area of a cell we obtain the
		//probability that the object is somewhere in that cell.  Basically
		//this converts the continuous pdf into a more usable discrete pmf.
		
		double retVal;
		try {
			retVal= 0.25*(Erf.erf(x1/(Math.sqrt(2)*stdDeviation)) - Erf.erf(x2/(Math.sqrt(2)*stdDeviation))) *
			             (Erf.erf(y1/(Math.sqrt(2)*stdDeviation)) - Erf.erf(y2/(Math.sqrt(2)*stdDeviation)));
		} catch (MathException e) {
			//uhh well this is bad
			retVal= -1; //Suppresses "possibly uninitialized" warning.
			e.printStackTrace();
		}
		
		return retVal;
		
	}
	
	public void testMakeBlockedAreas(int numAreas){
		Random rn = new Random();
		int startX;
		int startY;
		
		for (int i = 0; i < numAreas; i++){
			startX = rn.nextInt(this.xSize - 15);
			startY = rn.nextInt(this.ySize - 15);
			System.out.println(startX + " " + startY);
			
			for (int x = startX + 5; x < startX + 5 + 9; x++){
				for (int y = startY + 5; y < startY + 5 + 9; y++){
					this.cellSetBlockValue(new Coord(this.xOffset + x, this.yOffset + y), 1.0);
				}
			}
			
		}
	}
	
}
