package workerCode;

import java.util.ArrayList;

import communication.ChildInformationPacket;
import communication.Comm;
import communication.ScanCompletedKnownShadowingPacket;

public class Worker extends Robot {

	public Worker(int scanMax, int scanMin, Comm robotComm,
			int numChildren, Locomotion robotLocomotion) {
		super(scanMax, scanMin, robotComm, numChildren, robotLocomotion);
		this.robotCoverageAlgorithm= new CoverageAlgorithm();
	}

	@Override
	protected void scan(int radius) {
//		System.out.println("Worker Starting a scan with a radius of " + radius);
		this.robotMap.excludeOutOfRangePoints(this.robotLocomotion.getCurrentLocation(), radius);
		this.updateMapWithCurrentLocation();
		ArrayList<Child> childArrayList= new ArrayList<Child>();
		ArrayList<Child> childSendList= new ArrayList<Child>();
		childArrayList.addAll(this.childList.values());
		while(true) {
			int caStatus= this.robotCoverageAlgorithm.run(this.robotLocomotion.getCurrentLocation(), this.robotMap, childArrayList, radius);
			if(caStatus == CoverageAlgorithm.WORK_TO_DO) {
				
				
				//Determine how many waves are in this iteration of the CA
				
				for (int currentWave = 1; currentWave <= this.findNumWaves(); currentWave++){	
				
					childSendList.clear();
					for(Child c : this.childList.values()) {
						if(c.isMove() && c.getWave() == currentWave) {
							childSendList.add(c);
//							System.out.println("Worker sending child Scout " + c.getID() + " in wave " + c.getWave() + " currently at " + c.getCurrentLocation() + " to " + c.getNextLocation());

						}
	//					System.out.println("Queen Sending child Worker to: " + c.getNextLocation());
					}
					this.sendPathToChildren(childSendList);
					childSendList.clear();
					//wait for children to be ready to scan
					this.waitForChildren();
				
				}

				//form list of children to be sent a map
				for(Child c : this.childList.values()) {
					if(c.getRequiresMap()) {
						childSendList.add(c);
//						System.out.println("Worker sending child Scout " + c.getID() + " a Map");
					}
				}
				//send map to children
				this.sendMapToChildren(childSendList);
				childSendList.clear();
				//form list of children which should be sent scan
				for(Child c : this.childList.values()) {
					if(c.isScan()) {
						childSendList.add(c);
//						System.out.println("Worker sending child Scout " + c.getID() + " a Scan Packet");
//						System.out.println("Worker Telling Child Scout to Scan with Radius: " + c.getScanRadius());
					}
				}
				//send scan packets to children
				this.sendScanToChildren(childSendList);
				childSendList.clear();
				
				this.waitForChildren();
//				System.out.println("Worker: Done One Interation");
//				this.robotMap.testMapPrint();
		}
				
			else if(caStatus == CoverageAlgorithm.NO_MORE_VALID_SPACES) {
				//send back map completed
				ArrayList<Coord> unscannedCells = this.robotMap.getUnscannedList();
				this.robotComm.send(Comm.PARENT_ID, new ScanCompletedKnownShadowingPacket(unscannedCells));
				//break out of while loop
				break;
			}

			
			else if(caStatus == CoverageAlgorithm.UNREACHABLE_SPACES) {
				//send back map completed with shadowing
				ArrayList<Coord> unscannedCells = this.robotMap.getUnscannedList();
				this.robotComm.send(Comm.PARENT_ID, new ScanCompletedKnownShadowingPacket(unscannedCells));
				//break out of while loop
				break;
			}
				
		}
	}

	@Override
	protected void allChildrenReady() {
		this.robotComm.send(Comm.PARENT_ID, new ChildInformationPacket(this.robotLocomotion.getCurrentLocation(),
				                                                       this.scanRadiusMax, this.scanRadiusMin, true));
	}

}
