package workerTest;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.concurrent.CopyOnWriteArraySet;

import org.junit.*;

import workerCode.Child;
import workerCode.Coord;
import workerCode.CoverageAlgorithm;
import workerCode.ProbMap;

public class CoverageAlgorithmTest {
	private CoverageAlgorithm testAlgorithm;
	private ProbMap testMap;
	private Child child1;
	private ArrayList<Child> childList = new ArrayList<Child>();
	
	@Before
	public void setUp() throws Exception{
		testAlgorithm = new CoverageAlgorithm();
	}
	
	@Test
	public void completedMapTest(){
		child1 = new Child(1, new Coord(0,0), 10, 80);
		childList.add(child1);
		
		testMap = new ProbMap(1,1,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		assertEquals(CoverageAlgorithm.NO_MORE_VALID_SPACES, testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY));
	}
	
	@Test
	public void scan1x10MapOneRobotTest(){
		child1 = new Child(1, new Coord(0,0), 10, 80);
		childList.add(child1);
		
		testMap = new ProbMap(10,1,10, ProbMap.NO_OFFSET);
		
		testMap.testMapUnscan();
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(1,0));
		
		testMap.testMapPrint();
		
		assertEquals(CoverageAlgorithm.WORK_TO_DO, testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY));
		
		assertEquals(5, child1.getCurrentLocation().getX());
		assertEquals(5, child1.getCurrentLocation().getY());
		
		assertEquals(15, child1.getFinalPath().get(1).getX());
		assertEquals(5, child1.getFinalPath().get(1).getY());
		
		assertEquals(15, child1.getNextLocation().getX());
		assertEquals(5, child1.getNextLocation().getY());
		
		assertEquals(80, child1.getScanRadius());
	
	}
	
	@Test
	public void scan1x20MapTwoRobotTest(){
		child1 = new Child(1, new Coord(0,0), 10, 80);
		Child child2 = new Child(2, new Coord(190,0), 10, 80);
		
		childList.add(child1);
		childList.add(child2);
		
		testMap = new ProbMap(20,1,10, ProbMap.NO_OFFSET);
		
		testMap.testMapUnscan();
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(1,0));
		
		testMap.cellScan(new Coord(19,0));
		testMap.cellScan(new Coord(18,0));
		
		testMap.testMapPrint();
		
		assertEquals(CoverageAlgorithm.WORK_TO_DO, testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY));
		
		assertEquals(5, child1.getCurrentLocation().getX());
		assertEquals(5, child1.getCurrentLocation().getY());
		
		assertEquals(195, child2.getCurrentLocation().getX());
		assertEquals(5, child2.getCurrentLocation().getY());
		
		assertEquals(15, child1.getNextLocation().getX());
		assertEquals(5, child1.getNextLocation().getY());
		
		assertEquals(185, child2.getNextLocation().getX());
		assertEquals(5, child2.getNextLocation().getY());
		
		assertEquals(80, child1.getScanRadius());
		assertEquals(80, child2.getScanRadius());
	
	}
	
	@Test
	public void scan1x10MapTwoRobotDecreaseScanTest(){
		child1 = new Child(1, new Coord(0,0), 10, 80);
		Child child2 = new Child(2, new Coord(90,0), 10, 80);
		childList.add(child1);
		childList.add(child2);
		
		testMap = new ProbMap(10,1,10, ProbMap.NO_OFFSET);
		
		testMap.testMapUnscan();
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(1,0));
		
		testMap.cellScan(new Coord(8,0));
		testMap.cellScan(new Coord(9,0));
		
		testMap.testMapPrint();
		
		assertEquals(CoverageAlgorithm.WORK_TO_DO, testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY));
		
//		System.out.println("Robot = " + 0 + " Next Location = (" + child1.getCurrentLocation().getX() + "," + child1.getCurrentLocation().getY() + ")");
//		System.out.println("Robot = " + 1 + " Next Location = (" + child2.getCurrentLocation().getX() + "," + child2.getCurrentLocation().getY() + ")");
		
		assertEquals(5, child1.getCurrentLocation().getX());
		assertEquals(5, child1.getCurrentLocation().getY());
		
		assertEquals(95, child2.getCurrentLocation().getX());
		assertEquals(5, child2.getCurrentLocation().getY());
		
		assertEquals(30, child1.getScanRadius());
		assertEquals(30, child2.getScanRadius());
		
		assertEquals(85, child2.getNextLocation().getX());
		assertEquals(5, child2.getNextLocation().getY());
		
		assertEquals(15, child1.getNextLocation().getX());
		assertEquals(5, child1.getNextLocation().getY());
		
	
	}
	
	@Test
	public void removeRobotTest(){
		child1 = new Child(1, new Coord(0,0), 10, 80);
		Child child2 = new Child(2, new Coord(20,0), 10, 80);
		childList.add(child1);
		childList.add(child2);
		
		testMap = new ProbMap(3,1,10, ProbMap.NO_OFFSET);
		
		testMap.testMapUnscan();
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(2,0));
		
		testMap.testMapPrint();
		
		assertEquals(CoverageAlgorithm.WORK_TO_DO, testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY));
	}
	
	@Test
	public void noMoreSpacesTest(){
		child1 = new Child(1, new Coord(0,0), 10, 80);
		Child child2 = new Child(2, new Coord(80,0), 10, 80);
		childList.add(child1);
		childList.add(child2);
		
		testMap = new ProbMap(9,1,10, ProbMap.NO_OFFSET);
		
		testMap.testMapUnscan();
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(1,0));
		testMap.cellScan(new Coord(2,0));
		
		testMap.cellScan(new Coord(4,0));
		
		testMap.cellScan(new Coord(6,0));
		testMap.cellScan(new Coord(7,0));
		testMap.cellScan(new Coord(8,0));
		
		testMap.cellSetBlockValue(new Coord(2,0), 1);
		testMap.cellSetBlockValue(new Coord(6,0), 1);
		
		testMap.testMapPrint();
		
		assertEquals(CoverageAlgorithm.UNREACHABLE_SPACES, testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY));
	}
	
	@Test
	public void largeMapTest(){
		child1 = new Child(1, new Coord(0,0), 10, 80);
		Child child2 = new Child(2, new Coord(1990,1990), 10, 80);
		childList.add(child1);
		childList.add(child2);
		
		testMap = new ProbMap(200,200,10, new Coord(0,0));
		
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(199,199));
		
		assertEquals(CoverageAlgorithm.WORK_TO_DO, testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY));
	}
	
	@Test
	public void boxMap001Test(){
		child1 = new Child(1, new Coord(0,0), 10, 80);
		Child child2 = new Child(2, new Coord(90,90), 10, 80);
		childList.add(child1);
		childList.add(child2);
		
		testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		
		testMap.testMapUnscan();
		
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(1,0));
		testMap.cellScan(new Coord(9,9));
		testMap.cellScan(new Coord(8,9));
		
		testMap.testMapPrint();
		
		assertEquals(CoverageAlgorithm.WORK_TO_DO, testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY));
		
		assertEquals(60, child1.getScanRadius());
		assertEquals(60, child2.getScanRadius());
		
		assertEquals(15, child1.getNextLocation().getX());
		assertEquals(5, child1.getNextLocation().getY());
		
		assertEquals(95, child2.getNextLocation().getX());
		assertEquals(95, child2.getNextLocation().getY());
		
	}
	
	@Test
	public void sameDrivableAreaTest001(){
		child1 = new Child(1, new Coord(0,0), 10, 80);
		Child child2 = new Child(2, new Coord(90,0), 10, 80);
		childList.add(child1);
		childList.add(child2);
		
		testMap = new ProbMap(10,2,10, ProbMap.NO_OFFSET);
		
		testMap.testMapUnscan();
		
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(1,0));
		testMap.cellScan(new Coord(2,0));
		testMap.cellScan(new Coord(3,0));
		testMap.cellScan(new Coord(4,0));
		testMap.cellScan(new Coord(5,0));
		testMap.cellScan(new Coord(6,0));
		testMap.cellScan(new Coord(7,0));
		testMap.cellScan(new Coord(8,0));
		testMap.cellScan(new Coord(9,0));
		
		testMap.testMapPrint();
		
		assertEquals(CoverageAlgorithm.WORK_TO_DO, testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY));
		
		assertEquals(40, child1.getScanRadius());
		assertEquals(40, child2.getScanRadius());
		
		assertEquals(5, child1.getNextLocation().getX());
		assertEquals(5, child1.getNextLocation().getY());
		
		assertEquals(95, child2.getNextLocation().getX());
		assertEquals(5, child2.getNextLocation().getY());
		
	}
	
	@Test
	public void sameDrivableAreaTest002(){
		child1 = new Child(1, new Coord(10,0), 10, 80);
		Child child2 = new Child(2, new Coord(20,0), 10, 80);
		childList.add(child1);
		childList.add(child2);
		
		testMap = new ProbMap(5,5,10, ProbMap.NO_OFFSET);
		
		testMap.testMapUnscan();
		
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(1,0));
		testMap.cellScan(new Coord(2,0));
		
		testMap.cellScan(new Coord(0,1));
		testMap.cellScan(new Coord(1,1));
		testMap.cellScan(new Coord(2,1));
		
		testMap.cellScan(new Coord(0,2));
		testMap.cellScan(new Coord(1,2));
		testMap.cellScan(new Coord(2,2));
		
		testMap.testMapPrint();
		
		assertEquals(CoverageAlgorithm.WORK_TO_DO, testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY));
		
		assertEquals(10, child1.getScanRadius());
		assertEquals(10, child2.getScanRadius());
		
		assertEquals(25, child1.getNextLocation().getX());
		assertEquals(5, child1.getNextLocation().getY());
		
		assertEquals(15, child2.getNextLocation().getX());
		assertEquals(25, child2.getNextLocation().getY());
		
	}
	
	@Test
	public void largeOpenAreaTest(){
		child1 = new Child(1, new Coord(0,0), 10, 80);
		Child child2 = new Child(2, new Coord(0,100), 10, 80);
		Child child3 = new Child(3, new Coord(0,200), 10, 80);
		Child child4 = new Child(4, new Coord(0,300), 10, 80);
		Child child5 = new Child(5, new Coord(0,400), 10, 80);
		childList.add(child1);
		childList.add(child2);
		childList.add(child3);
		childList.add(child4);
		childList.add(child5);
		
		testMap = new ProbMap(200,200,10, ProbMap.NO_OFFSET);
		
		//Left half of the map is scanned
		testMap.testMapScanArea(new Coord(0,0), new Coord(100, 199));
		
//		testMap.testMapPrint();
		
		assertEquals(CoverageAlgorithm.WORK_TO_DO, testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY));
		assertEquals(80, child1.getScanRadius());
		
		for (int robot = 0; robot < childList.size(); robot++){
			System.out.println("Robot " + robot + " went to (" + childList.get(robot).getNextLocation().getX() + "," + childList.get(robot).getNextLocation().getY() + ")");
		}
		
//		System.out.println("Distance for Robot 1 at Final Destination = " + child1.getMyMap().cellGetDistance(new Coord(100,192)));
//		System.out.println("Distance for Robot 1 at Our Location = " + child1.getMyMap().cellGetDistance(new Coord(100,8)));
	}
	
	@Test
	public void testingError001(){
		child1 = new Child(1, new Coord(0,90), 10, 80);
		childList.clear();
		childList.add(child1);
		
		testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.cellScan(new Coord(0,9));

		assertEquals(CoverageAlgorithm.WORK_TO_DO, testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY));
		//assertEquals(80, child1.getScanRadius());
	}
	
	@Test
	public void scanAWholeMapTest001(){
		Child childA = new Child(1, new Coord(0,0), 10, 80);
		Child childB = new Child(2, new Coord(20,0), 10, 80);
		
		childList.clear();
		childList.add(childA);
		childList.add(childB);
		
		testMap = new ProbMap(30, 30, 10, ProbMap.NO_OFFSET);
//		testMap.testMapUnscan();
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(1,0));
		testMap.cellScan(new Coord(2,0));
		testMap.cellScan(new Coord(3,0));
//		testMap.testMapPrint();
		
		int iterations = 0;
		int coverageAlgorithmOpCode = testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY);
		
		for (int i = 0; i < 1000; i++){
			while(coverageAlgorithmOpCode == CoverageAlgorithm.WORK_TO_DO){
				//Add one to the iterations
				iterations++;
				
				//Move the children to their new location
				for (Child c : childList){
					if (c.getFinalPath().size() != 0){
						c.setCurrentLocation(c.getFinalPath().get(c.getFinalPath().size() - 1));
						System.out.println(c.getCurrentLocation());
					}
				}
				
				//Scan the map
				for (Child c : childList){
					testMap.updateCompletedScanAreaUnknownShadowing(c.getCurrentLocation(), c.getScanRadius());
					System.out.println(c.getCurrentLocation());
				}
			
				testMap.testMapPrint();
				coverageAlgorithmOpCode = testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY);
				System.out.println(coverageAlgorithmOpCode);
			}
			
			assertEquals(13, iterations);
		}	
	}
	
	@Test
	public void scanAWholeMapTest002(){
		Child childA = new Child(1, new Coord(100,100), 10, 80);
		Child childB = new Child(2, new Coord(120,100), 10, 80);
		
		childList.clear();
		childList.add(childA);
		childList.add(childB);
		
		testMap = new ProbMap(31, 31, 10, new Coord(10,10));
//		testMap.testMapUnscan();
		testMap.cellScan(new Coord(10,10));
		testMap.cellScan(new Coord(12,10));
//		testMap.testMapPrint();
		
		int iterations = 0;
		
		for (int i = 0; i < 1000; i++){
			while(testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY) == CoverageAlgorithm.WORK_TO_DO){
				//Add one to the iterations
				iterations++;
				
				//Move the children to their new location
				for (Child c : childList){
					if (c.getFinalPath().size() != 0){
						c.setCurrentLocation(c.getFinalPath().get(c.getFinalPath().size() - 1));
					}
				}
				
				//Scan the map
				for (Child c : childList){
					testMap.updateCompletedScanAreaUnknownShadowing(c.getCurrentLocation(), c.getScanRadius());
				}
			}
			
			assertEquals(14, iterations);
		}	
	}
	
	@Test
	public void scanAWholeMapTest003(){
		Child childA = new Child(1, new Coord(0,0), 10, 80);
		Child childB = new Child(2, new Coord(20,0), 10, 80);
		
		childList.clear();
		childList.add(childA);
		childList.add(childB);
		
		testMap = new ProbMap(32, 32, 10, ProbMap.NO_OFFSET);
		testMap.testMapUnscan();
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(2,0));
//		testMap.testMapPrint();
		
		int iterations = 0;
		
		for (int i = 0; i < 1000; i++){
			while(testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY) == CoverageAlgorithm.WORK_TO_DO){
				//Add one to the iterations
				iterations++;
				
				//Move the children to their new location
				for (Child c : childList){
					if (c.getFinalPath().size() != 0){
						c.setCurrentLocation(c.getFinalPath().get(c.getFinalPath().size() - 1));
					}
				}
				
				//Scan the map
				for (Child c : childList){
					testMap.updateCompletedScanAreaUnknownShadowing(c.getCurrentLocation(), c.getScanRadius());
				}
			}
			assertEquals(14, iterations);
		}	
	}
	
	@Test
	public void scanAWholeMapTest004(){
		Child childA = new Child(1, new Coord(0,0), 10, 80);
		Child childB = new Child(2, new Coord(20,0), 10, 80);
		
		childList.clear();
		childList.add(childA);
		childList.add(childB);
		
		testMap = new ProbMap(33, 33, 10, ProbMap.NO_OFFSET);
		testMap.testMapUnscan();
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(2,0));
//		testMap.testMapPrint();
		
		int iterations = 0;
		
		for (int i = 0; i < 1000; i++){
			while(testAlgorithm.run(new Coord(0,0), testMap, childList, Double.POSITIVE_INFINITY) == CoverageAlgorithm.WORK_TO_DO){
				//Add one to the iterations
				iterations++;
				
				//Move the children to their new location
				for (Child c : childList){
					if (c.getFinalPath().size() != 0){
						c.setCurrentLocation(c.getFinalPath().get(c.getFinalPath().size() - 1));
					}
				}
				
				//Scan the map
				for (Child c : childList){
					testMap.updateCompletedScanAreaUnknownShadowing(c.getCurrentLocation(), c.getScanRadius());
				}
			}
			
			assertEquals(16, iterations);
		}	
	}
	
	
	@Test
	public void scanBlockedMapTest001(){
		Child childA = new Child(1, new Coord(0,0), 10, 80);
		Child childB = new Child(2, new Coord(20,0), 10, 80);
		
		childList.clear();
		childList.add(childA);
		childList.add(childB);
		
		testMap = new ProbMap(30, 30, 10, ProbMap.NO_OFFSET);
//		testMap.testMapUnscan();
		testMap.cellScan(new Coord(0,0));
		testMap.cellScan(new Coord(1,0));
		testMap.cellScan(new Coord(2,0));
		testMap.cellScan(new Coord(3,0));
		
		for (int x = 0; x < testMap.getxSize(); x++){
			testMap.cellSetBlockValue(new Coord(x,20), 1.0);
		}
		
		testMap.testMapPrint();
		
		int iterations = 0;
		int coverageAlgorithmOpCode = testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY);
		
		for (int i = 0; i < 1000; i++){
			while(coverageAlgorithmOpCode == CoverageAlgorithm.WORK_TO_DO){
				//Add one to the iterations
				iterations++;
				
				//Move the children to their new location
				for (Child c : childList){
					if (c.getFinalPath().size() != 0){
						c.setCurrentLocation(c.getFinalPath().get(c.getFinalPath().size() - 1));
						System.out.println("Child Going to: " + c.getNextLocation());
					}
				}
				
				//Scan the map
				for (Child c : childList){
					testMap.updateCompletedScanAreaUnknownShadowing(c.getCurrentLocation(), c.getScanRadius());
					System.out.println("Child Scanning at: " + c.getCurrentLocation() + " radius: " + c.getScanRadius());
				}
			
				testMap.testMapPrint();
				coverageAlgorithmOpCode = testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY);
				System.out.println(coverageAlgorithmOpCode);
			}
			
			assertEquals(11, iterations);
		}	
	}
	
	@Test
	public void moveIdleRobotsTest(){
		Child childA = new Child(1, new Coord(30,0), 20, 80);
		Child childB = new Child(2, new Coord(40,0), 20, 80);
		
		childList.clear();
		childList.add(childA);
		childList.add(childB);
		
		testMap = new ProbMap(30, 30, 10, ProbMap.NO_OFFSET);
		testMap.cellScan(new Coord(1,0));
		testMap.cellScan(new Coord(2,0));
		testMap.cellScan(new Coord(3,0));
		testMap.cellScan(new Coord(4,0));
		
		testMap.testMapPrint();
	
		int coverageAlgorithmOpCode = testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY);
		
		System.out.println(childA.getNextLocation() + " " + childB.getNextLocation());
		
//		for (int i = 0; i < 1000; i++){
//			while(coverageAlgorithmOpCode == CoverageAlgorithm.WORK_TO_DO){
//				//Add one to the iterations
//				
//				//Move the children to their new location
//				for (Child c : childList){
//					if (c.getFinalPath().size() != 0){
//						c.setCurrentLocation(c.getFinalPath().get(c.getFinalPath().size() - 1));
//						System.out.println("Child Going to: " + c.getNextLocation());
//					}
//				}
//				
//				//Scan the map
//				for (Child c : childList){
//					testMap.updateCompletedScanAreaUnknownShadowing(c.getCurrentLocation(), c.getScanRadius());
//					System.out.println("Child Scanning at: " + c.getCurrentLocation() + " radius: " + c.getScanRadius());
//				}
//			
//				testMap.testMapPrint();
//				coverageAlgorithmOpCode = testAlgorithm.run(new Coord(), testMap, childList, Double.POSITIVE_INFINITY);
//				System.out.println(coverageAlgorithmOpCode);
//			}
//		}	
	}
	
	@Test
	public void placeOutOfAreaTest() {
		
		Coord currentLocation = new Coord(2,2);
		CopyOnWriteArraySet<Coord> unavailablePoints = new CopyOnWriteArraySet<Coord>();
		ProbMap thisMap = new ProbMap(5,5,10,ProbMap.NO_OFFSET);
		for(int x = 1; x < 4; x++) {
			for(int y = 1; y < 4; y++) {
				unavailablePoints.add(new Coord(x,y));
			}
		}
		
		thisMap.testMapScan();
		thisMap.testMapPrint();
		
		int sStep = 1;
		int rLimit = Math.max(thisMap.getxSize(), thisMap.getySize());
		int currX = currentLocation.getX();
		int currY = currentLocation.getY();
		int x = currX;
		int y = currY;
		Coord tempCoord = new Coord();
		
		//Spiral outwards for 'nearest' point allowed to place at
		while(sStep < rLimit) {
			//Start traversing x
			while(x < currX + sStep) {
				x++;
				tempCoord = new Coord(x,y);
				//If cell is in range of the parent's map
				if(thisMap.cellInMap(tempCoord.scaleCoordUp(thisMap.getCellConversion()))){
					//And is drivable
					if(thisMap.cellAreaIsDrivable(tempCoord)){
						//Is it allowed?
						if(!unavailablePoints.contains(tempCoord)) {
							System.out.println(tempCoord + "SUCCESS!");
							try {
								Thread.sleep(1000);
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
						}
					}
				}
			}

			//Start traversing y
			while(y < currX + sStep) {
				y++;
				tempCoord = new Coord(x,y);
				//If cell is in range of the parent's map
				if(thisMap.cellInMap(tempCoord.scaleCoordUp(thisMap.getCellConversion()))){
					//And is drivable
					if(thisMap.cellAreaIsDrivable(tempCoord)){
						//Is it allowed?
						if(!unavailablePoints.contains(tempCoord)) {
							System.out.println(tempCoord + "SUCCESS!");
							try {
								Thread.sleep(1000);
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
						}
					}
				}
			}

			//Start traversing x left
			while(x > currX - sStep) {
				x--;
				tempCoord = new Coord(x,y);
				//If cell is in range of the parent's map
				if(thisMap.cellInMap(tempCoord.scaleCoordUp(thisMap.getCellConversion()))){
					//And is drivable
					if(thisMap.cellAreaIsDrivable(tempCoord)){
						//Is it allowed?
						if(!unavailablePoints.contains(tempCoord)) {
							System.out.println(tempCoord + "SUCCESS!");
							try {
								Thread.sleep(1000);
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
						}
					}
				}
			}

			//Start traversing y down
			while(y > currX - sStep) {
				y--;
				tempCoord = new Coord(x,y);
				//If cell is in range of the parent's map
				if(thisMap.cellInMap(tempCoord.scaleCoordUp(thisMap.getCellConversion()))){
					//And is drivable
					if(thisMap.cellAreaIsDrivable(tempCoord)){
						//Is it allowed?
						if(!unavailablePoints.contains(tempCoord)) {
							System.out.println(tempCoord + "SUCCESS!");
							try {
								Thread.sleep(1000);
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
						}
					}
				}
			}
			sStep += 1;
		}
	}
	
	
}
