package workerTest;

import static org.junit.Assert.*;

import java.util.ArrayList;

import org.junit.Test;

import dijkstra.DijkstraAlgorithm;

import workerCode.Child;
import workerCode.Coord;
import workerCode.PathPlanner;
import workerCode.ProbMap;

public class PathPlannerTest {

	private PathPlanner myPathPlanner = new PathPlanner();
	private DijkstraAlgorithm myDijk = new DijkstraAlgorithm();
	
	@Test
	public void oneRobotStraitPathTest001(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child child1 = new Child(0, new Coord(0,0), 10, 80);
		ArrayList<Child> childList = new ArrayList<Child>();
		childList.add(child1);
		
		child1.setCurrentLocation(new Coord(0,0));
		child1.setNextLocation(new Coord(9,0));	
		child1.setDijkstraMap(new ProbMap(testMap), null);
		
		this.myDijk.run(child1.getDijkstraMap(), child1.getCurrentLocation());

		child1.setShouldTryToPlaceFlag(true);
		
		for (Child c : childList){
			c.setMove(true);
		}
		
//		myPathPlanner.run(childList, testMap);
		
		myPathPlanner.makeAndOptimizePaths(childList, testMap);
		myPathPlanner.assignWaves(childList, testMap);
		
		
		
		assertEquals(2, child1.getFinalPath().size());
		assertEquals(1, child1.getWave());
	}
	
	@Test
	public void twoRobotSmallPathCrossTest001(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child child1 = new Child(0, new Coord(0,0), 10, 80);
		Child child2 = new Child(0, new Coord(0,1), 10, 80);
		ArrayList<Child> childList = new ArrayList<Child>();
		childList.add(child1);
		childList.add(child2);
		
		child1.setCurrentLocation(new Coord(0,0));
		child2.setCurrentLocation(new Coord(0,1));
		child1.setNextLocation(new Coord(1,1));
		child2.setNextLocation(new Coord(1,0));
		child1.setDijkstraMap(new ProbMap(testMap), null);
		child2.setDijkstraMap(new ProbMap(testMap), null);
		
		this.myDijk.run(child1.getDijkstraMap(), child1.getCurrentLocation());
		this.myDijk.run(child2.getDijkstraMap(), child2.getCurrentLocation());

		child1.setShouldTryToPlaceFlag(true);
		child2.setShouldTryToPlaceFlag(true);
		
		
		for (Child c : childList){
			c.setMove(true);
		}
		
		myPathPlanner.makeAndOptimizePaths(childList, testMap);
		myPathPlanner.assignWaves(childList, testMap);
		
		assertEquals(2, child1.getFinalPath().size());
		assertEquals(1, child1.getWave());
		
		assertEquals(2, child2.getFinalPath().size());
		assertEquals(2, child2.getWave());
	}
	
	@Test
	public void twoRobotStraitPathTest001(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child child1 = new Child(0, new Coord(0,0), 10, 80);
		Child child2 = new Child(0, new Coord(0,9), 10, 80);
		ArrayList<Child> childList = new ArrayList<Child>();
		childList.add(child1);
		childList.add(child2);
		
		child1.setCurrentLocation(new Coord(0,0));
		child2.setCurrentLocation(new Coord(0,9));
		child1.setNextLocation(new Coord(9,0));
		child2.setNextLocation(new Coord(9,9));
		child1.setDijkstraMap(new ProbMap(testMap), null);
		child2.setDijkstraMap(new ProbMap(testMap), null);
		
		this.myDijk.run(child1.getDijkstraMap(), child1.getCurrentLocation());
		this.myDijk.run(child2.getDijkstraMap(), child2.getCurrentLocation());

		child1.setShouldTryToPlaceFlag(true);
		child2.setShouldTryToPlaceFlag(true);
		
		for (Child c : childList){
			c.setMove(true);
		}
		
		myPathPlanner.makeAndOptimizePaths(childList, testMap);
		myPathPlanner.assignWaves(childList, testMap);
		
		assertEquals(2, child1.getFinalPath().size());
		assertEquals(1, child1.getWave());
		
		assertEquals(2, child2.getFinalPath().size());
		assertEquals(1, child2.getWave());
	}
	
	@Test
	public void twoRobotStraitPathTest002(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child child1 = new Child(0, new Coord(0,0), 10, 80);
		Child child2 = new Child(0, new Coord(0,9), 10, 80);
		ArrayList<Child> childList = new ArrayList<Child>();
		childList.add(child1);
		childList.add(child2);
		
		child1.setCurrentLocation(new Coord(0,0));
		child2.setCurrentLocation(new Coord(0,9));
		child1.setNextLocation(new Coord(9,4));
		child2.setNextLocation(new Coord(9,5));
		child1.setDijkstraMap(new ProbMap(testMap), null);
		child2.setDijkstraMap(new ProbMap(testMap), null);
		
		this.myDijk.run(child1.getDijkstraMap(), child1.getCurrentLocation());
		this.myDijk.run(child2.getDijkstraMap(), child2.getCurrentLocation());

		child1.setShouldTryToPlaceFlag(true);
		child2.setShouldTryToPlaceFlag(true);
		
		for (Child c : childList){
			c.setMove(true);
		}
		
		myPathPlanner.makeAndOptimizePaths(childList, testMap);
		myPathPlanner.assignWaves(childList, testMap);
		
		assertEquals(2, child1.getFinalPath().size());
		assertEquals(1, child1.getWave());
		
		assertEquals(2, child2.getFinalPath().size());
		assertEquals(1, child2.getWave());
	}
	
	@Test
	public void twoRobotCrossPathTest001(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child child1 = new Child(0, new Coord(0,0), 10, 80);
		Child child2 = new Child(0, new Coord(0,9), 10, 80);
		ArrayList<Child> childList = new ArrayList<Child>();
		childList.add(child1);
		childList.add(child2);
		
		child1.setCurrentLocation(new Coord(0,0));
		child2.setCurrentLocation(new Coord(0,9));
		child1.setNextLocation(new Coord(9,9));
		child2.setNextLocation(new Coord(9,0));
		child1.setDijkstraMap(new ProbMap(testMap), null);
		child2.setDijkstraMap(new ProbMap(testMap), null);
		
		this.myDijk.run(child1.getDijkstraMap(), child1.getCurrentLocation());
		this.myDijk.run(child2.getDijkstraMap(), child2.getCurrentLocation());

		child1.setShouldTryToPlaceFlag(true);
		child2.setShouldTryToPlaceFlag(true);
		
		for (Child c : childList){
			c.setMove(true);
		}
		
		myPathPlanner.makeAndOptimizePaths(childList, testMap);
		myPathPlanner.assignWaves(childList, testMap);
		
		assertEquals(2, child1.getFinalPath().size());
		assertEquals(1, child1.getWave());
		
		assertEquals(2, child2.getFinalPath().size());
		assertEquals(2, child2.getWave());
	}
	
	@Test
	public void threeRobotCrossPathTest001(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child child1 = new Child(0, new Coord(0,0), 10, 80);
		Child child2 = new Child(0, new Coord(0,9), 10, 80);
		Child child3 = new Child(0, new Coord(0,5), 10, 80);
		ArrayList<Child> childList = new ArrayList<Child>();
		childList.add(child1);
		childList.add(child2);
		childList.add(child3);
		
		child1.setCurrentLocation(new Coord(0,0));
		child2.setCurrentLocation(new Coord(0,9));
		child3.setCurrentLocation(new Coord(0,5));
		child1.setNextLocation(new Coord(9,9));
		child2.setNextLocation(new Coord(9,0));
		child3.setNextLocation(new Coord(9,5));
		child1.setDijkstraMap(new ProbMap(testMap), null);
		child2.setDijkstraMap(new ProbMap(testMap), null);
		child3.setDijkstraMap(new ProbMap(testMap), null);
		
		this.myDijk.run(child1.getDijkstraMap(), child1.getCurrentLocation());
		this.myDijk.run(child2.getDijkstraMap(), child2.getCurrentLocation());
		this.myDijk.run(child3.getDijkstraMap(), child3.getCurrentLocation());

		child1.setShouldTryToPlaceFlag(true);
		child2.setShouldTryToPlaceFlag(true);
		child3.setShouldTryToPlaceFlag(true);
		
		for (Child c : childList){
			c.setMove(true);
		}
		
		myPathPlanner.makeAndOptimizePaths(childList, testMap);
		myPathPlanner.assignWaves(childList, testMap);
		
		assertEquals(2, child1.getFinalPath().size());
		assertEquals(1, child1.getWave());
		
		assertEquals(2, child2.getFinalPath().size());
		assertEquals(2, child2.getWave());
		
		assertEquals(2, child3.getFinalPath().size());
		assertEquals(3, child3.getWave());
	}
	
	@Test
	public void threeRobotCrossAndNotPathTest001(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child child1 = new Child(0, new Coord(0,5), 10, 80);
		Child child2 = new Child(0, new Coord(2,1), 10, 80);
		Child child3 = new Child(0, new Coord(7,1), 10, 80);
		ArrayList<Child> childList = new ArrayList<Child>();
		childList.add(child1);
		childList.add(child2);
		childList.add(child3);
		
		child1.setCurrentLocation(new Coord(0,5));
		child2.setCurrentLocation(new Coord(2,1));
		child3.setCurrentLocation(new Coord(7,1));
		child1.setNextLocation(new Coord(9,5));
		child2.setNextLocation(new Coord(2,8));
		child3.setNextLocation(new Coord(7,8));
		child1.setDijkstraMap(new ProbMap(testMap), null);
		child2.setDijkstraMap(new ProbMap(testMap), null);
		child3.setDijkstraMap(new ProbMap(testMap), null);
		
		this.myDijk.run(child1.getDijkstraMap(), child1.getCurrentLocation());
		this.myDijk.run(child2.getDijkstraMap(), child2.getCurrentLocation());
		this.myDijk.run(child3.getDijkstraMap(), child3.getCurrentLocation());

		child1.setShouldTryToPlaceFlag(true);
		child2.setShouldTryToPlaceFlag(true);
		child3.setShouldTryToPlaceFlag(true);
		
		for (Child c : childList){
			c.setMove(true);
		}
		
		myPathPlanner.makeAndOptimizePaths(childList, testMap);
		myPathPlanner.assignWaves(childList, testMap);
		
		assertEquals(2, child1.getFinalPath().size());
		assertEquals(1, child1.getWave());
		
		assertEquals(2, child2.getFinalPath().size());
		assertEquals(2, child2.getWave());
		
		assertEquals(2, child3.getFinalPath().size());
		assertEquals(2, child3.getWave());
	}
	
	@Test
	public void fourRobotTwoCrossTest001(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child child1 = new Child(1, new Coord(0,1), 10, 80);
		Child child2 = new Child(2, new Coord(1,0), 10, 80);
		Child child3 = new Child(3, new Coord(0,8), 10, 80);
		Child child4 = new Child(4, new Coord(8,0), 10, 80);
		ArrayList<Child> childList = new ArrayList<Child>();
		childList.add(child1);
		childList.add(child2);
		childList.add(child3);
		childList.add(child4);
		
		child1.setCurrentLocation(new Coord(0,1));
		child2.setCurrentLocation(new Coord(1,0));
		child3.setCurrentLocation(new Coord(0,8));
		child4.setCurrentLocation(new Coord(8,0));
		child1.setNextLocation(new Coord(9,1));
		child2.setNextLocation(new Coord(1,9));
		child3.setNextLocation(new Coord(9,8));
		child4.setNextLocation(new Coord(8,9));
		child1.setDijkstraMap(new ProbMap(testMap), null);
		child2.setDijkstraMap(new ProbMap(testMap), null);
		child3.setDijkstraMap(new ProbMap(testMap), null);
		child4.setDijkstraMap(new ProbMap(testMap), null);
		
		this.myDijk.run(child1.getDijkstraMap(), child1.getCurrentLocation());
		this.myDijk.run(child2.getDijkstraMap(), child2.getCurrentLocation());
		this.myDijk.run(child3.getDijkstraMap(), child3.getCurrentLocation());
		this.myDijk.run(child4.getDijkstraMap(), child4.getCurrentLocation());

		child1.setShouldTryToPlaceFlag(true);
		child2.setShouldTryToPlaceFlag(true);
		child3.setShouldTryToPlaceFlag(true);
		child4.setShouldTryToPlaceFlag(true);
		
		for (Child c : childList){
			c.setMove(true);
		}
		
		myPathPlanner.makeAndOptimizePaths(childList, testMap);
		myPathPlanner.assignWaves(childList, testMap);
		
		assertEquals(2, child1.getFinalPath().size());
		assertEquals(1, child1.getWave());
		
		assertEquals(2, child2.getFinalPath().size());
		assertEquals(2, child2.getWave());
		
		assertEquals(2, child3.getFinalPath().size());
		assertEquals(1, child3.getWave());
		
		assertEquals(2, child4.getFinalPath().size());
		assertEquals(2, child4.getWave());
	}
	
	@Test
	public void fourRobotTwoCrossTest002(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child child1 = new Child(1, new Coord(0,0), 10, 80);
		Child child2 = new Child(2, new Coord(0,9), 10, 80);
		Child child3 = new Child(3, new Coord(5,0), 10, 80);
		Child child4 = new Child(4, new Coord(5,9), 10, 80);
		ArrayList<Child> childList = new ArrayList<Child>();
		childList.add(child1);
		childList.add(child2);
		childList.add(child3);
		childList.add(child4);
		
		child1.setCurrentLocation(new Coord(0,0));
		child2.setCurrentLocation(new Coord(0,9));
		child3.setCurrentLocation(new Coord(5,0));
		child4.setCurrentLocation(new Coord(5,9));
		child1.setNextLocation(new Coord(4,9));
		child2.setNextLocation(new Coord(4,0));
		child3.setNextLocation(new Coord(9,9));
		child4.setNextLocation(new Coord(9,0));
		child1.setDijkstraMap(new ProbMap(testMap), null);
		child2.setDijkstraMap(new ProbMap(testMap), null);
		child3.setDijkstraMap(new ProbMap(testMap), null);
		child4.setDijkstraMap(new ProbMap(testMap), null);
		
		this.myDijk.run(child1.getDijkstraMap(), child1.getCurrentLocation());
		this.myDijk.run(child2.getDijkstraMap(), child2.getCurrentLocation());
		this.myDijk.run(child3.getDijkstraMap(), child3.getCurrentLocation());
		this.myDijk.run(child4.getDijkstraMap(), child4.getCurrentLocation());

		child1.setShouldTryToPlaceFlag(true);
		child2.setShouldTryToPlaceFlag(true);
		child3.setShouldTryToPlaceFlag(true);
		child4.setShouldTryToPlaceFlag(true);
		
		for (Child c : childList){
			c.setMove(true);
		}
		
		myPathPlanner.makeAndOptimizePaths(childList, testMap);
		myPathPlanner.assignWaves(childList, testMap);
		
		assertEquals(2, child1.getFinalPath().size());
		assertEquals(1, child1.getWave());
		
		assertEquals(2, child2.getFinalPath().size());
		assertEquals(2, child2.getWave());
		
		assertEquals(2, child3.getFinalPath().size());
		assertEquals(1, child3.getWave());
		
		assertEquals(2, child4.getFinalPath().size());
		assertEquals(2, child4.getWave());
	}
	
	@Test
	public void fiveRobot3Cross3WaveTest001(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child child1 = new Child(1, new Coord(0,0), 10, 80);
		Child child2 = new Child(2, new Coord(0,9), 10, 80);
		Child child3 = new Child(3, new Coord(5,0), 10, 80);
		Child child4 = new Child(4, new Coord(5,9), 10, 80);
		Child child5 = new Child(5, new Coord(0,5), 10, 80);
		ArrayList<Child> childList = new ArrayList<Child>();
		childList.add(child1);
		childList.add(child2);
		childList.add(child3);
		childList.add(child4);
		childList.add(child5);
		
		child1.setCurrentLocation(new Coord(0,0));
		child2.setCurrentLocation(new Coord(0,9));
		child3.setCurrentLocation(new Coord(5,0));
		child4.setCurrentLocation(new Coord(5,9));
		child5.setCurrentLocation(new Coord(0,5));
		child1.setNextLocation(new Coord(4,9));
		child2.setNextLocation(new Coord(4,0));
		child3.setNextLocation(new Coord(9,9));
		child4.setNextLocation(new Coord(9,0));
		child5.setNextLocation(new Coord(9,5));
		child1.setDijkstraMap(new ProbMap(testMap), null);
		child2.setDijkstraMap(new ProbMap(testMap), null);
		child3.setDijkstraMap(new ProbMap(testMap), null);
		child4.setDijkstraMap(new ProbMap(testMap), null);
		child5.setDijkstraMap(new ProbMap(testMap), null);
		
		this.myDijk.run(child1.getDijkstraMap(), child1.getCurrentLocation());
		this.myDijk.run(child2.getDijkstraMap(), child2.getCurrentLocation());
		this.myDijk.run(child3.getDijkstraMap(), child3.getCurrentLocation());
		this.myDijk.run(child4.getDijkstraMap(), child4.getCurrentLocation());
		this.myDijk.run(child5.getDijkstraMap(), child5.getCurrentLocation());

		child1.setShouldTryToPlaceFlag(true);
		child2.setShouldTryToPlaceFlag(true);
		child3.setShouldTryToPlaceFlag(true);
		child4.setShouldTryToPlaceFlag(true);
		child5.setShouldTryToPlaceFlag(true);
		
		for (Child c : childList){
			c.setMove(true);
		}
		
		myPathPlanner.makeAndOptimizePaths(childList, testMap);
		myPathPlanner.assignWaves(childList, testMap);
		
		assertEquals(2, child1.getFinalPath().size());
		assertEquals(1, child1.getWave());
		
		assertEquals(2, child2.getFinalPath().size());
		assertEquals(2, child2.getWave());
		
		assertEquals(2, child3.getFinalPath().size());
		assertEquals(1, child3.getWave());
		
		assertEquals(2, child4.getFinalPath().size());
		assertEquals(2, child4.getWave());
		
		assertEquals(2, child5.getFinalPath().size());
		assertEquals(3, child5.getWave());
	}
	
	@Test
	public void twoRobotCoverageAlgorithmStyleTest001(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child child1 = new Child(0, new Coord(0,0), 10, 80);
		Child child2 = new Child(0, new Coord(0,9), 10, 80);
		ArrayList<Child> childList1 = new ArrayList<Child>();
		ArrayList<Child> childList2 = new ArrayList<Child>();
		childList1.add(child1);
		childList2.add(child2);
		
		child1.setCurrentLocation(new Coord(0,0));
		child2.setCurrentLocation(new Coord(0,9));
		child1.setNextLocation(new Coord(9,0));
		child2.setNextLocation(new Coord(9,9));
		child1.setDijkstraMap(new ProbMap(testMap), null);
		child2.setDijkstraMap(new ProbMap(testMap), null);
		
		this.myDijk.run(child1.getDijkstraMap(), child1.getCurrentLocation());
		this.myDijk.run(child2.getDijkstraMap(), child2.getCurrentLocation());

		child1.setShouldTryToPlaceFlag(true);
		child2.setShouldTryToPlaceFlag(true);
		
		child1.setMove(true);
		child2.setMove(true);
		
		
		
		myPathPlanner.makeAndOptimizePaths(childList1, testMap);
		myPathPlanner.makeAndOptimizePaths(childList2, testMap);
		child1.setWave(1);
		childList1.add(child2);
		myPathPlanner.assignWaves(childList1, testMap);
		
		assertEquals(2, child1.getFinalPath().size());
		assertEquals(1, child1.getWave());
		
		assertEquals(2, child2.getFinalPath().size());
		assertEquals(2, child2.getWave());
	}
	
	@Test
	public void twoRobotCoverageAlgorithmStyleTest002(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child child1 = new Child(0, new Coord(0,0), 10, 80);
		Child child2 = new Child(0, new Coord(0,9), 10, 80);
		ArrayList<Child> childList1 = new ArrayList<Child>();
		ArrayList<Child> childList2 = new ArrayList<Child>();
		childList1.add(child1);
		childList2.add(child2);
		
		child1.setCurrentLocation(new Coord(0,0));
		child2.setCurrentLocation(new Coord(0,9));
		child1.setNextLocation(new Coord(9,0));
		child2.setNextLocation(new Coord(9,9));
		child1.setDijkstraMap(new ProbMap(testMap), null);
		child2.setDijkstraMap(new ProbMap(testMap), null);
		
		this.myDijk.run(child1.getDijkstraMap(), child1.getCurrentLocation());
		this.myDijk.run(child2.getDijkstraMap(), child2.getCurrentLocation());

		child1.setShouldTryToPlaceFlag(true);
		child2.setShouldTryToPlaceFlag(true);
		
		child1.setMove(true);
		child2.setMove(true);
		
		
		
		myPathPlanner.makeAndOptimizePaths(childList1, testMap);
		myPathPlanner.makeAndOptimizePaths(childList2, testMap);
		child2.setWave(1);
		childList1.add(child2);
		myPathPlanner.assignWaves(childList1, testMap);
		
		assertEquals(2, child1.getFinalPath().size());
		assertEquals(2, child1.getWave());
		
		assertEquals(2, child2.getFinalPath().size());
		assertEquals(1, child2.getWave());
	}
	
	@Test
	public void addToPathTest001(){
		ArrayList<Coord> path1 = new ArrayList<Coord>();
		ArrayList<Coord> path2 = new ArrayList<Coord>();
		
		path1.add(new Coord(0,0));
		path1.add(new Coord(1,0));
		path1.add(new Coord(2,0));
		
		path2.add(new Coord(3,0));
		path2.add(new Coord(4,0));
		path2.add(new Coord(5,0));
		
		myPathPlanner.addToPath(path1, path2);
		
		assertEquals(6, path1.size());
		assertEquals(new Coord(0,0), path1.get(0));
		assertEquals(new Coord(1,0), path1.get(1));
		assertEquals(new Coord(2,0), path1.get(2));
		assertEquals(new Coord(3,0), path1.get(3));
		assertEquals(new Coord(4,0), path1.get(4));
		assertEquals(new Coord(5,0), path1.get(5));
	}
	
	@Test
	public void findNewEndpointTest001(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child c1 = new Child(1, new Coord(0,0), 10, 80);
		
		ArrayList<Coord> tempFinalPath = new ArrayList<Coord>();
		
		tempFinalPath.add(new Coord(5,5));
		tempFinalPath.add(new Coord(95,5));
		
		c1.setFinalPath(tempFinalPath);
		
		myPathPlanner.findNewEndpointDistanceFromEnd(c1, 20, testMap);
		
		assertEquals(2, c1.getFinalPath().size());
		assertEquals(new Coord(5,5), c1.getFinalPath().get(0));
		assertEquals(new Coord(75,5), c1.getFinalPath().get(c1.getFinalPath().size() - 1));
		
	}
	
	@Test
	public void findNewEndpointTest002(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child c1 = new Child(1, new Coord(0,0), 10, 80);
		
		ArrayList<Coord> tempFinalPath = new ArrayList<Coord>();
		
		tempFinalPath.add(new Coord(5,5));
		tempFinalPath.add(new Coord(95,5));
		
		c1.setFinalPath(tempFinalPath);
		
		myPathPlanner.findNewEndpointDistanceFromEnd(c1, 40, testMap);
		
		assertEquals(2, c1.getFinalPath().size());
		assertEquals(new Coord(5,5), c1.getFinalPath().get(0));
		assertEquals(new Coord(55,5), c1.getFinalPath().get(c1.getFinalPath().size() - 1));
		
	}
	
	@Test
	public void findNewEndpointTest003(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child c1 = new Child(1, new Coord(0,0), 10, 80);
		
		ArrayList<Coord> tempFinalPath = new ArrayList<Coord>();
		
		tempFinalPath.add(new Coord(5,5));
		tempFinalPath.add(new Coord(15,5));
		tempFinalPath.add(new Coord(25,5));
		tempFinalPath.add(new Coord(35,5));
		tempFinalPath.add(new Coord(45,5));
		tempFinalPath.add(new Coord(55,5));
		tempFinalPath.add(new Coord(65,5));
		tempFinalPath.add(new Coord(75,5));
		tempFinalPath.add(new Coord(85,5));
		tempFinalPath.add(new Coord(95,5));
		
		c1.setFinalPath(tempFinalPath);
		
		myPathPlanner.findNewEndpointDistanceFromEnd(c1, 40, testMap);
		
		assertEquals(6, c1.getFinalPath().size());
		assertEquals(new Coord(5,5), c1.getFinalPath().get(0));
		assertEquals(new Coord(55,5), c1.getFinalPath().get(c1.getFinalPath().size() - 1));
		
	}
	
	@Test
	public void findNewEndpointTest004(){
		ProbMap testMap = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		testMap.testMapScan();
		
		Child c1 = new Child(1, new Coord(0,0), 10, 80);
		
		ArrayList<Coord> tempFinalPath = new ArrayList<Coord>();
		
		tempFinalPath.add(new Coord(5,5));
		tempFinalPath.add(new Coord(15,5));
		tempFinalPath.add(new Coord(25,5));
		tempFinalPath.add(new Coord(35,5));
		tempFinalPath.add(new Coord(45,5));
		tempFinalPath.add(new Coord(55,5));
		tempFinalPath.add(new Coord(65,5));
		tempFinalPath.add(new Coord(75,5));
		tempFinalPath.add(new Coord(85,5));
		tempFinalPath.add(new Coord(95,5));
		
		c1.setFinalPath(tempFinalPath);
		
		myPathPlanner.findNewEndpointDistanceFromEnd(c1, 100, testMap);
		
		assertEquals(0, c1.getFinalPath().size());
//		assertEquals(new Coord(5,5), c1.getFinalPath().get(0));
//		assertEquals(new Coord(55,5), c1.getFinalPath().get(c1.getFinalPath().size() - 1));
		
	}
}
