package workerTest;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.concurrent.CopyOnWriteArraySet;

import org.junit.*;

import workerCode.Cell;
import workerCode.Coord;
import workerCode.ProbMap;

public class ProbMapTest {
	
	private int xSize = 20;
	private int ySize = 30;
	private int cellConversion = 10;
	
	private ProbMap map = new ProbMap(xSize, ySize, cellConversion, ProbMap.NO_OFFSET);
	
	@Test
	public void xSizeGetTest(){
		assertEquals(xSize, map.getxSize());
	}
	
	@Test
	public void ySizeGetTest(){
		assertEquals(ySize, map.getySize());
	}
	
	@Test
	public void conversionFactorGetTest(){
		assertEquals(cellConversion, map.getCellConversion());
	}
	
	@Test
	public void cellGetTopLeftTest(){
		assertNotNull(map.getCell(new Coord(0,0)));
	}
	
	@Test
	public void cellGetTopLeftScanAnswerUnscannedTest(){
		map.testMapUnscan();
		assertEquals(false, map.cellIsScanned(new Coord(0,0)));
		map.cellScan(new Coord(0,0));
		assertEquals(true, map.cellIsScanned(new Coord(0,0)));
	}
		
	@Test
	public void cellGetBottomRightTest(){
		assertNotNull(map.getCell(new Coord(xSize-1,ySize-1)));	
	}
	
	@Test
	public void cellGetBottomRightScanAnswerUnscannedTest(){
		map.testMapUnscan();
		assertEquals(false, map.cellIsScanned(new Coord(xSize-1,ySize-1)));
		map.cellScan(new Coord(xSize-1,ySize-1));
		assertEquals(true, map.cellIsScanned(new Coord(xSize-1,ySize-1)));
	}
	
	@Test
	public void distanceTest(){
		assertEquals(34.66, map.distance(new Coord(0,0), new Coord(xSize-1,ySize-1)), .01);
	}
	
	@Test
	public void distanceGreaterThanPassTest(){
		assertTrue(map.distanceGreaterThan(new Coord(0,0), new Coord(xSize-1,ySize-1), 34));
	}
	
	@Test
	public void distanceGreaterThanFailTest(){
		assertFalse(map.distanceGreaterThan(new Coord(0,0), new Coord(xSize-1,ySize-1), 35));
	}
	
	@Test
	public void distanceLessThanPassTest(){
		assertTrue(map.distanceLessThan(new Coord(0,0), new Coord(xSize-1,ySize-1), 35));
	}
	
	@Test
	public void distanceLessThanFailTest(){
		assertFalse(map.distanceLessThan(new Coord(0,0), new Coord(xSize-1,ySize-1), 34));
	}
	
	@Test
	public void isCellValid001(){
		map.testMapUnscan();
		map.cellScan(new Coord(0,0));
		assertTrue(map.isCellValid(new Coord(0,0)));
	}
	
	@Test
	public void isCellValid002(){
		map.testMapUnscan();
		map.cellScan(new Coord(0,0));
		map.cellScan(new Coord(0, 1));
		assertTrue(map.isCellValid(new Coord(0, 0)));
	}
	
	@Test
	public void isCellValid003(){
		map.testMapUnscan();
		map.cellScan(new Coord(0,0));
		map.cellScan(new Coord(1, 0));
		assertTrue(map.isCellValid(new Coord(0, 0)));
	}
	
	@Test
	public void isCellValidXY1(){
		map.testMapUnscan();
		map.cellScan(new Coord(xSize-1,ySize-1));
		assertTrue(map.isCellValid(new Coord(xSize-1,ySize-1)));
	}
	
	@Test
	public void isCellValidXY2(){
		map.testMapUnscan();
		map.cellScan(new Coord(xSize-1,ySize-1));
		map.cellScan(new Coord(xSize-1,ySize-2));
		assertTrue(map.isCellValid(new Coord(xSize-1,ySize-1)));
	}
	
	@Test
	public void isCellValidXY3(){
		map.testMapUnscan();
		map.cellScan(new Coord(xSize-1,ySize-1));
		map.cellScan(new Coord(xSize-2,ySize-1));
		assertTrue(map.isCellValid(new Coord(xSize-1,ySize-1)));
	}
	
	@Test
	public void isCellValidFailTest1(){
		map.testMapUnscan();
		assertFalse(map.isCellValid(new Coord(0,0)));
	}
	
	@Test
	public void isCellValidFailTest2(){
		map.testMapUnscan();
		map.cellScan(new Coord(0,0));
		map.cellScan(new Coord(1,0));
		map.cellScan(new Coord(0,1));
		assertFalse(map.isCellValid(new Coord(0, 0)));
	}
	
	
	@Test
	public void getSetBlockValueTest(){
		assertEquals(0, map.cellGetBlockValue(new Coord(0,0)), 0.0);
		map.cellSetBlockValue(new Coord(0,0), 1);
		assertEquals(1, map.cellGetBlockValue(new Coord(0,0)), 0.0);
		map.cellSetBlockValue(new Coord(0,0), 0);
	}
	
//	@Test
//	public void getSetVisitedTest(){
//		assertFalse(map.cellGetVisited(new Coord(0,0)));
//		map.cellSetVisited(new Coord(0,0), true);
//		assertTrue(map.cellGetVisited(new Coord(0,0)));
//		map.cellSetVisited(new Coord(0,0), false);
//	}
	
//	@Test
//	public void cellDistanceIsInfinityTest(){
//		assertEquals(Dijkstra.infinity, map.cellGetDistance(new Coord(0,0)));
//	}
//	
//	@Test
//	public void setCellDistanceTest(){
//		map.cellSetDistance(new Coord(0,0), 1);
//		assertEquals(1, map.cellGetDistance(new Coord(0,0)), 0);
//	}
//	
//	@Test
//	public void cellIsDrivableFailTest(){
//		map.cellSetDistance(new Coord(0,0), Dijkstra.infinity);
//		assertFalse(map.cellIsReachable(new Coord(0,0)));
//	}
//	
//	@Test
//	public void cellIsDrivablePassTest(){
//		map.cellSetDistance(new Coord(0,0), 1);
//		assertTrue(map.cellIsReachable(new Coord(0,0)));
//	}
	
//	@Test
//	public void cellGetPreviousTest(){
//		assertEquals(new Coord(), map.cellGetPrevious(new Coord(0,0)));
//	}
//	
//	@Test
//	public void cellSetPreviousTest(){
//		map.cellSetPrevious(new Coord(0,0), new Coord(1,1));
//		map.cellSetPrevious(new Coord(0,0), new Coord());
//		assertEquals(new Coord(), map.cellGetPrevious(new Coord(0,0)));
//	}
	
//	@Test
//	public void probMapCopy(){
//		map.testMapScan();
//		map.cellSetDistance(new Coord(0,0), 0);
//		ProbMap testMap = new ProbMap(this.map);
//		assertEquals(true, testMap.cellIsScanned(new Coord(0,0)));
//		assertEquals(Dijkstra.infinity, testMap.cellGetDistance(new Coord(0,0)));
//	}
	
	@Test
	public void drivablePass001(){
		map.testMapUnscan();
		map.cellScan(new Coord(0,0));
		assertTrue(map.cellIsDrivable(new Coord(0,0)));
	}
	
	@Test
	public void drivablePass002(){
		map.testMapUnscan();
		map.cellScan(new Coord(map.getxSize() -1 ,map.getySize()-1));
		assertTrue(map.cellIsDrivable(new Coord(map.getxSize() -1 ,map.getySize()-1)));
	}
	
	@Test
	public void drivableFail001(){
		map.testMapUnscan();
		assertFalse(map.cellIsDrivable(new Coord(0,0)));
	}
	
	@Test
	public void drivableFail002(){
		map.testMapUnscan();
		assertFalse(map.cellIsDrivable(new Coord(map.getxSize() -1 ,map.getySize()-1)));
	}
	
	@Test
	public void cellDirectionsTest(){
		assertFalse(map.CUp(new Coord(9999,9999)).isValid());
		assertFalse(map.CLeft(new Coord()).isValid());
		assertFalse(map.CDown(new Coord()).isValid());
		assertFalse(map.CRight(new Coord(9999,9999)).isValid());	
	}
	
	@Test
	public void cellsUncoveredTest001(){
		this.map.testMapUnscan();
		this.map.testMapScanArea(new Coord(0,0), new Coord(10,29));
		
		assertTrue(map.cellIsScanned(new Coord(10, 15)));
		assertEquals(1, map.computeCellsUncovered(new Coord(10,15), 1));
		assertEquals(4, map.computeCellsUncovered(new Coord(10,15), 2));
	}
	
	@Test
	public void cellsUncoveredTest002(){
		this.map.testMapUnscan();
		this.map.cellScan(new Coord(10,15));
		
		assertTrue(map.cellIsScanned(new Coord(10, 15)));
		assertEquals(4, map.computeCellsUncovered(new Coord(10,15), 1));
		assertEquals(12, map.computeCellsUncovered(new Coord(10,15), 2));
	}
	
//	@Test
//	public void updateScannedAreaTest001(){
//		map.testMapUnscan();
//		map.cellScan(new Coord(10,15));
//		
//		assertTrue(!map.cellIsScanned(new Coord(9,15)));
//		assertTrue(!map.cellIsScanned(new Coord(11,15)));
//		assertTrue(!map.cellIsScanned(new Coord(10,14)));
//		assertTrue(!map.cellIsScanned(new Coord(10,16)));
//		
//		map.testMapPrint();
//		map.updateCompletedScanArea(new Coord(100,150), 10);
//		map.testMapPrint();
//		
//		assertTrue(map.cellIsScanned(new Coord(9,15)));
//		assertTrue(map.cellIsScanned(new Coord(11,15)));
//		assertTrue(map.cellIsScanned(new Coord(10,14)));
//		assertTrue(map.cellIsScanned(new Coord(10,16)));	
//		map.updateCompletedScanArea(new Coord(100,150), 20);
//		map.testMapPrint();
//		
//		assertTrue(map.cellIsScanned(new Coord(8,15)));
//		assertTrue(map.cellIsScanned(new Coord(12,15)));
//		assertTrue(map.cellIsScanned(new Coord(10,13)));
//		assertTrue(map.cellIsScanned(new Coord(10,17)));
//	}
	
//	@Test
//	public void cellVisitedTest(){
//		assertFalse(map.cellIsVisited(new Coord(0,0)));
//		map.cellSetVisited(new Coord(0,0), true);
//		assertTrue(map.cellIsVisited(new Coord(0,0)));
//	}
//	
//	@Test
//	public void cellQueueTest(){
//		assertFalse(map.cellIsInQueue(new Coord(0,0)));
//		map.cellSetInQueue(new Coord(0,0), true);
//		assertTrue(map.cellIsInQueue(new Coord(0,0)));
//	}
//	
//	@Test
//	public void cellPreviousTest(){
//		assertEquals(-1, map.cellGetPrevious(new Coord(0,0)).getX());
//		assertEquals(-1, map.cellGetPrevious(new Coord(0,0)).getY());
//		map.cellSetPrevious(new Coord(0,0), new Coord(1,1));
//		assertEquals(1, map.cellGetPrevious(new Coord(0,0)).getX());
//		assertEquals(1, map.cellGetPrevious(new Coord(0,0)).getY());
//
//	}
	
//	@Test
//	public void cellAreaIsValidTest(){
//		map.testMapScan();
//		assertTrue(map.cellAreaIsDrivable(new Coord(0,0)));
//		assertTrue(map.cellAreaIsDrivable(new Coord(19,29)));
//		map.testMapUnscan();
//
//		map.testMapScanArea(new Coord(3,3), new Coord(3,3));
//		assertFalse(map.cellAreaIsDrivable(new Coord(3,3)));
//		
//		map.testMapScanArea(new Coord(3,4), new Coord(3,4));
//		assertFalse(map.cellAreaIsDrivable(new Coord(3,3)));
//		
//		map.testMapScanArea(new Coord(2,4), new Coord(2,4));
//		assertFalse(map.cellAreaIsDrivable(new Coord(3,3)));
//		
//		map.testMapScanArea(new Coord(4,3), new Coord(4,3));
//		assertFalse(map.cellAreaIsDrivable(new Coord(3,3)));
//		
//		map.testMapScanArea(new Coord(4,4), new Coord(4,4));
//		assertFalse(map.cellAreaIsDrivable(new Coord(3,3)));
//		
//		map.testMapScanArea(new Coord(3,2), new Coord(3,2));
//		assertFalse(map.cellAreaIsDrivable(new Coord(3,3)));
//		
//		map.testMapScanArea(new Coord(4,2), new Coord(4,2));
//		assertFalse(map.cellAreaIsDrivable(new Coord(3,3)));
//		
//		map.testMapScanArea(new Coord(2,3), new Coord(2,3));
//		assertFalse(map.cellAreaIsDrivable(new Coord(3,3)));
//		
//		map.testMapScanArea(new Coord(2,2), new Coord(2,2));
//		map.testMapPrint();
//		assertTrue(map.cellAreaIsDrivable(new Coord(3,3)));
//		
//	}
	
	@Test 
	public void raytraceTest() {
		ArrayList<Coord> tempList = map.raytrace(new Coord(0,0), new Coord(30,30));

		assertEquals(10, tempList.size());
		
		tempList = map.raytrace(new Coord(0,0), new Coord(50,0));
		assertEquals(6, tempList.size());
		
		tempList = map.raytrace(new Coord(10,10), new Coord(10,60));
		assertEquals(6, tempList.size());
		
		tempList = map.raytrace(new Coord(30,30), new Coord(0,20));
		assertEquals(6, tempList.size());
		
		tempList = map.raytrace(new Coord(0,0), new Coord(20,12));
		assertEquals(4, tempList.size());
		
		tempList = map.raytrace(new Coord(3,7), new Coord(14,19));
		assertEquals(4, tempList.size());
		
		ProbMap test = new ProbMap(3,3,10, ProbMap.NO_OFFSET);
		tempList = test.raytrace(new Coord(5,5), new Coord(95,95));
		assertEquals(7, tempList.size());
	}
	
	@Test
	public void cellInMapTest() {
		ProbMap test = new ProbMap(2,2,10, ProbMap.NO_OFFSET);
		
		System.out.println(test.getXBoundMin());
		System.out.println(test.getXBoundMax());
		System.out.println(test.getYBoundMin());
		System.out.println(test.getYBoundMax());
		
		
		assertTrue(test.cellInMap(new Coord(0,0)));
		assertTrue(test.cellInMap(new Coord(1,0)));
		assertTrue(test.cellInMap(new Coord(0,1)));
		assertTrue(test.cellInMap(new Coord(1,1)));
		
		assertFalse(test.cellInMap(new Coord(-1,0)));
		assertFalse(test.cellInMap(new Coord(0,-1)));
		assertFalse(test.cellInMap(new Coord(-1,-1)));
	}
	
	@Test
	public void cellAreaDrivableTest001(){
		ProbMap test = new ProbMap(3,3,10, ProbMap.NO_OFFSET);
		
		test.testMapUnscan();
		
		test.cellScan(new Coord(0,0));
		test.cellScan(new Coord(1,0));
		test.cellScan(new Coord(2,0));
		
		test.cellScan(new Coord(1,1));
		test.cellScan(new Coord(2,1));
		
		test.cellScan(new Coord(0,2));
		test.cellScan(new Coord(1,2));
		test.cellScan(new Coord(2,2));
		
		test.testMapPrint();
		
		assertTrue(test.cellAreaIsDrivable(new Coord(1,1)));
	}
	
	@Test
	public void cellAreaDrivableTest002(){
		ProbMap test = new ProbMap(3,3,10, ProbMap.NO_OFFSET);
		
		test.testMapUnscan();
		
		test.cellScan(new Coord(0,0));
		test.cellScan(new Coord(1,0));
		test.cellScan(new Coord(2,0));
		
		test.cellScan(new Coord(1,1));
		test.cellScan(new Coord(0,1));
		
		test.cellScan(new Coord(0,2));
		test.cellScan(new Coord(1,2));
		test.cellScan(new Coord(2,2));
		
		test.testMapPrint();
		
		assertTrue(test.cellAreaIsDrivable(new Coord(1,1)));
	}
	
	@Test
	public void cellAreaDrivableTest003(){
		ProbMap test = new ProbMap(3,3,10, ProbMap.NO_OFFSET);
		
		test.testMapUnscan();
		
		test.cellScan(new Coord(0,0));
		test.cellScan(new Coord(1,0));
		test.cellScan(new Coord(2,0));
		
		test.cellScan(new Coord(0,1));
		test.cellScan(new Coord(1,1));
		test.cellScan(new Coord(2,1));
		
		test.cellScan(new Coord(0,2));
		test.cellScan(new Coord(2,2));
		
		test.testMapPrint();
		
		assertTrue(test.cellAreaIsDrivable(new Coord(1,1)));
	}
	
	@Test
	public void cellAreaDrivableTest004(){
		ProbMap test = new ProbMap(3,3,10, ProbMap.NO_OFFSET);
		
		test.testMapUnscan();
		
		test.cellScan(new Coord(0,0));
		test.cellScan(new Coord(2,0));
		
		test.cellScan(new Coord(0,1));
		test.cellScan(new Coord(1,1));
		test.cellScan(new Coord(2,1));
		
		test.cellScan(new Coord(0,2));
		test.cellScan(new Coord(1,2));
		test.cellScan(new Coord(2,2));
		
		test.testMapPrint();
		
		assertTrue(test.cellAreaIsDrivable(new Coord(1,1)));
	}
	
	@Test
	public void cellAreaDrivableTest005(){
		ProbMap test = new ProbMap(3,3,10, ProbMap.NO_OFFSET);
		
		test.testMapUnscan();
		
		test.cellScan(new Coord(0,0));
		test.cellScan(new Coord(2,0));
		
		test.cellScan(new Coord(0,1));
		test.cellScan(new Coord(1,1));
		test.cellScan(new Coord(2,1));
		
		test.cellScan(new Coord(0,2));
		test.cellScan(new Coord(2,2));
		
		test.testMapPrint();
		
		assertFalse(test.cellAreaIsDrivable(new Coord(1,1)));
	}
	
	@Test
	public void cellAreaDrivableTest006(){
		ProbMap test = new ProbMap(3,3,10, ProbMap.NO_OFFSET);
		
		test.testMapUnscan();
		
		test.cellScan(new Coord(0,0));
		test.cellScan(new Coord(1,0));
		test.cellScan(new Coord(2,0));
		

		test.cellScan(new Coord(1,1));

		
		test.cellScan(new Coord(0,2));
		test.cellScan(new Coord(1,2));
		test.cellScan(new Coord(2,2));
		
		test.testMapPrint();
		System.out.println("===== Done cellAreaDrivable tests =====");
		
		assertFalse(test.cellAreaIsDrivable(new Coord(1,1)));
	}
	
//	@Test
//	public void probabilityDistributionTest001() {
//		ProbMap test= new ProbMap(10, 10, 10, ProbMap.NO_OFFSET);
//		
//		Coord objectPos= new Coord(55, 55);
//		test.updateCellAreaBlockValue(objectPos, 5);
//		
//		
//		for(int i= 9; i >= 0; i--) {
//			for(int j= 0; j < 10; j++) {
//				Cell aCell= test.getCell(new Coord(j,i));
//				System.out.printf("%10.5f ", aCell.getBlockVal());
//			}
//			System.out.println();
//		}
//	}
	
	@Test
	public void shadowingTest() {
		ProbMap test = new ProbMap(3,3,10, ProbMap.NO_OFFSET);
		
		test.testMapUnscan();
		test.cellSetBlockValue(new Coord(1,1), 1);
		test.testMapPrint();
		test.updateCompletedScanAreaUnknownShadowing(new Coord(0,0), 50);
		test.testMapPrint();
		assertFalse(test.cellIsScanned(new Coord(2,2)));
		
		test.testMapUnscan();
		test.updateCompletedScanAreaUnknownShadowing(new Coord(5,5), 70);
		test.testMapPrint();
		
		ProbMap test2 = new ProbMap(10,10,10, ProbMap.NO_OFFSET);

		for(int i = 0; i < 10; i++) {
			test2.cellSetBlockValue(new Coord(i, 7), 1);
		}
		test2.updateCompletedScanAreaUnknownShadowing(new Coord(45,45), 80);
		test2.testMapPrint();
		
		ProbMap test3 = new ProbMap(10,1,10, ProbMap.NO_OFFSET);
		test3.testMapPrint();
		test3.cellSetBlockValue(new Coord(4,0), 1);
		test3.updateCompletedScanAreaUnknownShadowing(new Coord(5,5), 80);
		test3.testMapPrint();
		
		
		/***** testing with offset ******/
		
		test2 = new ProbMap(10,10,10, new Coord (10,10));
		System.out.println(test2.getXOffset()+ ", " + test2.getYOffset());
		System.out.println(test2.getXBoundMin()+ " -> " + test2.getXBoundMax() + ", " +
						   test2.getYBoundMin()+ " -> " + test2.getYBoundMax());
		for(int i = 10; i < 20; i++) {
			test2.cellSetBlockValue(new Coord(i, 17), 1);
		}
		test2.updateCompletedScanAreaUnknownShadowing(new Coord(145,145), 80);
		test2.testMapPrint();
	}
	
	@Test
	public void shadowingTest2() {
		ProbMap test = new ProbMap(30,30,10, ProbMap.NO_OFFSET);
		for(int i = 0; i < 30; i++) {
			test.cellSetBlockValue(new Coord(i, 20), 1);
		}
		//test.cellScan(new Coord(7,13));
		test.testMapPrint();
		test.updateCompletedScanAreaUnknownShadowing(new Coord(75,135), 80);
		test.testMapPrint();
	}
	
	@Test
	public void randomBlockedTest001(){
		ProbMap test = new ProbMap(50, 50, 10, new Coord(10,10));
//		test.testMapUnscan();
		test.testMakeBlockedAreas(5);
		test.testMapPrint();
		assertTrue(true);
	}

//	@Test
//	public void getUnscannedListTest() {
//		ProbMap test = new ProbMap(3,3,10, ProbMap.NO_OFFSET);
//		test.testMapUnscan();
//		ArrayList<Coord> unscannedList = test.getUnscannedList();
//		assertEquals(9, unscannedList.size());
//		test.cellScan(new Coord(0,0));
//		unscannedList = test.getUnscannedList();
//		assertEquals(8, unscannedList.size());
//	}
	
	@Test
	public void findVisiblePointsInBubbleTest001() {
		ProbMap test = new ProbMap(30,30,10, ProbMap.NO_OFFSET);
		for(int i = 0; i < 30; i++) {
			test.cellSetBlockValue(new Coord(i, 20), 1);
		}
		
		CopyOnWriteArraySet<Coord> visiblePoints = (CopyOnWriteArraySet<Coord>) test.findVisiblePointsInBubble(new Coord(75,135), 80);
		assertEquals(220, visiblePoints.size());
		for(Coord c : visiblePoints) {
			test.cellScan(c.scaleCoordDown(test.getCellConversion()));
		}
		test.testMapPrint();
		
		visiblePoints = (CopyOnWriteArraySet<Coord>) test.findVisiblePointsInBubble(new Coord(75,115), 80);
		assertEquals(226, visiblePoints.size());
	}
	
	@Test
	public void findVisiblePointsInBubbleTest002() {
		ProbMap test = new ProbMap(9,9,10, ProbMap.NO_OFFSET);
		for(int i = 2; i < 7; i++) {
			test.cellSetBlockValue(new Coord(i, 6), 1);
			test.cellSetBlockValue(new Coord(i, 2), 1);
		}
		for(int i = 2; i < 7; i++) {
			test.cellSetBlockValue(new Coord(6, i), 1);
			test.cellSetBlockValue(new Coord(2, i), 1);
		}
		test.testMapPrint();
		
		CopyOnWriteArraySet<Coord> visiblePoints = (CopyOnWriteArraySet<Coord>) test.findVisiblePointsInBubble(new Coord(45,45), 80);
		assertEquals(21, visiblePoints.size()); //???
		for(Coord c : visiblePoints) {
			test.cellScan(c.scaleCoordDown(test.getCellConversion()));
		}
		test.testMapPrint();
	}
	
	@Test
	public void robotBodyPointsTest001(){
		ProbMap test = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		ArrayList<Coord> robotBodyPoints = new ArrayList<Coord>();
		robotBodyPoints = test.getRobotBodyCells(new Coord(0,0));
		
		for (Coord c : robotBodyPoints){
			System.out.println(c);
		}
		
		assertEquals(4, robotBodyPoints.size());
	
	}
	
	@Test
	public void robotBodyPointsTest002(){
		ProbMap test = new ProbMap(10,10,10, ProbMap.NO_OFFSET);
		ArrayList<Coord> robotBodyPoints = new ArrayList<Coord>();
		robotBodyPoints = test.getRobotBodyCells(new Coord(55,55));
		
		for (Coord c : robotBodyPoints){
			System.out.println(c);
		}
		
		assertEquals(9, robotBodyPoints.size());
	
	}
	
	@Test
	public void robotBodyPointsTest003(){
		ProbMap test = new ProbMap(10,10,10, new Coord(10,10));
		ArrayList<Coord> robotBodyPoints = new ArrayList<Coord>();
		robotBodyPoints = test.getRobotBodyCells(new Coord(155,155));
		
		for (Coord c : robotBodyPoints){
			System.out.println(c);
		}
		
		assertEquals(9, robotBodyPoints.size());
	
	}
	
	@Test
	public void probabilityDistributionTest001() {
		ProbMap test= new ProbMap(10, 10, 10, ProbMap.NO_OFFSET);
		
		Coord objectPos= new Coord(55, 55);
		test.updateCellAreaBlockValue(objectPos, 5);
		
		
		for(int i= 9; i >= 0; i--) {
			for(int j= 0; j < 10; j++) {
				Cell aCell= test.getCell(new Coord(j,i));
				System.out.printf("%10.5f ", aCell.getBlockVal());
			}
			System.out.println();
		}
	}
	
}
