/*
 * commnuications.h
 *
 *  Created on: Dec 19, 2010
 *      Author: hsrmqp
 */

#ifndef COMMUNICATIONS_H_
#define COMMUNICATIONS_H_

#include "includes.h"
#include "localization.h"

#define PARENT_ID 0
#define INVALID_SENDER (0xFFu)
#define BROADCAST_SENDER (0xFEu)

#define SCAN_PACKET_OPCODE 0
#define INFORMATION_REPLY_PACKET_OPCODE 1
#define PATH_PACKET_OPCODE 4
#define REACHED_DESTINATION_PACKET_OPCODE 5
#define ADD_POINT_MAP_PACKET_OPCODE 9
#define MAP_COMPLETE_UNKNOWN_SHADOW_OPCODE 10


//Communication struct
typedef struct swarm_communication {
   uint8_t id; //This is probably always PARENT_ID
   uint8_t opcode;
   void* packet;
} swarm_communication_t;
//Packet structs
//Scan packet
typedef struct scan_packet {
   uint16_t radius;
} scan_packet_t;

//Request Information Packet
typedef struct request_information_packet {
   //No body in this packet
} request_information_packet_t;

//Path Packet
typedef struct path_packet {
   coordinate_t* waypoints;
   uint8_t numWaypoints;
} path_packet_t;

//Information Reply Packet
typedef struct information_reply_packet {
   coordinate_t location;
   uint16_t maxScanRadius;
   uint16_t minScanRadius;
   uint8_t requiresMap; //0 for does not require a map to be sent before scan,
                        //1 if it does need a map transmit.  The scout doesn't
                        //need one so leave it at zero.
} information_reply_packet_t;

//Add Single Point Packet
typedef struct add_point_map_packet {
   coordinate_t location;
   uint16_t stdDeviation;
   uint8_t propertiesBitmask; //bitmask for the properties of the object.
                              //All zeros indicates that it is just an ordinary obstacle.
                              //b0 - candle
                              //b1 - beacon
} add_point_map_packet_t;

//Map complete with unknown shadowing
typedef struct map_complete_unknown_shadow {
   //No body in this packet
} map_complete_unknown_shadow_t;

//Reached destination packet
typedef struct reached_destination_packet {
   coordinate_t destination;
} reached_destination_packet;

void send(uint8_t dst, uint8_t opcode, void* pkt);
void remote_RXDriverCallback(uint8_t src, uint8_t* data, uint8_t len);

#endif /* COMMUNICATIONS_H_ */
