/*
 * adc.h
 *
 *  Created on: Nov 16, 2010
 *      Author: hsrmqp
 *
 * 	This file contains all the user created header material for the ADC on the XMega128A1
 */

#ifndef ADC_H_
#define ADC_H_

//ADC Struct

/*
 * This function servers as an init and set function for the ADC
 * @param ADC The ADC to use either 'A' or 'B'
 * @param isSigned whether or not to use signed conversion true = signed, false = unsigned
 * @param vRef the Reference voltage to use 0 = internal 1.6V
 * @param preScale set the preScaler either 4, 8, 16, 32, 64, 128, 256, 512
 * @param chan the channel to use either 0, 1, 2, 3
 * @param mode to use 0 = single ended
 * @param gain the gain to use either 1, 2, 4, 8, 16, 32, or 64
 * @param pinPos the Posative pin to sample off of 0, 1, 2, 3, 4, 5, 6, 7
 * @param pinNeg the Negative pin to sample off of 0, 1, 2, 3, 4, 5, 6, 7
 * @param oversampling whether or not to oversample (false for 1 measurement true for averaging multiple)
 */
typedef struct adcStruct
{
	char ADC;
	bool isSigned;
	int vRef;
	int preScale;
	int chan;
	int mode;
	int gain;
	int pinPos;
	int pinNeg;
	bool oversampling;

    int8_t offset;


}adcStruct_t;

//Function Prototypes
void adcSet(adcStruct_t* myADC);
int adcTestMain1();


#endif /* ADC_H_ */
