/*
 * encoder_counter.h
 *
 *  Created on: Nov 28, 2010
 *      Author: hsrmqp
 */

#ifndef ENCODER_COUNTER_H_
#define ENCODER_COUNTER_H_

#include "avr/io.h"
#include "spi_driver.h"
#include "mechanical_properties.h"

//#defines
//Registers
#define ENC_CNTR_MDR0 0x01
#define ENC_CNTR_MDR1 0x02
#define ENC_CNTR_DTR  0x03
#define ENC_CNTR_CNTR 0x04
#define ENC_CNTR_OTR  0x05
#define ENC_CNTR_STR  0x06

//Opcodes
#define ENC_CNTR_CLR  0x00
#define ENC_CNTR_RD   0x01
#define ENC_CNTR_WR   0x02
#define ENC_CNTR_LOAD 0x03

//Quadrature counts per rev
#define Q_COUNTS_PER_REV 1440

//The point of this struct is ONLY to set up hardware defined things known at
//compile time such as port/pin numbers or other setup stuff
typedef struct encoder_counter_setup {
   SPI_Master_t* spiDevice; //Pointer to the SPI setup struct
   SPI_init_f SPIinitFunc;  //function pointer to initialize spi
   PORT_t** ssPort;         //Array of SPI SS ports
   uint8_t* ssPinMask;      //Array of SPI SS pinmasks

} encoder_counter_setup_t;


//Function Prototypes
void init_encoder_counter(encoder_counter_setup_t* setup);
int32_t getEncDeltaCount(MOTOR_t motor);
int32_t getEncRunningCount(MOTOR_t motor);

#endif /* ENCODER_COUNTER_H_ */
