/*
 * motors.h
 *
 *  Created on: Dec 2, 2010
 *      Author: hsrmqp
 */

#ifndef MECHANICAL_PROPERTIES_H_
#define MECHANICAL_PROPERTIES_H_

#define WHEEL_RADIUS 3.47091 //wheel radius in cm
#define WHEEL_BASE 14.3129f //wheel base in cm (14.9225f S1, 14.4018f S2, 14.47800f S3, 14.3129f S4)
#define MOTOR0_POLARITY (1)  //Determines which of the 2 motors needs a reverse
#define MOTOR1_POLARITY (-1) //biased voltage to drive "forwards"
#define ENC0_POLARITY (-1)    //Determines which of the 2 encoders will read
#define ENC1_POLARITY (1)   //negative motion as "forwards"
#define MOUSE0_OFFSET 2.2225f  //Offset from the left mouse sensor to the center (in cm)
#define MOUSE1_OFFSET 2.8575f  //Offset from the right mouse sensor to the center (in cm)

//Servo stuff
#define MAIN_SERVO 0
#define SERVO_DEG_PER_S 1

//IR sensors
#define IR_FRONT 0
#define IR_REAR 1
#define IR0_X_OFFSET 0 //Offset of the front ir sensor in the x direction (in cm)
#define IR0_Y_OFFSET 0 //Offset of the front ir sensor in the y direction (in cm)
#define IR1_X_OFFSET 0 //Offset of the rear ir sensor in the x direction (in cm)
#define IR1_Y_OFFSET 0 //Offset of the rear ir sensor in the y direction (in cm)


//Enums
typedef enum Motor_enum
{
	MOTOR0,
	MOTOR1,
	NUM_MOTORS
}MOTOR_t;

#define ROBOT_MAX_MOTOR_OMEGA 4.18f //Maximum angular velocity of the motor in rad/s //this number is based of max rpm of 40
                                                                                      //which was measured experimentally
#define ROBOT_MAX_WHEEL_VELOCITY (ROBOT_MAX_MOTOR_OMEGA*WHEEL_RADIUS)
#define ROBOT_MAX_FORWARD_VELOCITY (ROBOT_MAX_WHEEL_VELOCITY) //Max forwards velocity in cm/s
#define ROBOT_MAX_ANGULAR_VELOCITY (2*ROBOT_MAX_WHEEL_VELOCITY/WHEEL_BASE)//Max angular velocity in rad/s

#define DRIFT_PER_CM .01 //expected amount of drift per cm driven (in cm)

#endif /* MECHANICAL_PROPERTIES_H_ */
