/*
 * mouse_sensor.h
 *
 *  Created on: Dec 2, 2010
 *      Author: hsrmqp
 */

#ifndef MOUSE_SENSOR_H_
#define MOUSE_SENSOR_H_

#include "avr/io.h"
#include "spi_driver.h"

#define NUM_MOUSE_SENSORS 2u
#define MAX_MOTION_BURST 8u
#define MOTION_BURST_REGISTER 0x42u
#define OBSERVATION_REGISTER 0x2eu
#define H_RESOLUTION_REGISTER 0x36u
#define LASER_CTRL0_REGISTER 0x1a
#define LASER_CTRL1_REGISTER 0x1f
#define LSRPWR_CFG0 0x1c
#define LSRPWR_CFG1 0x1d


#define MOUSE_RW_bm (0x80u)
#define HI_RES_2000 (0x1Au)
#define CURRENT_RANGE_4_10 (0xC0u)
#define CURRENT_RANGE_4_10_COMP (0x3Fu)
#define LASER_PWR_100 (0xFFu)
#define LASER_PWR_100_COMP (0x00u)

#define MOUSE_TSRAD 4    //Delay 4 ms between read and write
#define MOUSE_TSWR  14   //Delay between write and either another read or
                         //write command
#define MOUSE_TSRW  1    //Delay 1 us between read and a subsequent read/write
#define MOUSE_FRAME 2000u //Define 1 frame as 2000us

#define MOUSE_CPI 2000

//The point of this struct is ONLY to set up hardware defined things known at
//compile time such as port/pin numbers or other setup stuff
typedef struct mouse_sensor_setup {
	   SPI_Master_t* spiDevice; //The spi device to use
	   SPI_init_f SPIinitFunc;  //function pointer to initialize spi
	   PORT_t** ssPort;         //array of ss ports (array of PORT_t*)
	   uint8_t* ssPinMask;       //array of  ss pinmask (array of uint8_t)

} mouse_sensor_setup_t;

typedef struct mouse_sensor_reading {
   int16_t x; //x displacement in counts
   int16_t y; //y displacement in counts
} mouse_sensor_reading_t;

//Function Prototypes
void init_mouse_sensor(mouse_sensor_setup_t* setup);
void getDisplacement(mouse_sensor_reading_t* reading, uint8_t sensor);

#endif /* MOUSE_SENSOR_H_ */
