/*
 * rangefinder.h
 *
 *  Created on: Dec 6, 2010
 *      Author: hsrmqp
 */

#ifndef RANGEFINDER_H_
#define RANGEFINDER_H_


#include "avr/io.h"
#include "adc_driver.h"
#include "stdbool.h"

//Defines
#define SAMPLE_TIME 53 //Time to read a distance (in ms)


/* The point of this struct is ONLY to set up hardware defined things known at
 * compile time such as port/pin numbers or other setup stuff */
typedef struct rangefinder_setup {

   ADC_t** adc;			// Array of actual ADCs
   ADC_CH_t** channel;	// Array of ADC channels

   PORT_t** enablePort;	// Array of enable ports
   uint8_t* enablePin;	// Array of enable pins (for the above ports)
   int8_t* offset;		// Array of ADC offsets

   /* Below are the calibration values for the linearization function:
    * Distance = (kprime / (rawADC - bprime) - k)
    * They will be unique to each set of sensors.
    */

   int32_t* k; 		// Array of k constants
   int32_t* kprime;	// Array of kprime constants
   int32_t* bprime;	// Array of kprime constants

} range_finder_setup_t;

//Function Prototypes
void init_rangefinder(range_finder_setup_t* setup);
uint8_t rangefinderGetDist(uint8_t irSel);
void rangeFinderEnable(uint8_t irSel, bool enable);

#endif /* RANGEFINDER_H_ */
