/*
 * servo.h
 *
 *  Created on: Dec 2, 2010
 *      Author: hsrmqp
 */

#ifndef SERVO_H_
#define SERVO_H_

#include "avr/io.h"
#include "tc_driver.h"

//Defined Constants
#define SERVO_PWM_MAX_PERIOD 10000

//Servo Constants
#define SERVO_PULSE_CYCLE 20 //ms
#define SERVO_MIN_PULSE .6 //ms
#define SERVO_MAX_PULSE 2.4 //ms
#define SERVO_PULSE_DIFFERENCE (SERVO_MAX_PULSE - SERVO_MIN_PULSE)
#define SERVO_SPEED .1 //speed in degrees/ms

#define SERVO_MIN_CC_LEVEL (SERVO_PWM_MAX_PERIOD * (SERVO_MIN_PULSE / SERVO_PULSE_CYCLE))
#define SERVO_MAX_CC_LEVEL (SERVO_PWM_MAX_PERIOD * (SERVO_MAX_PULSE / SERVO_PULSE_CYCLE))

//The point of this struct is ONLY to set up hardware defined things known at
//compile time such as port/pin numbers or other setup stuff
typedef struct servo_setup {
   TC1_t** servoPWMTimer; //Array of Timer Counter 1 PWM Generators
   TCCOMPARE_t* servoCompare; //Array of CC register enum selectors

   PORT_t** servoEnablePort; //Array of enable ports
   uint8_t* servoEnablePin; //Array of enable pins

   int8_t offset; //offset to account for the fact that the servo may not be
                  //aligned perfectly.  This angle measures amount to add to
                  //the given servo angle so that the servo roates to the
                  //correct position.
} servo_setup_t;

void init_servo(servo_setup_t* setup);
void setServoAngle(uint8_t servoSel, int8_t angle);
int8_t getServoAngle(uint8_t servoSel);
void servoEnable(uint8_t servoSel, bool enable);

#endif /* SERVO_H_ */
