/*
 * ucos_ii_interrupt.h
 *
 *  Created on: Dec 13, 2010
 *      Author: hsrmqp
 *
 *      This file contains useful marcos for servicing an interrupt under uC/OS-II
 */

#ifndef UCOS_II_INTERRUPT_H_
#define UCOS_II_INTERRUPT_H_

#include <ucos_ii.h>

/* Note
 * SREG    = 0x3F
 * SPH     = 0x3E
 * SPL     = 0x3D
 * EIND    = 0x3C
 * RAMPZ   = 0x3B
 * RAMPY   = 0x3A
 * RAMPX   = 0x39
 * RAMPD   = 0x38
 */
#define SREG_REG  "0x3F"
#define SPH_REG   "0x3E"
#define SPL_REG   "0x3D"
#define EIND_REG  "0x3C"
#define RAMPZ_REG "0x3B"
#define RAMPY_REG "0x3A"
#define RAMPX_REG "0x39"
#define RAMPD_REG "0x38"

#define SAVE_SP() asm volatile("IN R26, "SPL_REG"\n\t"  \
                               "IN R27, "SPH_REG"\n\t")

#define RESTORE_SP() asm volatile("LD R28, X+\n\t"          \
                                  "OUT "SPL_REG", R28\n\t"  \
                                  "LD R29, X+\n\t"          \
                                  "OUT "SPH_REG", R29\n\t")


#define PUSH_ALL_REG() asm volatile("PUSH    R0\n\t"         \
                                    "IN      R0, "SREG_REG"\n\t" \
                                    "PUSH    R16\n\t"        \
                                    "MOV     R16, R0\n\t"    \
                                    "SBR     R16, 0x80\n\t"  \
                                    "MOV     R0, R16\n\t"    \
                                    "POP     R16\n\t"        \
                                    "PUSH    R0\n\t"         \
                                    "PUSH    R1\n\t"         \
                                    "CLR     R1\n\t"         \
                                    "PUSH    R2\n\t"         \
                                    "PUSH    R3\n\t"         \
                                    "PUSH    R4\n\t"         \
                                    "PUSH    R5\n\t"         \
                                    "PUSH    R6\n\t"         \
                                    "PUSH    R7\n\t"         \
                                    "PUSH    R8\n\t"         \
                                    "PUSH    R9\n\t"         \
                                    "PUSH    R10\n\t"        \
                                    "PUSH    R11\n\t"        \
                                    "PUSH    R12\n\t"        \
                                    "PUSH    R13\n\t"        \
                                    "PUSH    R14\n\t"        \
                                    "PUSH    R15\n\t"        \
                                    "PUSH    R16\n\t"        \
                                    "PUSH    R17\n\t"        \
                                    "PUSH    R18\n\t"        \
                                    "PUSH    R19\n\t"        \
                                    "PUSH    R20\n\t"        \
                                    "PUSH    R21\n\t"        \
                                    "PUSH    R22\n\t"        \
                                    "PUSH    R23\n\t"        \
                                    "PUSH    R24\n\t"        \
                                    "PUSH    R25\n\t"        \
                                    "PUSH    R26\n\t"        \
                                    "PUSH    R27\n\t"        \
                                    "PUSH    R28\n\t"        \
                                    "PUSH    R29\n\t"        \
                                    "PUSH    R30\n\t"        \
                                    "PUSH    R31\n\t"        \
                                    "IN      R16, "EIND_REG"\n\t"  \
                                    "PUSH    R16\n\t"        \
                                    "IN      R16, "RAMPD_REG"\n\t" \
                                    "PUSH    R16\n\t"        \
                                    "IN      R16, "RAMPX_REG"\n\t" \
                                    "PUSH    R16\n\t"        \
                                    "IN      R16, "RAMPY_REG"\n\t" \
                                    "PUSH    R16\n\t"        \
                                    "IN      R16, "RAMPZ_REG"\n\t" \
                                    "PUSH    R16\n\t")

#define POP_ALL_REG() asm volatile("POP     R16\n\t"         \
                                   "OUT     "RAMPZ_REG", R16\n\t"  \
                                   "POP     R16\n\t"         \
                                   "OUT     "RAMPY_REG", R16\n\t"  \
                                   "POP     R16\n\t"         \
                                   "OUT     "RAMPX_REG", R16\n\t"  \
                                   "POP     R16\n\t"         \
                                   "OUT     "RAMPD_REG", R16\n\t"  \
                                   "POP     R16\n\t"         \
                                   "OUT     "EIND_REG",  R16\n\t"  \
                                   "POP     R31\n\t"         \
                                   "POP     R30\n\t"         \
                                   "POP     R29\n\t"         \
                                   "POP     R28\n\t"         \
                                   "POP     R27\n\t"         \
                                   "POP     R26\n\t"         \
                                   "POP     R25\n\t"         \
                                   "POP     R24\n\t"         \
                                   "POP     R23\n\t"         \
                                   "POP     R22\n\t"         \
                                   "POP     R21\n\t"         \
                                   "POP     R20\n\t"         \
                                   "POP     R19\n\t"         \
                                   "POP     R18\n\t"         \
                                   "POP     R17\n\t"         \
                                   "POP     R16\n\t"         \
                                   "POP     R15\n\t"         \
                                   "POP     R14\n\t"         \
                                   "POP     R13\n\t"         \
                                   "POP     R12\n\t"         \
                                   "POP     R11\n\t"         \
                                   "POP     R10\n\t"         \
                                   "POP     R9\n\t"          \
                                   "POP     R8\n\t"          \
                                   "POP     R7\n\t"          \
                                   "POP     R6\n\t"          \
                                   "POP     R5\n\t"          \
                                   "POP     R4\n\t"          \
                                   "POP     R3\n\t"          \
                                   "POP     R2\n\t"          \
                                   "POP     R1\n\t"          \
                                   "POP     R0\n\t"          \
                                   "OUT     "SREG_REG", R0\n\t"    \
                                   "POP     R0\n\t")

/**
 * This macro is used to notify uCOS that you have entered an interrupt.
 * To use it correctly, it should be called in the first line of an interrupt
 * declared with ISR( ..., ISR_NAKED ) { ...}.  The reason for the ISR_NAKED
 * specifier is to let the compiler know NOT to attempt to save any registers
 * because this macro will do this for it.  After a call to this macro any
 * user code can be executed.  This macro must also be paired with UCOS_EXIT_ISR
 * @param _name The _name parameter isn't all that important however it should
 *              be a name not shared by any other function.
 *              Something like <<Interrupt vector name>> would probably be fine
 */
#define UCOS_ENTER_ISR(_name)  asm volatile("cli\n\t");                 \
                               PUSH_ALL_REG();                          \
                               asm volatile("LDS R16, OSIntNesting\n\t" \
                                            "INC R16\n\t"               \
                                            "STS OSIntNesting,R16\n\t"  \
                                            "CPI R16, 1\n\t"            \
                                            "BRNE "_name"_1\n\t");      \
                                            SAVE_SP();                  \
                               asm volatile("LDS R28, OSTCBCur\n\t"     \
                                            "LDS R29, OSTCBCur+1\n\t"   \
                                            "ST Y+, R26\n\t"            \
                                            "ST Y+, R27\n\t"            \
                                            _name"_1:\n\t"              \
                                            "sei\n\t")

/**
 * This macro (similar to UCOS_ENTER_ISR) is called at the END of an interrupt.
 * Like UCOS_ENTER_ISR the ISR must be declared as ISR_NAKED to notify the
 * compiler not to try and restore the stack registers because this macro will
 * do this.  UCOS_EXIT_ISR takes no parameters and must be called in a pair
 * with UCOS_ENTER_ISR.
 */
#define UCOS_EXIT_ISR() asm volatile("CALL OSIntExit\n\t");  \
                                     POP_ALL_REG();          \
                        asm volatile("RETI\n\t");

/**
 * This enter/exit macro pair is similar to UCOS_ENTER/EXIT_ISR however it will
 * not notify UCOS about the interrupt.  This is useful for interrupts which
 * will never cause a context switch to occur and therefore don't need the
 * overhead imposed by uC/OS.
 */
#define ENTER_ISR() cli();         \
                    PUSH_ALL_REG()
#define EXIT_ISR()  POP_ALL_REG(); \
		              asm volatile("RETI\n\t")



#endif /* UCOS_II_INTERRUPT_H_ */
