/*
 * pid.h
 *
 *  Created on: Dec 17, 2010
 *      Author: hsrmqp
 */

#ifndef PID_H_
#define PID_H_

#include <stdint.h>

#define PID_BUFFER_SIZE 16
#define PID_BUFFER_MASK (PID_BUFFER_SIZE - 1)

#if (PID_BUFFER_SIZE & PID_BUFFER_MASK)
#error PID_BUFFER_SIZE must be a power of 2
#endif

#define INTEGER_PID_PRECISION 10000ul

#if ( INTEGER_PID_PRECISION % 10 )
#error INTEGER_PID_PRECISION must be a power of 10
#endif

//floating point PID buffer
typedef struct pid_f_buffer {
   float errors[PID_BUFFER_SIZE];
   uint8_t bufIndex;
} pid_f_buffer_t;

//integer PID buffer
typedef struct pid_i_buffer {
   uint32_t errors[PID_BUFFER_SIZE];
   uint8_t bufIndex;
} pid_i_buffer_t;

//pid struct for floating point math PID loops
typedef struct pid_f {
   float err;

   float kp;
   float ki;
   float kd;

   pid_f_buffer_t* prevErr; //previous errors

   float lastErr;  //the error from the last iteration of the loop
                   //this is here in case the user doesn't need the
                   //full error buffer but still needs the previous
                   //error to do derivative control

} pid_f_t;

//PID struct for integer math PID loops
typedef struct pid_i {
   int32_t err;

   uint16_t kp;
   uint16_t ki;
   uint16_t kd;

   pid_i_buffer_t* prevErr;  //previous errors

   int32_t lastErr;

} pid_i_t;

typedef struct pid {
   int32_t err;

   uint16_t kp;
   uint16_t ki;
   uint16_t kd;

   pid_i_buffer_t* prevErr;  //previous errors

   int32_t lastErr;

} pid_t;

float runPID_f(pid_f_t* pidStruct, float deltaTime_s);
void setupPID_f(pid_f_t* pidStruct, pid_f_buffer_t* pidBuf, float kp, float ki, float kd);
void setErrorPID_f(pid_f_t* pidStruct, float err);

int32_t runPID_i(pid_i_t* pidStruct, uint64_t deltaTime_us);
void setupPID_i(pid_i_t* pidStruct, pid_i_buffer_t* pidBuf, float kp, float ki, float kd);
void setErrorPID_i(pid_i_t* pidStruct, uint32_t err);

float runPID(pid_t* pidStruct, uint64_t deltaTime_us);
void setupPID(pid_t* pidStruct, pid_i_buffer_t* pidBuf, float kp, float ki, float kd);
void setErrorPID(pid_t* pidStruct, float val);

#endif /* PID_H_ */
