/*
 * xbee.h
 *
 *  Created on: Dec 6, 2010
 *      Author: hsrmqp
 */

#ifndef XBEE_H_
#define XBEE_H_

#include "usart_driver.h"

//#defines
#define XBEE_SEND_SUCCESS 0 //- Everything went ok
#define XBEE_SEND_FAILURE 1 //- The destination did not recieve the packet
#define XBEE_SEND_NO_AD   2 //- The given ad does not exist

#define XBEE_TX_SUCCESS_FLAG    0x01u
#define XBEE_TX_FAIL_FLAG       0x02u
#define XBEE_CHECKSUM_FAIL_FLAG 0x04u
#define XBEE_TX_TIMEOUT         200  //timeout for xbee tx in os ticks (100 ticks == 1 second)

#define XBEE_START_DELIM 0x7E
#define XBEE_MAX_ADDRESSES 5
#define XBEE_64BIT_UNKNOWN (0xFFFFFFFFFFFFFFFFu)
#define XBEE_16BIT_UNKNOWN (0xFFFEu)

#define XBEE_API_AT_COMMAND 0x08
#define XBEE_API_AT_COMMAND_QUEUE 0x09
#define XBEE_API_TX_REQ 0x10
#define XBEE_API_EXPLICIT_COMMAND 0x11
#define XBEE_API_REMOTE_COMMAND 0x17
#define XBEE_API_CREATE_SOURCE_ROUTE 0x21
#define XBEE_API_AT_COMMAND_RESPONSE 0x88
#define XBEE_API_MODEM_STATUS 0x8A
#define XBEE_API_TX_STATUS 0x8B
#define XBEE_API_RX_PACKET 0x90
#define XBEE_API_RX_EXPLICIT 0x91
#define XBEE_API_IO_DATA_SAMPLE_RX_INDICATOR 0x92
#define XBEE_SENSOR_READ_INDICATOR 0x94
#define XBEE_API_NODE_ID_INDICATOR 0x95
#define XBEE_API_REMOTE_COMMAND_RESPONSE 0x97
#define XBEE_API_FW_UPDATE_STATUS 0xA0
#define XBEE_API_ROUTE_RECORD_INDICATOR 0xA1
#define XBEE_API_MANY_TO_ONE_ROUTE_REQUEST_INDICATOR 0xA3

#define XBEE_TX_FRAME_LENGTH(_txFrame) (14 + _txFrame->payloadLength)

#define XBEE_MAX_PAYLOAD 255

//Listener function pointer
typedef void (*xbee_listener_f)(uint8_t, uint8_t*, uint8_t);

//structs
typedef struct xbee_setup {
   USART_data_t* usart_info;
   PORT_t* RTS_Port;
   uint8_t RTS_PinMask;
   PORT_t* CTS_Port;
   uint8_t CTS_PinMask;
   xbee_listener_f callback;
} xbee_setup_t;

//Child table entry
typedef struct xbee_address_lookup {
   uint16_t networkAddr; //16 bit destination network address
   uint64_t addr64bit;   ///64 bit destination address
} xbee_address_lookup_t;

//transmit frame struct
typedef struct xbee_api_tx_req_frame {
   uint8_t frame_id;        //frame ID (must be non-zero)
   uint64_t addr64bit;      //64 bit destination address
   uint16_t addr16bit;      //16 bit network destination address
   uint8_t broadcastRadius; //number of hops allowed for broadcast (0 for max)
   uint8_t options;         //tx options bitmask
   const uint8_t* data;           //array of data bytes to send
   uint8_t payloadLength;   //the number of bytes in the payload.  Note that
                            //this is not the same as the packet length.
} xbee_api_tx_req_frame;

//recieve packet frame struct
typedef struct xbee_api_rx_pac_frame {
   uint64_t addr64bit;    //64 bit destination address
   uint16_t addr16bit;    //16 bit network destination address
   uint8_t options;       //rx options bitmask
   uint8_t* data;         //payload
   uint8_t payloadLength; //the number of bytes in the payload.  This is not
                          //the same as the frame length which includes xbee overhead
} xbee_api_rx_pac_frame_t;

//tx status frame struct
typedef struct xbee_api_tx_status_frame {
   uint8_t frameID;
   uint16_t addr16bit;
   uint8_t retryCount;
   uint8_t deliveryStatus;
   uint8_t discoverStatus;
} xbee_api_tx_status_frame_t;


//Function Prototypes
void init_xbee(xbee_setup_t* setup);
void xbee_receive_task(void* pd);
int8_t xbee_send(uint8_t ad, const uint8_t* data, uint8_t size);


#endif /* XBEE_H_ */
