/*
 * beacondetector.c
 *
 *  Created on: Dec 6, 2010
 *      Author: hsrmqp
 */


#include "beacondetector.h"

//Structs
static const beacon_detector_setup_t* sSetup= 0;

void init_beaconDetector(beacon_detector_setup_t* setup) {
   sSetup= setup;
}

/*
 * Function to get the current reading on the detector  This function is
 * actually responsible for scanning for both candles and beacons.  The
 * return value is a bitmask which indicates which of the two (if any) is
 * actually found.  NOTE beacons should have a frequency of 5kHz and 50% duty cycle
 * @param baconSel Select the beacon/candle detector to use
 * @return A bitmask indicating if a beacon or candle is seen.
 *         bit 0: Candle
 *         bit 1: Beacon
 *         all other bits: unused
 */
uint8_t irledDetectorGet(uint8_t beaconSel){
   uint16_t count= 0;
   //get 1000 readings spaced 1us apart to sample for 1ms
   for(uint16_t i=0; i < 1000; i++) {
      //if the beacon/candle sensor detects something increment the count
      //otherwise do nothing
      if((sSetup->port[beaconSel]->IN & sSetup->pin[beaconSel]) == 0) {
         count++;
      }
      _delay_us(1);
   }

   uint8_t retVal= 0;
   //if the count is nearly 1000 (>=900), the beacon has a 100% duty cycle and is
   //therefore a candle
   if(count >= 900) {
      retVal|= CANDLE_DETECT_bm;
   }

   //if the count is nearly 500 (400 - 600), the beacon has a 50% duty cycle
   //and is therefore a positioning beacon
   else if((count >= 400) && (count <= 600)) {
      retVal|= BEACON_DETECT_bm;
   }

   //otherwise neither a candle or beacon was seen so return 0;
   else {
      retVal= 0;
   }

	return retVal;

}
