/*
 * hbridge.c
 *
 *  Created on: Dec 2, 2010
 *      Author: hsrmqp
 */

#include "hbridge.h"

static const hbridge_setup_t* sSetup= 0;

static float motorPWMs[2];

void init_hbridge(hbridge_setup_t* setup) {
   sSetup= setup;
}

//OUTSET = Set HIGH
//OUTCLR = Set LOW
//OUT = Set All

/**
 * This function sets the duty cycle on an individual motor
 *
 * @param motor an enum representing the motor
 * @param dutyCycle the Duty Cycle to set to, will be a value between -1 and 1
 */
void setMotorPWM(uint8_t motor, float dutyCycle){
	//Set the motor PWM and direction
   int8_t dir= 0; //track direction
	if (dutyCycle == 0){
		sSetup->motorOutputPort[motor]->OUTSET = sSetup->motorDirPins[motor].dir0;
		sSetup->motorOutputPort[motor]->OUTSET = sSetup->motorDirPins[motor].dir1;
	}

	else if (dutyCycle > 0){
		sSetup->motorOutputPort[motor]->OUTSET = sSetup->motorDirPins[motor].dir0;
		sSetup->motorOutputPort[motor]->OUTCLR = sSetup->motorDirPins[motor].dir1;
		dir= 1;
	}

	else if (dutyCycle < 0){
		sSetup->motorOutputPort[motor]->OUTCLR = sSetup->motorDirPins[motor].dir0;
		sSetup->motorOutputPort[motor]->OUTSET = sSetup->motorDirPins[motor].dir1;
		dir= -1;
		dutyCycle *= -1; //Invert the Duty Cycle since it has to be positive
	}
	if(dutyCycle >= 1) dutyCycle= 1;

	switch (sSetup->motorCompare[motor]){
		case TC_CCA_REG: TC_SetCompareA( sSetup->motorPWMTimer[motor], MOTOR_PWM_PERIOD * dutyCycle); break;
		case TC_CCB_REG: TC_SetCompareB( sSetup->motorPWMTimer[motor], MOTOR_PWM_PERIOD * dutyCycle); break;
		case TC_CCC_REG: TC_SetCompareC( sSetup->motorPWMTimer[motor], MOTOR_PWM_PERIOD * dutyCycle); break;
		case TC_CCD_REG: TC_SetCompareD( sSetup->motorPWMTimer[motor], MOTOR_PWM_PERIOD * dutyCycle); break;
		default : break;
	}

	motorPWMs[motor]= dutyCycle*dir;

}

/**
 * This function can be used to set all of the motor PWM's "simultaneously"
 *
 * @param motorPWMArray[] an array of doubles holding all the motor PWM values
 * @param numMotors since C can't determine the size of the array, you have to pass it how many motors to set
 */
void setMotorsPWM(float* motorPWMArray, uint8_t numMotors){

	for(int i=0; i < numMotors; i++){
		setMotorPWM(i, motorPWMArray[i]);
	}
}

/**
 * This function returns the current setpoint of a motor PWM
 *
 * @param motor the motor who's PWM we want to get
 * @return double the value of the PWM between o and 1
 */
float getMotorPWM(uint8_t motor){
   return motorPWMs[motor];
}
