/*
 * rangefinder.c
 *
 *  Created on: Dec 6, 2010
 *      Author: hsrmqp
 */


#include "rangefinder.h"

//Structs
static const range_finder_setup_t* sSetup= 0;

void init_rangefinder(range_finder_setup_t* setup) {
   sSetup= setup;
}

/*
 * Function to get the current range on the IR.  The IR sensor must be enabled
 * before calling this function.
 *
 * Distance = (kprime / (rawADC - bprime) - k)
 */
uint8_t rangefinderGetDist(uint8_t irSel){

	//delay until distance is measured
	_delay_ms(SAMPLE_TIME);


	uint16_t result, adcRaw;
	//start the conversion
	ADC_Ch_Conversion_Start(sSetup->channel[irSel]);
	while(!ADC_Ch_Conversion_Complete(sSetup->channel[irSel]));
	adcRaw = ADC_ResultCh_GetWord_Unsigned(sSetup->channel[irSel], sSetup->offset[irSel]);
	//if adcRaw + bprime <= 0 there is effectively "infinite" distance to an object
	//we can't really represent infinity so instead use the max value of an
	//8 bit int (the max number we can return).
	if((adcRaw + sSetup->bprime[irSel]) <= 0) {
	   return result= 0xFF;
	}
	//otherwise run the linearization which will return a positive number as
	//long as adcRaw + bprime > 0
	else {
	   result = (sSetup->kprime[irSel] / (adcRaw + sSetup->bprime[irSel])) - sSetup->k[irSel];
	   //clip result to the max value we can return
	   if(result > 0xFF) {
	      result= 0xFF;
	   }
	}


	return result;

}

/**
 * This function enables or disables a range finder
 *
 * @param irSel the ir to enable or disable
 * @param enable whether or not to enable the IR
 */
void rangeFinderEnable(uint8_t irSel, bool enable){
	if (enable){
		sSetup->enablePort[irSel]->OUTCLR = sSetup->enablePin[irSel];
	}
	else{
		sSetup->enablePort[irSel]->OUTSET = sSetup->enablePin[irSel];
	}
}
