/*
 * servo.c
 *
 *  Created on: Dec 2, 2010
 *      Author: hsrmqp
 */

#include "servo.h"
#include "util/delay.h"
#include "math.h"
#include "stdio.h"


static const servo_setup_t* sSetup= 0;
static int8_t currentAngle= 0; //The currentSetpoint angle of the servo.

void init_servo(servo_setup_t* setup) {
   sSetup= setup;
}

/**
 * This function sets the servo to a given angle.  The servo must be enabled
 * prior to calling this function.
 *
 * @param the angle to set to, can be between -90 and 90
 */
void setServoAngle(uint8_t servoSel, int8_t angle){
	float anglePercent;
	float pulseLength;
	uint8_t deltaAngle= abs((int16_t)angle - (int16_t)getServoAngle(servoSel));  //number of degrees we're changing
	uint16_t ccLevel;

   //angle needs to be offset slightly
   angle+=sSetup->offset;

	//Determine what percentage of the whole we want to go to
	anglePercent = (float)(angle + 90) / 180;

	//Determine the pulse length from .6 - 2.4
	pulseLength = SERVO_MIN_PULSE + (anglePercent * SERVO_PULSE_DIFFERENCE);

	//Determine the corresponding CC Trigger Level
	ccLevel = (SERVO_PWM_MAX_PERIOD * (pulseLength / SERVO_PULSE_CYCLE));

	switch(sSetup->servoCompare[servoSel]){
	case TC_CCA_REG: TC_SetCompareA( sSetup->servoPWMTimer[servoSel], ccLevel ); break;
	case TC_CCB_REG: TC_SetCompareB( sSetup->servoPWMTimer[servoSel], ccLevel ); break;
	default : break;
	}

	//wait for servo to arrive at destination
	_delay_ms(deltaAngle/SERVO_SPEED);

	//update current angle
	currentAngle= angle-sSetup->offset;

}

/**
 * This function returns the current servo angle
 *
 * Using the current counter compare register, it computes the current
 * servo angle between -90 and 90.  -90 Indicates rotated all the way to the right.
 * @return The angle in degrees
 */
int8_t getServoAngle(uint8_t servoSel){

	return currentAngle;
}

/**
 * This function enables or disables a servo
 *
 * @param servoSel the servo to enable or disable
 * @param enable whether or not to enable the servo
 */
void servoEnable(uint8_t servoSel, bool enable){
	if (enable){
		sSetup->servoEnablePort[servoSel]->OUTCLR = sSetup->servoEnablePin[servoSel];
	}
	else{
		sSetup->servoEnablePort[servoSel]->OUTSET = sSetup->servoEnablePin[servoSel];
	}
}

