/*
 * debug_module.c
 *
 *  Created on: Dec 15, 2010
 *      Author: hsrmqp
 */

#include "utils/debug_module.h"

#ifdef USART_DEBUG_ENABLE

static const debug_module_setup_t* sSetup= 0;

//Static protptypes
static int debug_usart_putchar(char c, FILE* stream);
static int debug_usart_putchar_blocking(char c, FILE* stream);
static int debug_usart_getchar(FILE* stream);
static int debug_usart_getchar_blocking(FILE* stream);

//setup file streams for stdout/stdin
#ifndef DEBUG_BLOCKING_PRINTS
static FILE debug_stdout= FDEV_SETUP_STREAM(debug_usart_putchar, 0, _FDEV_SETUP_WRITE);
static FILE debug_stdin= FDEV_SETUP_STREAM(0, debug_usart_getchar, _FDEV_SETUP_READ);
#endif
#ifdef DEBUG_BLOCKING_PRINTS
static FILE debug_stdout= FDEV_SETUP_STREAM(debug_usart_putchar_blocking, 0, _FDEV_SETUP_WRITE);
static FILE debug_stdin= FDEV_SETUP_STREAM(0, debug_usart_getchar_blocking, _FDEV_SETUP_READ);
#endif



void init_debug_module(debug_module_setup_t* setup) {
   sSetup= setup;

   stdout= &debug_stdout;
   stdin= &debug_stdin;
}

/**
 * Function to add a single character to the usart buffer.  This is pretty
 * much just a wrapper for the USART_TXBuffer_PutByte function that allows
 * printf to be able to write to a usart.  This also adds a \r whenever a
 * \n is seen for the sake of convenience.
 */
static int debug_usart_putchar(char c, FILE* stream) {
   //automatically add a \r if a \n is seen
   if(c == '\n') {
      debug_usart_putchar('\r', stream);
   }
   //sit in a loop trying to add the character to the buffer
   while(!USART_TXBuffer_PutByte(sSetup->usart, c));

   return 0;
}

static int debug_usart_putchar_blocking(char c, FILE* stream) {
   //automatically add a \r if a \n is seen
   if(c == '\n') {
      debug_usart_putchar_blocking('\r', stream);
   }
   while(!(sSetup->usart->usart->STATUS & USART_DREIF_bm));
   sSetup->usart->usart->DATA= c;

   return 0;
}

/**
 * Get a byte from the usart buffer.
 * @return The byte received off of the buffer if data exists in it
 *         _FDEV_EOF otherwise
 */
static int debug_usart_getchar(FILE* stream) {
   if(USART_RXBufferData_Available(sSetup->usart)) {
      return USART_RXBuffer_GetByte(sSetup->usart);
   }
   else {
      return _FDEV_EOF;
   }
}

static int debug_usart_getchar_blocking(FILE* stream) {
   while(!(sSetup->usart->usart->STATUS & USART_RXCIF_bm));

   return sSetup->usart->usart->DATA;
}

#endif //#ifdef USART_DEBUG_ENABLE

