/*
 * emb_malloc.c
 *
 *  Created on: Dec 16, 2010
 *      Author: hsrmqp
 */

#include "utils/emb_malloc.h"
#include "includes.h"


/**
 * Allocate an array of contiguous bytes of a given size.  The values stored
 * in these bytes are random and should be initialized by the user.
 * @param size The number of bytes to allocate
 * @return A pointer to an allocated portion of memory of size size.
 */
void* emb_malloc(uint16_t size) {
   uint8_t* retVal; //return pointer
   //make sure we allocate enough space to store the memory partition we
   //malloc'd from
   size++;
   uint8_t err;
   //find the smallest available memory partition size to use
   //check small heap
   if(size < SMALL_MEM_BLOCK_SIZE) {
      //get memory block
      retVal= OSMemGet(small_mem, &err);
      if(err == OS_ERR_NONE) {
         //store which heap was used in 1st byte
         *retVal= 0;
         //return offset by one to account for the byte tracking which
         //partition it came from
         return retVal+1;
      }
   }
   //check medium heap
   if(size < MEDIUM_MEM_BLOCK_SIZE) {
      retVal= OSMemGet(medium_mem, &err);
      if(err == OS_ERR_NONE) {
         //store which heap was used in 1st byte
         *retVal= 1;
         //return offset by one to account for the byte tracking which
         //partition it came from
         return retVal+1;
      }
   }
   //check large heap
   if(size < LARGE_MEM_BLOCK_SIZE) {
      retVal= OSMemGet(large_mem, &err);
      if(err == OS_ERR_NONE) {
         //store which heap was used in 1st byte
         *retVal= 2;
         //return offset by one to account for the byte tracking which
         //partition it came from
         return retVal+1;
      }
   }

   return 0;

}

/**
 * Free a series of bytes allocated with emb_malloc
 * @param ptr Pointer to a series of bytes allocated with emb_malloc
 */
void emb_free(void* ptr) {
   //cast ptr to point to a byte
   uint8_t* bytePtr= (uint8_t*)ptr;
   //decrement by one since we store the partition one byte back
   //and this represents the place we actually malloc'd from
   bytePtr--;

   //see which memory partition we need to return to
   uint8_t partNum= *(bytePtr);

   switch(partNum) {
   case 0:
      OSMemPut(small_mem, bytePtr);
      break;
   case 1:
      OSMemPut(medium_mem, bytePtr);
      break;
   case 2:
      OSMemPut(large_mem, bytePtr);
      break;
   default:
      return;
   }

}
